@@ -1,7 +1,7 @@
 /*
  * wee-backtrace.c - backtrace after a segfault
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,7 +56,7 @@ weechat_backtrace_printf (const char *message, ...)
     weechat_va_format (message);
     if (vbuffer)
     {
-        string_iconv_fprintf (stderr, "%s\n", vbuffer);
+        string_fprintf (stderr, "%s\n", vbuffer);
         log_printf ("%s", vbuffer);
         free (vbuffer);
     }
@@ -111,12 +111,13 @@ weechat_backtrace_addr2line (int number, void *address, const char *symbol)
                 pos[0] = '\0';
             if (strchr (ptr_line, ':'))
             {
-                weechat_backtrace_printf ("%03d  %s%s%s%s",
-                                          number,
-                                          ptr_line,
-                                          (function_name[0]) ? " [function " : "",
-                                          function_name,
-                                          (function_name[0]) ? "]" : "");
+                weechat_backtrace_printf (
+                    "%03d  %s%s%s%s",
+                    number,
+                    ptr_line,
+                    (function_name[0]) ? " [function " : "",
+                    function_name,
+                    (function_name[0]) ? "]" : "");
                 function_name[0] = '\0';
             }
             else
@@ -140,7 +141,7 @@ weechat_backtrace_addr2line (int number, void *address, const char *symbol)
     (void) symbol;
 
     /* no backtrace possible, we display nothing */
-#endif
+#endif /* HAVE_BACKTRACE */
 }
 
 /*
@@ -154,7 +155,7 @@ weechat_backtrace ()
     void *trace[BACKTRACE_MAX];
     int trace_size, i;
     char **symbols;
-#endif
+#endif /* HAVE_BACKTRACE */
 
     weechat_backtrace_printf ("======= WeeChat backtrace =======");
     weechat_backtrace_printf ("(written by WeeChat %s, compiled on %s %s)",
@@ -173,7 +174,7 @@ weechat_backtrace ()
 #else
     weechat_backtrace_printf ("  No backtrace info (no debug info available "
                               "or no backtrace possible on your system).");
-#endif
+#endif /* HAVE_BACKTRACE */
 
     weechat_backtrace_printf ("======= End of  backtrace =======");
 }
