@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2010-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,15 +14,44 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_HASHTABLE_H
 #define WEECHAT_HASHTABLE_H
 
 struct t_hashtable;
+struct t_infolist;
 struct t_infolist_item;
 
+/*
+ * Macros to set various values as string value in the hashtable;
+ * variable hashtable must be defined and str_value must be a static
+ * area where the value as string will be put (the size must be large enough
+ * for any value stored with these macros).
+ */
+#define HASHTABLE_SET_STR(__name, __string)                              \
+    hashtable_set (hashtable, __name, __string);
+#define HASHTABLE_SET_STR_NOT_NULL(__name, __string)                     \
+    hashtable_set (hashtable, __name, (__string) ? __string : "");
+#define HASHTABLE_SET_INT(__name, __int)                                 \
+    snprintf (str_value, sizeof (str_value), "%d", __int);               \
+    hashtable_set (hashtable, __name, str_value);
+#define HASHTABLE_SET_TIME(__name, __time)                               \
+    snprintf (str_value, sizeof (str_value), "%lld", (long long)__time); \
+    hashtable_set (hashtable, __name, str_value);
+#define HASHTABLE_SET_POINTER(__name, __pointer)                         \
+    if (__pointer)                                                       \
+    {                                                                    \
+        snprintf (str_value, sizeof (str_value),                         \
+                  "0x%lx", (unsigned long)__pointer);                    \
+        hashtable_set (hashtable, __name, str_value);                    \
+    }                                                                    \
+    else                                                                 \
+    {                                                                    \
+        hashtable_set (hashtable, __name, "");                           \
+    }
+
 typedef unsigned long long (t_hashtable_hash_key)(struct t_hashtable *hashtable,
                                                   const void *key);
 typedef int (t_hashtable_keycmp)(struct t_hashtable *hashtable,
@@ -88,6 +117,10 @@ struct t_hashtable_item
     int value_size;                     /* size of value (in bytes)         */
     struct t_hashtable_item *prev_item; /* link to previous item            */
     struct t_hashtable_item *next_item; /* link to next item                */
+    /* previous/next item by order of creation in the hashtable */
+    struct t_hashtable_item *prev_created_item;
+    struct t_hashtable_item *next_created_item;
+
 };
 
 struct t_hashtable
@@ -96,6 +129,8 @@ struct t_hashtable
     struct t_hashtable_item **htable;  /* table to map hashes with linked   */
                                        /* lists                             */
     int items_count;                   /* number of items in hashtable      */
+    struct t_hashtable_item *oldest_item; /* oldest item in hashtable       */
+    struct t_hashtable_item *newest_item; /* newest item in hashtable       */
 
     /* type for keys and values */
     enum t_hashtable_type type_keys;   /* type for keys: int/str/pointer    */
@@ -149,6 +184,9 @@ extern void hashtable_set_pointer (struct t_hashtable *hashtable,
 extern int hashtable_add_to_infolist (struct t_hashtable *hashtable,
                                       struct t_infolist_item *infolist_item,
                                       const char *prefix);
+extern int hashtable_add_from_infolist (struct t_hashtable *hashtable,
+                                        struct t_infolist *infolist,
+                                        const char *prefix);
 extern void hashtable_remove (struct t_hashtable *hashtable, const void *key);
 extern void hashtable_remove_all (struct t_hashtable *hashtable);
 extern void hashtable_free (struct t_hashtable *hashtable);
