@@ -1,7 +1,7 @@
 /*
  * wee-hdata.c - direct access to WeeChat data using hashtables
  *
- * Copyright (C) 2011-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2011-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -496,7 +496,7 @@ hdata_check_pointer_map_cb (void *data, struct t_hashtable *hashtable,
     if (!ptr_list || !(ptr_list->flags & WEECHAT_HDATA_LIST_CHECK_POINTERS))
         return;
 
-    *found = (void *)((unsigned long int)hdata_check_pointer_in_list (
+    *found = (void *)((unsigned long)hdata_check_pointer_in_list (
                           ptr_hdata,
                           *((void **)(ptr_list->pointer)),
                           pointer));
@@ -557,7 +557,7 @@ hdata_move (struct t_hdata *hdata, void *pointer, int count)
         return NULL;
 
     ptr_var = (count < 0) ? hdata->var_prev : hdata->var_next;
-    abs_count = abs(count);
+    abs_count = abs (count);
 
     for (i = 0; i < abs_count; i++)
     {
@@ -876,6 +876,105 @@ hdata_hashtable (struct t_hdata *hdata, void *pointer, const char *name)
     return NULL;
 }
 
+/*
+ * Compares a hdata variable of two objects.
+ *
+ * If case_sensitive == 1, the comparison of strings is case sensitive.
+ *
+ * Returns:
+ *   -1: variable1 < variable2
+ *    0: variable1 == variable2
+ *    1: variable1 > variable2
+ */
+
+int
+hdata_compare (struct t_hdata *hdata, void *pointer1, void *pointer2,
+               const char *name, int case_sensitive)
+{
+    int rc, int_value1, int_value2;
+    long long_value1, long_value2;
+    char char_value1, char_value2;
+    const char *ptr_name, *str_value1, *str_value2;
+    void *ptr_value1, *ptr_value2;
+    time_t time_value1, time_value2;
+
+    if (!pointer1 && pointer2)
+        return -1;
+    if (pointer1 && !pointer2)
+        return 1;
+    if (!pointer1 && !pointer2)
+        return 0;
+
+    rc = 0;
+
+    hdata_get_index_and_name (name, NULL, &ptr_name);
+    switch (hdata_get_var_type (hdata, ptr_name))
+    {
+        case WEECHAT_HDATA_CHAR:
+            char_value1 = hdata_char (hdata, pointer1, name);
+            char_value2 = hdata_char (hdata, pointer2, name);
+            rc = (char_value1 < char_value2) ?
+                -1 : ((char_value1 > char_value2) ? 1 : 0);
+            break;
+        case WEECHAT_HDATA_INTEGER:
+            int_value1 = hdata_integer (hdata, pointer1, name);
+            int_value2 = hdata_integer (hdata, pointer2, name);
+            rc = (int_value1 < int_value2) ?
+                -1 : ((int_value1 > int_value2) ? 1 : 0);
+            break;
+        case WEECHAT_HDATA_LONG:
+            long_value1 = hdata_long (hdata, pointer1, name);
+            long_value2 = hdata_long (hdata, pointer2, name);
+            rc = (long_value1 < long_value2) ?
+                -1 : ((long_value1 > long_value2) ? 1 : 0);
+            break;
+        case WEECHAT_HDATA_STRING:
+        case WEECHAT_HDATA_SHARED_STRING:
+            str_value1 = hdata_string (hdata, pointer1, name);
+            str_value2 = hdata_string (hdata, pointer2, name);
+            if (!str_value1 && !str_value2)
+                rc = 0;
+            else if (str_value1 && !str_value2)
+                rc = 1;
+            else if (!str_value1 && str_value2)
+                rc = -1;
+            else
+            {
+                if (case_sensitive)
+                    rc = strcmp (str_value1, str_value2);
+                else
+                    rc = string_strcasecmp (str_value1, str_value2);
+                if (rc < 0)
+                    rc = -1;
+                else if (rc > 0)
+                    rc = 1;
+            }
+            break;
+        case WEECHAT_HDATA_POINTER:
+            ptr_value1 = hdata_pointer (hdata, pointer1, name);
+            ptr_value2 = hdata_pointer (hdata, pointer2, name);
+            rc = (ptr_value1 < ptr_value2) ?
+                -1 : ((ptr_value1 > ptr_value2) ? 1 : 0);
+            break;
+        case WEECHAT_HDATA_TIME:
+            time_value1 = hdata_time (hdata, pointer1, name);
+            time_value2 = hdata_time (hdata, pointer2, name);
+            rc = (time_value1 < time_value2) ?
+                -1 : ((time_value1 > time_value2) ? 1 : 0);
+            break;
+        case WEECHAT_HDATA_HASHTABLE:
+            /* no comparison for hashtables */
+            rc = 0;
+            break;
+        case WEECHAT_HDATA_OTHER:
+            /* no comparison for other types */
+            rc = 0;
+            break;
+    }
+
+    return rc;
+}
+
 /*
  * Sets value for a variable in hdata.
  *
@@ -895,7 +994,7 @@ hdata_set (struct t_hdata *hdata, void *pointer, const char *name,
     struct t_hdata_var *var;
     char **ptr_string, *error;
     long number;
-    long unsigned int ptr;
+    unsigned long ptr;
     int rc;
 
     if (!hdata->update_pending)
@@ -1062,6 +1161,9 @@ hdata_get_string (struct t_hdata *hdata, const char *property)
 void
 hdata_free (struct t_hdata *hdata)
 {
+    if (!hdata)
+        return;
+
     if (hdata->hash_var)
         hashtable_free (hdata->hash_var);
     if (hdata->var_prev)
