@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_INPUT_H
@@ -23,10 +23,16 @@
 struct t_gui_buffer;
 struct t_weechat_plugin;
 
+extern char **input_commands_allowed;
+
 extern int input_exec_command (struct t_gui_buffer *buffer,
                                int any_plugin,
                                struct t_weechat_plugin *plugin,
-                               const char *string);
-extern int input_data (struct t_gui_buffer *buffer, const char *data);
+                               const char *string,
+                               const char *commands_allowed);
+extern int input_data (struct t_gui_buffer *buffer, const char *data,
+                       const char *commands_allowed);
+extern int input_data_delayed (struct t_gui_buffer *buffer, const char *data,
+                               const char *commands_allowed, long delay);
 
 #endif /* WEECHAT_INPUT_H */
