@@ -56,16 +56,23 @@
 
 #include "weechat.h"
 #include "wee-network.h"
+#include "wee-config.h"
 #include "wee-eval.h"
+#include "wee-hashtable.h"
 #include "wee-hook.h"
 #include "wee-config.h"
 #include "wee-proxy.h"
 #include "wee-string.h"
+#include "../gui/gui-chat.h"
 #include "../plugins/plugin.h"
 
 
 int network_init_gnutls_ok = 0;
 
+int network_num_certs_system = 0; /* number of system certs loaded          */
+int network_num_certs_user = 0;   /* number of user certs loaded            */
+int network_num_certs = 0;        /* number of certs loaded (system + user) */
+
 gnutls_certificate_credentials_t gnutls_xcred; /* GnuTLS client credentials */
 
 
@@ -85,29 +92,177 @@ network_init_gcrypt ()
 }
 
 /*
- * Sets trust file with option "gnutls_ca_file".
+ * Loads system's default trusted certificate authorities.
+ *
+ * Returns the number of certificates loaded.
+ */
+
+int
+network_load_system_ca_file (int force_display)
+{
+    int rc;
+
+    if (weechat_no_gnutls)
+        return 0;
+
+    if (!CONFIG_BOOLEAN(config_network_gnutls_ca_system))
+        return 0;
+
+    rc = gnutls_certificate_set_x509_system_trust (gnutls_xcred);
+    if (rc < 0)
+    {
+        gui_chat_printf (
+            NULL,
+            _("%sWarning: failed to load system certificate authorities"),
+            gui_chat_prefix[GUI_CHAT_PREFIX_ERROR]);
+        return 0;
+    }
+
+    if (force_display || (weechat_debug_core >= 1))
+    {
+        gui_chat_printf (
+            NULL,
+            NG_("%d certificate loaded (system)",
+                "%d certificates loaded (system)",
+                rc),
+            rc);
+    }
+
+    network_num_certs_system = rc;
+
+    return rc;
+}
+
+/*
+ * Loads user's trusted certificate authorities.
+ */
+
+int
+network_load_user_ca_files (int force_display)
+{
+    int i, rc, num_loaded, num_paths;
+    char **paths, *ca_path;
+    struct t_hashtable *options;
+
+    if (weechat_no_gnutls)
+        return 0;
+
+    num_loaded = 0;
+    paths = NULL;
+    options = NULL;
+
+    paths = string_split (CONFIG_STRING(config_network_gnutls_ca_user), ":",
+                          NULL, 0, 0, &num_paths);
+    if (!paths)
+        goto end;
+
+    options = hashtable_new (
+        32,
+        WEECHAT_HASHTABLE_STRING,
+        WEECHAT_HASHTABLE_STRING,
+        NULL, NULL);
+    if (!options)
+        goto end;
+    hashtable_set (options, "directory", "config");
+
+    for (i = 0; i < num_paths; i++)
+    {
+        ca_path = string_eval_path_home (paths[i], NULL, NULL, options);
+        if (ca_path && ca_path[0])
+        {
+            if (access (ca_path, R_OK) == 0)
+            {
+                rc = gnutls_certificate_set_x509_trust_file (
+                    gnutls_xcred, ca_path, GNUTLS_X509_FMT_PEM);
+                if (rc < 0)
+                {
+                    gui_chat_printf (
+                        NULL,
+                        _("%sWarning: failed to load certificate authorities "
+                          "from file %s"),
+                        gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
+                        ca_path);
+                }
+                else
+                {
+                    num_loaded += rc;
+                    if (force_display || (weechat_debug_core >= 1))
+                    {
+                        gui_chat_printf (
+                            NULL,
+                            NG_("%d certificate loaded (file: %s)",
+                                "%d certificates loaded (file: %s)",
+                                rc),
+                            rc, ca_path);
+                    }
+                }
+            }
+            else
+            {
+                gui_chat_printf (
+                    NULL,
+                    _("%sWarning: failed to load certificate authorities "
+                      "from file %s (file not found)"),
+                    gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
+                    ca_path);
+            }
+        }
+        if (ca_path)
+            free (ca_path);
+    }
+
+end:
+    if (paths)
+        string_free_split (paths);
+    if (options)
+        hashtable_free (options);
+
+    network_num_certs_user = num_loaded;
+
+    return num_loaded;
+}
+
+/*
+ * Loads system's default and user's trusted certificate authorities.
  */
 
 void
-network_set_gnutls_ca_file ()
+network_load_ca_files (int force_display)
 {
-    char *ca_path, *ca_path2;
-
     if (weechat_no_gnutls)
         return;
 
-    ca_path = string_expand_home (CONFIG_STRING(config_network_gnutls_ca_file));
-    if (ca_path)
+    network_num_certs_system = 0;
+    network_num_certs_user = 0;
+    network_num_certs = 0;
+
+    network_num_certs += network_load_system_ca_file (force_display);
+    network_num_certs += network_load_user_ca_files (force_display);
+}
+
+/*
+ * Reloads system's default and user's trusted certificate authorities.
+ */
+
+void
+network_reload_ca_files (int force_display)
+{
+    if (weechat_no_gnutls)
+        return;
+
+    gnutls_certificate_free_credentials (gnutls_xcred);
+    if (force_display || (weechat_debug_core >= 1))
     {
-        ca_path2 = string_replace (ca_path, "%h", weechat_home);
-        if (ca_path2)
-        {
-            gnutls_certificate_set_x509_trust_file (gnutls_xcred, ca_path2,
-                                                    GNUTLS_X509_FMT_PEM);
-            free (ca_path2);
-        }
-        free (ca_path);
+        gui_chat_printf (NULL,
+                         NG_("%d certificate purged",
+                             "%d certificates purged",
+                             network_num_certs),
+                         network_num_certs);
     }
+
+    gnutls_certificate_allocate_credentials (&gnutls_xcred);
+
+    network_load_ca_files (force_display);
 }
 
 /*
@@ -121,8 +276,7 @@ network_init_gnutls ()
     {
         gnutls_global_init ();
         gnutls_certificate_allocate_credentials (&gnutls_xcred);
-
-        network_set_gnutls_ca_file ();
+        network_load_ca_files (0);
 #if LIBGNUTLS_VERSION_NUMBER >= 0x02090a /* 2.9.10 */
         gnutls_certificate_set_verify_function (gnutls_xcred,
                                                 &hook_connect_gnutls_verify_certificates);
@@ -157,6 +311,35 @@ network_end ()
     }
 }
 
+/*
+ * Checks if a string contains a valid IP address (IPv4 or IPv6).
+ *
+ * Returns:
+ *   1: string is a valid IPv4 or IPv6
+ *   0: string is not a valid IP address
+ */
+
+int
+network_is_ip_address (const char *address)
+{
+    struct sockaddr_in server_addr;
+    struct sockaddr_in6 server_addr6;
+
+    if (!address || !address[0])
+        return 0;
+
+    /* valid IPv4? */
+    if (inet_pton (AF_INET, address, &server_addr.sin_addr))
+        return 1;
+
+    /* valid IPv6? */
+    if (inet_pton (AF_INET6, address, &server_addr6.sin6_addr))
+        return 1;
+
+    /* not a valid IP address */
+    return 0;
+}
+
 /*
  * Sends data on a socket with retry.
  *
@@ -1648,19 +1831,23 @@ network_connect_with_fork (struct t_hook *hook_connect)
             unhook (hook_connect);
             return;
         }
-        rc = gnutls_server_name_set (*HOOK_CONNECT(hook_connect, gnutls_sess),
-                                     GNUTLS_NAME_DNS,
-                                     HOOK_CONNECT(hook_connect, address),
-                                     strlen (HOOK_CONNECT(hook_connect, address)));
-        if (rc != GNUTLS_E_SUCCESS)
+        if (!network_is_ip_address (HOOK_CONNECT(hook_connect, address)))
         {
-            (void) (HOOK_CONNECT(hook_connect, callback))
-                (hook_connect->callback_pointer,
-                 hook_connect->callback_data,
-                 WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
-                 0, -1, _("set server name indication (SNI) failed"), NULL);
-            unhook (hook_connect);
-            return;
+            /* set the server name (only if it's NOT an IPv4/IPv6) */
+            rc = gnutls_server_name_set (*HOOK_CONNECT(hook_connect, gnutls_sess),
+                                         GNUTLS_NAME_DNS,
+                                         HOOK_CONNECT(hook_connect, address),
+                                         strlen (HOOK_CONNECT(hook_connect, address)));
+            if (rc != GNUTLS_E_SUCCESS)
+            {
+                (void) (HOOK_CONNECT(hook_connect, callback))
+                    (hook_connect->callback_pointer,
+                     hook_connect->callback_data,
+                     WEECHAT_HOOK_CONNECT_GNUTLS_INIT_ERROR,
+                     0, -1, _("set server name indication (SNI) failed"), NULL);
+                unhook (hook_connect);
+                return;
+            }
         }
         rc = gnutls_priority_set_direct (*HOOK_CONNECT(hook_connect, gnutls_sess),
                                          HOOK_CONNECT(hook_connect, gnutls_priorities),
