@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -40,6 +40,7 @@ extern char *string_strndup (const char *string, int length);
 extern char *string_cut (const char *string, int length, int count_suffix,
                          int screen, const char *cut_suffix);
 extern char *string_reverse (const char *string);
+extern char *string_reverse_screen (const char *string);
 extern char *string_repeat (const char *string, int count);
 extern void string_tolower (char *string);
 extern void string_toupper (char *string);
@@ -114,6 +115,9 @@ extern int string_base32_encode (const char *from, int length, char *to);
 extern int string_base32_decode (const char *from, char *to);
 extern int string_base64_encode (const char *from, int length, char *to);
 extern int string_base64_decode (const char *from, char *to);
+extern int string_base_encode (int base, const char *from, int length,
+                               char *to);
+extern int string_base_decode (int base, const char *from, char *to);
 extern char *string_hex_dump (const char *data, int data_size,
                               int bytes_per_line,
                               const char *prefix, const char *suffix);
@@ -130,7 +134,7 @@ extern const char *string_shared_get (const char *string);
 extern void string_shared_free (const char *string);
 extern char **string_dyn_alloc (int size_alloc);
 extern int string_dyn_copy (char **string, const char *new_string);
-extern int string_dyn_concat (char **string, const char *add);
+extern int string_dyn_concat (char **string, const char *add, int bytes);
 extern char *string_dyn_free (char **string, int free_string);
 extern void string_end ();
 
