@@ -47,6 +47,7 @@
 #include <getopt.h>
 #include <sys/stat.h>
 #include <sys/time.h>
+#include <sys/types.h>
 #include <time.h>
 #include <signal.h>
 
@@ -59,6 +60,7 @@
 #include "wee-completion.h"
 #include "wee-config.h"
 #include "wee-debug.h"
+#include "wee-dir.h"
 #include "wee-eval.h"
 #include "wee-hdata.h"
 #include "wee-hook.h"
@@ -68,6 +70,7 @@
 #include "wee-proxy.h"
 #include "wee-secure.h"
 #include "wee-secure-config.h"
+#include "wee-signal.h"
 #include "wee-string.h"
 #include "wee-upgrade.h"
 #include "wee-utf8.h"
@@ -99,14 +102,18 @@ time_t weechat_first_start_time = 0;   /* start time (used by /uptime cmd)  */
 int weechat_upgrade_count = 0;         /* number of /upgrade done           */
 struct timeval weechat_current_start_timeval; /* start time used to display */
                                        /* duration of /upgrade              */
-volatile sig_atomic_t weechat_quit = 0;   /* = 1 if quit request from user  */
+int weechat_quit = 0;                  /* = 1 if quit request from user     */
 volatile sig_atomic_t weechat_quit_signal = 0; /* signal received,          */
                                        /* WeeChat must quit                 */
 volatile sig_atomic_t weechat_reload_signal = 0; /* signal received,        */
                                        /* WeeChat must reload configuration */
-char *weechat_home = NULL;             /* home dir. (default: ~/.weechat)   */
+char *weechat_home_force = NULL;       /* forced home (with -d/--dir)       */
 int weechat_home_temp = 0;             /* 1 if using a temporary home       */
 int weechat_home_delete_on_exit = 0;   /* 1 if home is deleted on exit      */
+char *weechat_config_dir = NULL;       /* config directory                  */
+char *weechat_data_dir = NULL;         /* data directory                    */
+char *weechat_cache_dir = NULL;        /* cache directory                   */
+char *weechat_runtime_dir = NULL;      /* runtime directory                 */
 int weechat_locale_ok = 0;             /* is locale OK?                     */
 char *weechat_local_charset = NULL;    /* example: ISO-8859-1, UTF-8        */
 int weechat_server_cmd_line = 0;       /* at least 1 server on cmd line     */
@@ -162,8 +169,9 @@ weechat_display_usage ()
         _("  -a, --no-connect         disable auto-connect to servers at "
           "startup\n"
           "  -c, --colors             display default colors in terminal\n"
-          "  -d, --dir <path>         set WeeChat home directory "
-          "(default: ~/.weechat)\n"
+          "  -d, --dir <path>         force a single WeeChat home directory\n"
+          "                           or 4 different directories separated "
+          "by colons (in this order: config, data, cache, runtime)\n"
           "                           (environment variable WEECHAT_HOME is "
           "read if this option is not given)\n"
           "  -t, --temp-dir           create a temporary WeeChat home"
@@ -176,7 +184,7 @@ weechat_display_usage ()
           "                           (see /help weechat.plugin.autoload)\n"
           "  -r, --run-command <cmd>  run command(s) after startup;\n"
           "                           many commands can be separated by "
-          "semicolons,\n"
+          "semicolons and are evaluated,\n"
           "                           this option can be given multiple "
           "times\n"
           "  -s, --no-script          don't load any script at startup\n"
@@ -253,7 +261,7 @@ weechat_parse_args (int argc, char *argv[])
 
     weechat_argv0 = (argv[0]) ? strdup (argv[0]) : NULL;
     weechat_upgrading = 0;
-    weechat_home = NULL;
+    weechat_home_force = NULL;
     weechat_home_temp = 0;
     weechat_home_delete_on_exit = 0;
     weechat_server_cmd_line = 0;
@@ -278,16 +286,16 @@ weechat_parse_args (int argc, char *argv[])
                 break;
             case 'd': /* -d / --dir */
                 weechat_home_temp = 0;
-                if (weechat_home)
-                    free (weechat_home);
-                weechat_home = strdup (optarg);
+                if (weechat_home_force)
+                    free (weechat_home_force);
+                weechat_home_force = strdup (optarg);
                 break;
             case 't': /* -t / --temp-dir */
                 weechat_home_temp = 1;
-                if (weechat_home)
+                if (weechat_home_force)
                 {
-                    free (weechat_home);
-                    weechat_home = NULL;
+                    free (weechat_home_force);
+                    weechat_home_force = NULL;
                 }
                 break;
             case 'h': /* -h / --help */
@@ -372,167 +380,6 @@ weechat_parse_args (int argc, char *argv[])
     }
 }
 
-/*
- * Expands and assigns given path to "weechat_home".
- */
-
-void
-weechat_set_home_path (char *home_path)
-{
-    char *ptr_home;
-    int dir_length;
-
-    if (home_path[0] == '~')
-    {
-        /* replace leading '~' by $HOME */
-        ptr_home = getenv ("HOME");
-        if (!ptr_home)
-        {
-            string_fprintf (stderr, _("Error: unable to get HOME directory\n"));
-            weechat_shutdown (EXIT_FAILURE, 0);
-            /* make C static analyzer happy (never executed) */
-            return;
-        }
-        dir_length = strlen (ptr_home) + strlen (home_path + 1) + 1;
-        weechat_home = malloc (dir_length);
-        if (weechat_home)
-        {
-            snprintf (weechat_home, dir_length,
-                      "%s%s", ptr_home, home_path + 1);
-        }
-    }
-    else
-    {
-        weechat_home = strdup (home_path);
-    }
-
-    if (!weechat_home)
-    {
-        string_fprintf (stderr,
-                        _("Error: not enough memory for home directory\n"));
-        weechat_shutdown (EXIT_FAILURE, 0);
-        /* make C static analyzer happy (never executed) */
-        return;
-    }
-}
-
-/*
- * Creates WeeChat home directory (by default ~/.weechat).
- *
- * Any error in this function is fatal: WeeChat can not run without a home
- * directory.
- */
-
-void
-weechat_create_home_dir ()
-{
-    char *temp_dir, *temp_home_template, *ptr_weechat_home;
-    char *config_weechat_home;
-    int length, add_separator;
-    struct stat statinfo;
-
-    /* temporary WeeChat home */
-    if (weechat_home_temp)
-    {
-        temp_dir = util_get_temp_dir ();
-        if (!temp_dir || !temp_dir[0])
-        {
-            string_fprintf (stderr,
-                            _("Error: not enough memory for home "
-                              "directory\n"));
-            weechat_shutdown (EXIT_FAILURE, 0);
-            /* make C static analyzer happy (never executed) */
-            return;
-        }
-        length = strlen (temp_dir) + 32 + 1;
-        temp_home_template = malloc (length);
-        if (!temp_home_template)
-        {
-            free (temp_dir);
-            string_fprintf (stderr,
-                            _("Error: not enough memory for home "
-                              "directory\n"));
-            weechat_shutdown (EXIT_FAILURE, 0);
-            /* make C static analyzer happy (never executed) */
-            return;
-        }
-        add_separator = (temp_dir[strlen (temp_dir) - 1] != DIR_SEPARATOR_CHAR);
-        snprintf (temp_home_template, length,
-                  "%s%sweechat_temp_XXXXXX",
-                  temp_dir,
-                  add_separator ? DIR_SEPARATOR : "");
-        free (temp_dir);
-        ptr_weechat_home = mkdtemp (temp_home_template);
-        if (!ptr_weechat_home)
-        {
-            string_fprintf (stderr,
-                            _("Error: unable to create a temporary "
-                              "home directory (using template: \"%s\")\n"),
-                            temp_home_template);
-            free (temp_home_template);
-            weechat_shutdown (EXIT_FAILURE, 0);
-            /* make C static analyzer happy (never executed) */
-            return;
-        }
-        weechat_home = strdup (ptr_weechat_home);
-        free (temp_home_template);
-        weechat_home_delete_on_exit = 1;
-        return;
-    }
-
-    /*
-     * weechat_home is not set yet: look for environment variable
-     * "WEECHAT_HOME"
-     */
-    if (!weechat_home)
-    {
-        ptr_weechat_home = getenv ("WEECHAT_HOME");
-        if (ptr_weechat_home && ptr_weechat_home[0])
-            weechat_set_home_path (ptr_weechat_home);
-    }
-
-    /* weechat_home is still not set: try to use compile time default */
-    if (!weechat_home)
-    {
-        config_weechat_home = WEECHAT_HOME;
-        if (!config_weechat_home[0])
-        {
-            string_fprintf (stderr,
-                            _("Error: WEECHAT_HOME is undefined, check build "
-                              "options\n"));
-            weechat_shutdown (EXIT_FAILURE, 0);
-            /* make C static analyzer happy (never executed) */
-            return;
-        }
-        weechat_set_home_path (config_weechat_home);
-    }
-
-    /* if home already exists, it has to be a directory */
-    if (stat (weechat_home, &statinfo) == 0)
-    {
-        if (!S_ISDIR (statinfo.st_mode))
-        {
-            string_fprintf (stderr,
-                            _("Error: home (%s) is not a directory\n"),
-                            weechat_home);
-            weechat_shutdown (EXIT_FAILURE, 0);
-            /* make C static analyzer happy (never executed) */
-            return;
-        }
-    }
-
-    /* create home directory; error is fatal */
-    if (!util_mkdir (weechat_home, 0755))
-    {
-        string_fprintf (stderr,
-                        _("Error: cannot create directory \"%s\"\n"),
-                        weechat_home);
-        weechat_shutdown (EXIT_FAILURE, 0);
-        /* make C static analyzer happy (never executed) */
-        return;
-    }
-}
-
 /*
  * Displays WeeChat startup message.
  */
@@ -688,36 +535,6 @@ weechat_locale_check ()
     }
 }
 
-/*
- * Callback for system signal SIGHUP: reloads configuration.
- */
-
-void
-weechat_sighup ()
-{
-    weechat_reload_signal = SIGHUP;
-}
-
-/*
- * Callback for system signal SIGQUIT: quits WeeChat.
- */
-
-void
-weechat_sigquit ()
-{
-    weechat_quit_signal = SIGQUIT;
-}
-
-/*
- * Callback for system signal SIGTERM: quits WeeChat.
- */
-
-void
-weechat_sigterm ()
-{
-    weechat_quit_signal = SIGTERM;
-}
-
 /*
  * Shutdowns WeeChat.
  */
@@ -734,13 +551,21 @@ weechat_shutdown (int return_code, int crash)
     if (!crash && weechat_home_delete_on_exit)
     {
         /* remove temporary home (only if not crashing) */
-        util_rmtree (weechat_home);
+        dir_remove_home_dirs ();
     }
 
     if (weechat_argv0)
         free (weechat_argv0);
-    if (weechat_home)
-        free (weechat_home);
+    if (weechat_home_force)
+        free (weechat_home_force);
+    if (weechat_config_dir)
+        free (weechat_config_dir);
+    if (weechat_data_dir)
+        free (weechat_data_dir);
+    if (weechat_cache_dir)
+        free (weechat_cache_dir);
+    if (weechat_runtime_dir)
+        free (weechat_runtime_dir);
     if (weechat_local_charset)
         free (weechat_local_charset);
     if (weechat_force_plugin_autoload)
@@ -786,14 +611,12 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     weechat_first_start_time = time (NULL); /* initialize start time        */
     gettimeofday (&weechat_current_start_timeval, NULL);
 
-    /* catch signals */
-    util_catch_signal (SIGINT, SIG_IGN);           /* signal ignored        */
-    util_catch_signal (SIGPIPE, SIG_IGN);          /* signal ignored        */
-    util_catch_signal (SIGSEGV, &debug_sigsegv);   /* crash dump            */
-    util_catch_signal (SIGHUP, &weechat_sighup);   /* exit WeeChat          */
-    util_catch_signal (SIGQUIT, &weechat_sigquit); /* exit WeeChat          */
-    util_catch_signal (SIGTERM, &weechat_sigterm); /* exit WeeChat          */
+    /* set the seed for the pseudo-random integer generator */
+    srand ((weechat_current_start_timeval.tv_sec
+            * weechat_current_start_timeval.tv_usec)
+           ^ getpid ());
 
+    signal_init ();                     /* initialize signals               */
     hdata_init ();                      /* initialize hdata                 */
     hook_init ();                       /* initialize hooks                 */
     debug_init ();                      /* hook signals for debug           */
@@ -810,7 +633,7 @@ weechat_init (int argc, char *argv[], void (*gui_init_cb)())
     if (!config_weechat_init ())        /* init WeeChat options (weechat.*) */
         weechat_shutdown (EXIT_FAILURE, 0);
     weechat_parse_args (argc, argv);    /* parse command line args          */
-    weechat_create_home_dir ();         /* create WeeChat home directory    */
+    dir_create_home_dirs ();            /* create WeeChat home directories  */
     log_init ();                        /* init log file                    */
     plugin_api_init ();                 /* create some hooks (info,hdata,..)*/
     secure_config_read ();              /* read secured data options        */
