@@ -1,7 +1,7 @@
 /*
  * gui-curses-key.c - keyboard functions for Curses GUI
  *
- * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -83,6 +83,7 @@ gui_key_default_bindings (int context)
     {
         BIND(/* <enter>       */ "ctrl-M",             "/input return");
         BIND(/* <enter>       */ "ctrl-J",             "/input return");
+        BIND(/* m-<enter>     */ "meta-ctrl-M",        "/input insert \\n");
         BIND(/* <tab>         */ "ctrl-I",             "/input complete_next");
         BIND(/* s-<tab>       */ "meta2-Z",            "/input complete_previous");
         BIND(/* ^R            */ "ctrl-R",             "/input search_text_here");
@@ -136,9 +137,12 @@ gui_key_default_bindings (int context)
         BIND(/* m-j,m-l       */ "meta-jmeta-l",       "/buffer +");
         BIND(/* m-j,m-r       */ "meta-jmeta-r",       "/server raw");
         BIND(/* m-j,m-s       */ "meta-jmeta-s",       "/server jump");
-        BIND(/* m-h           */ "meta-h",             "/input hotlist_clear");
+        BIND(/* m-h,m-c       */ "meta-hmeta-c",       "/input hotlist_clear");
+        BIND(/* m-h,m-m       */ "meta-hmeta-m",       "/input hotlist_remove_buffer");
+        BIND(/* m-h,m-r       */ "meta-hmeta-r",       "/input hotlist_restore_buffer");
+        BIND(/* m-h,m-R       */ "meta-hmeta-R",       "/input hotlist_restore_all");
         BIND(/* m-k           */ "meta-k",             "/input grab_key_command");
-        BIND(/* m-s           */ "meta-s",             "/mute aspell toggle");
+        BIND(/* m-s           */ "meta-s",             "/mute spell toggle");
         BIND(/* m-u           */ "meta-u",             "/window scroll_unread");
         BIND(/* ^S^U          */ "ctrl-Sctrl-U",       "/input set_unread");
         BIND(/* ^Cb           */ "ctrl-Cb",            "/input insert \\x02");
@@ -176,10 +180,15 @@ gui_key_default_bindings (int context)
         BIND(/* m-<end>       */ "meta2-1;3F",         "/window scroll_bottom");
         BIND(/* m-n           */ "meta-n",             "/window scroll_next_highlight");
         BIND(/* m-p           */ "meta-p",             "/window scroll_previous_highlight");
+        BIND(/* m-N           */ "meta-N",             "/bar toggle nicklist");
         BIND(/* <f9>          */ "meta2-20~",          "/bar scroll title * -30%");
         BIND(/* <f10>         */ "meta2-21~",          "/bar scroll title * +30%");
         BIND(/* <f11>         */ "meta2-23~",          "/bar scroll nicklist * -100%");
         BIND(/* <f12>         */ "meta2-24~",          "/bar scroll nicklist * +100%");
+        BIND(/* c-<f11>       */ "meta2-23^",          "/bar scroll nicklist * -100%");
+        BIND(/* c-<f11>       */ "meta2-23;5~",        "/bar scroll nicklist * -100%");
+        BIND(/* c-<f12>       */ "meta2-24^",          "/bar scroll nicklist * +100%");
+        BIND(/* c-<f12>       */ "meta2-24;5~",        "/bar scroll nicklist * +100%");
         BIND(/* m-<f11>       */ "meta2-23;3~",        "/bar scroll nicklist * b");
         BIND(/* m-<f11>       */ "meta-meta2-23~",     "/bar scroll nicklist * b");
         BIND(/* m-<f12>       */ "meta2-24;3~",        "/bar scroll nicklist * e");
@@ -542,29 +551,20 @@ gui_key_read_cb (const void *pointer, void *data, int fd)
 
     for (i = 0; i < ret; i++)
     {
-        /*
-         * add all chars, but ignore a newline ('\r' or '\n') after
-         * another one)
-         */
-        if ((i == 0)
-            || ((buffer[i] != '\r') && (buffer[i] != '\n'))
-            || ((buffer[i - 1] != '\r') && (buffer[i - 1] != '\n')))
+        if (gui_key_paste_pending && (buffer[i] == 25))
         {
-            if (gui_key_paste_pending && (buffer[i] == 25))
-            {
-                /* ctrl-Y: accept paste */
-                accept_paste = 1;
-            }
-            else if (gui_key_paste_pending && (buffer[i] == 14))
-            {
-                /* ctrl-N: cancel paste */
-                cancel_paste = 1;
-            }
-            else
-            {
-                gui_key_buffer_add (buffer[i]);
-                text_added_to_buffer = 1;
-            }
+            /* ctrl-Y: accept paste */
+            accept_paste = 1;
+        }
+        else if (gui_key_paste_pending && (buffer[i] == 14))
+        {
+            /* ctrl-N: cancel paste */
+            cancel_paste = 1;
+        }
+        else
+        {
+            gui_key_buffer_add (buffer[i]);
+            text_added_to_buffer = 1;
         }
     }
 
