@@ -1,7 +1,7 @@
 /*
  * gui-curses-window.c - window display functions for Curses GUI
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -181,7 +181,7 @@ gui_window_clear (WINDOW *window, int fg, int bg)
     if (!gui_init_ok)
         return;
 
-    attrs = gui_color_get_extended_attrs (fg);
+    attrs = gui_color_get_gui_attrs (fg);
 
     if ((fg > 0) && (fg & GUI_COLOR_EXTENDED_FLAG))
         fg &= GUI_COLOR_EXTENDED_MASK;
@@ -420,7 +420,7 @@ gui_window_set_custom_color_fg (WINDOW *window, int fg)
         {
             if (!(fg & GUI_COLOR_EXTENDED_KEEPATTR_FLAG))
                 gui_window_remove_color_style (window, A_ALL_ATTR);
-            attributes = gui_color_get_extended_attrs (fg) |
+            attributes = gui_color_get_gui_attrs (fg) |
                 gui_weechat_colors[fg & GUI_COLOR_EXTENDED_MASK].attributes;
             gui_window_set_color_style (window, attributes);
             fg = gui_weechat_colors[fg & GUI_COLOR_EXTENDED_MASK].foreground;
@@ -505,7 +505,7 @@ gui_window_set_custom_color_fg_bg (WINDOW *window, int fg, int bg,
         {
             if (reset_attributes && !(fg & GUI_COLOR_EXTENDED_KEEPATTR_FLAG))
                 gui_window_remove_color_style (window, A_ALL_ATTR);
-            attributes = gui_color_get_extended_attrs (fg) |
+            attributes = gui_color_get_gui_attrs (fg) |
                 gui_weechat_colors[fg & GUI_COLOR_EXTENDED_MASK].attributes;
             gui_window_set_color_style (window, attributes);
             fg = gui_weechat_colors[fg & GUI_COLOR_EXTENDED_MASK].foreground;
@@ -1115,7 +1115,9 @@ gui_window_vline (WINDOW *window, int x, int y, int height, const char *string)
 void
 gui_window_draw_separators (struct t_gui_window *window)
 {
-    int separator_horizontal, separator_vertical, x, width;
+    int separator_horizontal, separator_vertical;
+    int horiz_overlap, x, width, height;
+    struct t_gui_window_tree *ptr_tree, *ptr_child;
 
     /* remove separators */
     if (GUI_WINDOW_OBJECTS(window)->win_separator_horiz)
@@ -1136,17 +1138,42 @@ gui_window_draw_separators (struct t_gui_window *window)
     separator_vertical = (CONFIG_BOOLEAN(config_look_window_separator_vertical)
                           && (window->win_x > gui_bar_root_get_size (NULL, GUI_BAR_POSITION_LEFT)));
 
+    /* check if horizontal separator must overlap in lower left corner */
+    horiz_overlap = 0;
+    ptr_child = window->ptr_tree;
+    ptr_tree = ptr_child->parent_node;
+    while (ptr_tree)
+    {
+        /* is right/top child */
+        if (ptr_tree->child2 == ptr_child)
+        {
+            /*
+             * horizontal overlap is needed when window is right in vertical
+             * split; it is not needed when window is top in horizontal split
+             */
+            horiz_overlap = !ptr_tree->split_horizontal;
+            break;
+        }
+
+        ptr_child = ptr_child->parent_node;
+        ptr_tree = ptr_tree->parent_node;
+    }
+
     /* create/draw horizontal separator */
     if (separator_horizontal)
     {
-        x = (separator_vertical) ? window->win_x - 1 : window->win_x;
-        width = (separator_vertical) ? window->win_width + 1 : window->win_width;
-        GUI_WINDOW_OBJECTS(window)->win_separator_horiz = newwin (1,
-                                                                  width,
-                                                                  window->win_y + window->win_height,
-                                                                  x);
-        gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_separator_horiz,
-                                      GUI_COLOR_SEPARATOR);
+        x = (separator_vertical && horiz_overlap) ?
+            window->win_x - 1 : window->win_x;
+        width = (separator_vertical && horiz_overlap) ?
+            window->win_width + 1 : window->win_width;
+        GUI_WINDOW_OBJECTS(window)->win_separator_horiz = newwin (
+            1,
+            width,
+            window->win_y + window->win_height,
+            x);
+        gui_window_set_weechat_color (
+            GUI_WINDOW_OBJECTS(window)->win_separator_horiz,
+            GUI_COLOR_SEPARATOR);
         gui_window_hline (GUI_WINDOW_OBJECTS(window)->win_separator_horiz,
                           0, 0, width,
                           CONFIG_STRING(config_look_separator_horizontal));
@@ -1156,14 +1183,18 @@ gui_window_draw_separators (struct t_gui_window *window)
     /* create/draw vertical separator */
     if (separator_vertical)
     {
-        GUI_WINDOW_OBJECTS(window)->win_separator_vertic = newwin (window->win_height,
-                                                                   1,
-                                                                   window->win_y,
-                                                                   window->win_x - 1);
-        gui_window_set_weechat_color (GUI_WINDOW_OBJECTS(window)->win_separator_vertic,
-                                      GUI_COLOR_SEPARATOR);
+        height = (separator_horizontal && !horiz_overlap) ?
+            window->win_height + 1 : window->win_height;
+        GUI_WINDOW_OBJECTS(window)->win_separator_vertic = newwin (
+            height,
+            1,
+            window->win_y,
+            window->win_x - 1);
+        gui_window_set_weechat_color (
+            GUI_WINDOW_OBJECTS(window)->win_separator_vertic,
+            GUI_COLOR_SEPARATOR);
         gui_window_vline (GUI_WINDOW_OBJECTS(window)->win_separator_vertic,
-                          0, 0, window->win_height,
+                          0, 0, height,
                           CONFIG_STRING(config_look_separator_vertical));
         wnoutrefresh (GUI_WINDOW_OBJECTS(window)->win_separator_vertic);
     }
@@ -1204,13 +1235,17 @@ gui_window_switch_to_buffer (struct t_gui_window *window,
             gui_buffer_visited_add (window->buffer);
             gui_buffer_visited_add (buffer);
         }
-        if (set_last_read)
+        if (set_last_read
+            && CONFIG_BOOLEAN(config_look_read_marker_update_on_buffer_switch))
         {
             if (window->buffer->num_displayed == 0)
             {
                 window->buffer->lines->last_read_line = window->buffer->lines->last_line;
                 window->buffer->lines->first_line_not_read = 0;
             }
+        }
+        if (set_last_read)
+        {
             /*
              * if there is no line displayed after last read line,
              * then remove the read marker
@@ -1229,8 +1264,12 @@ gui_window_switch_to_buffer (struct t_gui_window *window,
     gui_buffer_set_active_buffer (buffer);
     gui_buffer_compute_num_displayed ();
 
-    if (!weechat_upgrading && (old_buffer != buffer))
+    if (!weechat_upgrading
+        && (old_buffer != buffer)
+        && CONFIG_BOOLEAN(config_look_hotlist_update_on_buffer_switch))
+    {
         gui_hotlist_remove_buffer (buffer, 0);
+    }
 
     /* remove unused bars and add missing bars in window */
     gui_bar_window_remove_unused_bars (window);
@@ -1617,6 +1656,7 @@ gui_window_scroll_beyond_end (struct t_gui_window *window)
     {
         window->scroll->start_line = window->buffer->lines->last_line;
         window->scroll->start_line_pos = -1;
+        window->scroll->first_line_displayed = 0;
         gui_buffer_ask_chat_refresh (window->buffer, 2);
     }
 }
@@ -1666,7 +1706,7 @@ gui_window_auto_resize (struct t_gui_window_tree *tree,
                     else
                     {
                         tree->window->win_width_pct = (tree == parent->child1) ?
-                            parent->split_pct : 100 - parent->split_pct;
+                            100 - parent->split_pct : parent->split_pct;
                         tree->window->win_height_pct = 100;
                     }
                 }
@@ -1689,8 +1729,8 @@ gui_window_auto_resize (struct t_gui_window_tree *tree,
             else
             {
                 separator = (CONFIG_BOOLEAN(config_look_window_separator_vertical)) ? 1 : 0;
-                size1 = (width * tree->split_pct) / 100;
-                size2 = width - size1 - separator;
+                size2 = (width * tree->split_pct) / 100;
+                size1 = width - size2 - separator;
                 if (gui_window_auto_resize (tree->child1, x, y,
                                             size1, height, simulate) < 0)
                     return -1;
@@ -1817,6 +1857,9 @@ gui_window_split_horizontal (struct t_gui_window *window, int percentage)
             gui_window_ask_refresh (1);
 
             gui_window_switch (new_window);
+
+            /* create & draw separators */
+            gui_window_draw_separators (gui_current_window);
         }
     }
 
@@ -1842,8 +1885,8 @@ gui_window_split_vertical (struct t_gui_window *window, int percentage)
 
     separator = (CONFIG_BOOLEAN(config_look_window_separator_vertical)) ? 1 : 0;
 
-    width1 = (window->win_width * percentage) / 100;
-    width2 = window->win_width - width1 - separator;
+    width2 = (window->win_width * percentage) / 100;
+    width1 = window->win_width - width2 - separator;
 
     if ((width1 >= 1) && (width2 >= 1)
         && (percentage > 0) && (percentage < 100))
@@ -1892,15 +1935,10 @@ gui_window_resize (struct t_gui_window_tree *tree, int percentage)
     if (parent)
     {
         old_split_pct = parent->split_pct;
-        if (((parent->split_horizontal) && (tree == parent->child2))
-            || ((!(parent->split_horizontal)) && (tree == parent->child1)))
-        {
+        if (tree == parent->child2)
             parent->split_pct = percentage;
-        }
         else
-        {
             parent->split_pct = 100 - percentage;
-        }
 
         add_bottom = gui_bar_root_get_size (NULL, GUI_BAR_POSITION_BOTTOM);
         add_top = gui_bar_root_get_size (NULL, GUI_BAR_POSITION_TOP);
@@ -1934,15 +1972,10 @@ gui_window_resize_delta (struct t_gui_window_tree *tree, int delta_percentage)
     if (parent)
     {
         old_split_pct = parent->split_pct;
-        if (((parent->split_horizontal) && (tree == parent->child2))
-            || ((!(parent->split_horizontal)) && (tree == parent->child1)))
-        {
+        if (tree == parent->child2)
             parent->split_pct += delta_percentage;
-        }
         else
-        {
             parent->split_pct -= delta_percentage;
-        }
         if (parent->split_pct < 1)
             parent->split_pct = 1;
         else if (parent->split_pct > 99)
@@ -2495,6 +2528,7 @@ gui_window_set_title (const char *title)
     /* the following terminals support the xterm escape codes */
     else if ((strncmp (envterm, "xterm", 5) == 0)
              || (strncmp (envterm, "rxvt", 4) == 0)
+             || (strncmp (envterm, "alacritty", 9) == 0)
              || (strcmp (envterm, "Eterm") == 0)
              || (strcmp (envterm, "aixterm") == 0)
              || (strcmp (envterm, "iris-ansi") == 0)
