@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_GUI_CURSES_H
@@ -22,6 +22,10 @@
 
 #include <time.h>
 
+#ifdef WEECHAT_HEADLESS
+#include "ncurses-fake.h"
+#else
+#define NCURSES_WIDECHAR 1
 #ifdef HAVE_NCURSESW_CURSES_H
 #include <ncursesw/ncurses.h>
 #elif HAVE_NCURSES_H
@@ -29,6 +33,7 @@
 #else
 #include <curses.h>
 #endif /* HAVE_NCURSESW_CURSES_H */
+#endif /* WEECHAT_HEADLESS */
 
 struct t_gui_buffer;
 struct t_gui_line;
@@ -86,7 +91,8 @@ extern void gui_main_init ();
 extern void gui_main_loop ();
 
 /* color functions */
-extern int gui_color_get_extended_attrs (int color);
+extern int gui_color_get_gui_attrs (int color);
+extern int gui_color_get_extended_flags (int attrs);
 extern int gui_color_get_pair (int fg, int bg);
 extern int gui_color_weechat_get_pair (int weechat_color);
 extern void gui_color_alloc ();
