@@ -1,7 +1,7 @@
 /*
  * main.c - entry point for headless mode (no GUI)
  *
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -88,7 +88,7 @@ daemonize ()
 int
 main (int argc, char *argv[])
 {
-    int i, daemon;
+    int i;
 
     weechat_init_gettext ();
 
@@ -101,20 +101,27 @@ main (int argc, char *argv[])
     weechat_headless = 1;
 
     /*
-     * If "--daemon" is received in command line arguments,
-     * daemonize the process.
+     * Parse extra options for headless mode:
+     * - "--daemon": daemonize the process
+     * - "--stdout": log messages to stdout (instead of log file)
      */
-    daemon = 0;
+    weechat_daemon = 0;
     for (i = 1; i < argc; i++)
     {
         if (strcmp (argv[i], "--daemon") == 0)
         {
-            daemon = 1;
-            break;
+            weechat_daemon = 1;
+        }
+        else if (strcmp (argv[i], "--stdout") == 0)
+        {
+            weechat_log_stdout = 1;
         }
     }
-    if (daemon)
+    if (weechat_daemon)
+    {
+        weechat_log_stdout = 0;
         daemonize ();
+    }
 
     /* init, main loop and end */
     weechat_init (argc, argv, &gui_main_init);
