@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -186,8 +186,10 @@ extern void gui_color_attr_build_string (int color, char *str_attr);
 extern const char *gui_color_get_custom (const char *color_name);
 extern int gui_color_convert_term_to_rgb (int color);
 extern int gui_color_convert_rgb_to_term (int rgb, int limit);
+extern int gui_color_code_size (const char *string);
 extern char *gui_color_decode (const char *string, const char *replacement);
 extern char *gui_color_decode_ansi (const char *string, int keep_colors);
+extern char *gui_color_encode_ansi (const char *string);
 extern char *gui_color_emphasize (const char *string, const char *search,
                                   int case_sensitive, regex_t *regex);
 extern void gui_color_free (struct t_gui_color *color);
@@ -202,11 +204,14 @@ extern void gui_color_end ();
 /* color functions (GUI dependent) */
 
 extern int gui_color_search (const char *color_name);
+extern const char *gui_color_search_index (int index);
+extern int gui_color_get_extended_flags (int attrs);
 extern int gui_color_assign (int *color, char const *color_name);
 extern int gui_color_assign_by_diff (int *color, const char *color_name,
                                      int diff);
 extern int gui_color_get_weechat_colors_number ();
 extern int gui_color_get_term_colors ();
+extern int gui_color_get_term_color_pairs ();
 extern const char *gui_color_get_name (int num_color);
 extern void gui_color_free_vars ();
 extern void gui_color_init_weechat ();
