@@ -1,7 +1,7 @@
 /*
  * gui-filter.c - filter functions (used by all GUI)
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -38,6 +38,7 @@
 #include "../plugins/plugin.h"
 #include "gui-filter.h"
 #include "gui-buffer.h"
+#include "gui-chat.h"
 #include "gui-line.h"
 #include "gui-window.h"
 
@@ -74,9 +75,9 @@ gui_filter_check_line (struct t_gui_line_data *line_data)
         if (ptr_filter->enabled)
         {
             /* check buffer */
-            if (gui_buffer_match_list_split (line_data->buffer,
-                                             ptr_filter->num_buffers,
-                                             ptr_filter->buffers))
+            if (string_match_list (line_data->buffer->full_name,
+                                   (const char **)ptr_filter->buffers,
+                                   0))
             {
                 if ((strcmp (ptr_filter->tags, "*") == 0)
                     || (gui_line_match_tags (line_data,
@@ -185,17 +186,25 @@ gui_filter_buffer (struct t_gui_buffer *buffer,
 
 /*
  * Filters all buffers, using message filters.
+ *
+ * If filter is NULL, filters all buffers.
+ * If filter is not NULL, filters only buffers matched by this filter.
  */
 
 void
-gui_filter_all_buffers ()
+gui_filter_all_buffers (struct t_gui_filter *filter)
 {
     struct t_gui_buffer *ptr_buffer;
 
     for (ptr_buffer = gui_buffers; ptr_buffer;
          ptr_buffer = ptr_buffer->next_buffer)
     {
-        gui_filter_buffer (ptr_buffer, NULL);
+        if (!filter
+            || string_match_list (ptr_buffer->full_name,
+                                  (const char **)filter->buffers, 0))
+        {
+            gui_filter_buffer (ptr_buffer, NULL);
+        }
     }
 }
 
@@ -209,7 +218,7 @@ gui_filter_global_enable ()
     if (!gui_filters_enabled)
     {
         gui_filters_enabled = 1;
-        gui_filter_all_buffers ();
+        gui_filter_all_buffers (NULL);
         (void) hook_signal_send ("filters_enabled",
                                  WEECHAT_HOOK_SIGNAL_STRING, NULL);
     }
@@ -225,7 +234,7 @@ gui_filter_global_disable ()
     if (gui_filters_enabled)
     {
         gui_filters_enabled = 0;
-        gui_filter_all_buffers ();
+        gui_filter_all_buffers (NULL);
         (void) hook_signal_send ("filters_disabled",
                                  WEECHAT_HOOK_SIGNAL_STRING, NULL);
     }
@@ -253,6 +262,21 @@ gui_filter_search_by_name (const char *name)
     return NULL;
 }
 
+/*
+ * Displays an error when a new filter is created.
+ */
+
+void
+gui_filter_new_error (const char *name, const char *error)
+{
+    gui_chat_printf_date_tags (
+        NULL, 0, GUI_FILTER_TAG_NO_FILTER,
+        _("%sUnable to add filter \"%s\": %s"),
+        gui_chat_prefix[GUI_CHAT_PREFIX_ERROR],
+        (name) ? name : "",
+        error);
+}
+
 /*
  * Creates a new filter.
  *
@@ -265,15 +289,24 @@ gui_filter_new (int enabled, const char *name, const char *buffer_name,
 {
     struct t_gui_filter *new_filter;
     regex_t *regex1, *regex2;
-    char *pos_tab, *regex_prefix, **tags_array;
+    char *pos_tab, *regex_prefix, buf[512], str_error[512];
     const char *ptr_start_regex, *pos_regex_message;
-    int i;
+    int rc;
 
     if (!name || !buffer_name || !tags || !regex)
+    {
+        gui_filter_new_error (name, _("not enough arguments"));
         return NULL;
+    }
 
     if (gui_filter_search_by_name (name))
+    {
+        gui_filter_new_error (name,
+                              _("a filter with same name already exists "
+                                "(choose another name or use option "
+                                "\"addreplace\" to overwrite it)"));
         return NULL;
+    }
 
     ptr_start_regex = regex;
     if ((ptr_start_regex[0] == '!')
@@ -305,9 +338,16 @@ gui_filter_new (int enabled, const char *name, const char *buffer_name,
             regex1 = malloc (sizeof (*regex1));
             if (regex1)
             {
-                if (string_regcomp (regex1, regex_prefix,
-                                    REG_EXTENDED | REG_ICASE | REG_NOSUB) != 0)
+                rc = string_regcomp (regex1, regex_prefix,
+                                     REG_EXTENDED | REG_ICASE | REG_NOSUB);
+                if (rc != 0)
                 {
+                    regerror (rc, regex1, buf, sizeof (buf));
+                    snprintf (str_error, sizeof (str_error),
+                              /* TRANSLATORS: %s is the error returned by regerror */
+                              _("invalid regular expression (%s)"),
+                              buf);
+                    gui_filter_new_error (name, str_error);
                     free (regex_prefix);
                     free (regex1);
                     return NULL;
@@ -320,9 +360,16 @@ gui_filter_new (int enabled, const char *name, const char *buffer_name,
             regex2 = malloc (sizeof (*regex2));
             if (regex2)
             {
-                if (string_regcomp (regex2, pos_regex_message,
-                                    REG_EXTENDED | REG_ICASE | REG_NOSUB) != 0)
+                rc = string_regcomp (regex2, pos_regex_message,
+                                     REG_EXTENDED | REG_ICASE | REG_NOSUB);
+                if (rc != 0)
                 {
+                    regerror (rc, regex2, buf, sizeof (buf));
+                    snprintf (str_error, sizeof (str_error),
+                              /* TRANSLATORS: %s is the error returned by regerror */
+                              _("invalid regular expression (%s)"),
+                              buf);
+                    gui_filter_new_error (name, str_error);
                     if (regex_prefix)
                         free (regex_prefix);
                     if (regex1)
@@ -348,39 +395,25 @@ gui_filter_new (int enabled, const char *name, const char *buffer_name,
         new_filter->enabled = enabled;
         new_filter->name = strdup (name);
         new_filter->buffer_name = strdup ((buffer_name) ? buffer_name : "*");
-        new_filter->buffers = string_split (new_filter->buffer_name,
-                                            ",", 0, 0,
-                                            &new_filter->num_buffers);
+        new_filter->buffers = string_split (
+            new_filter->buffer_name,
+            ",",
+            NULL,
+            WEECHAT_STRING_SPLIT_STRIP_LEFT
+            | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+            | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+            0,
+            &new_filter->num_buffers);
         new_filter->tags = (tags) ? strdup (tags) : NULL;
-        new_filter->tags_count = 0;
-        new_filter->tags_array = NULL;
-        if (new_filter->tags)
-        {
-            tags_array = string_split (new_filter->tags, ",", 0, 0,
-                                       &new_filter->tags_count);
-            if (tags_array)
-            {
-                new_filter->tags_array = malloc (new_filter->tags_count *
-                                                 sizeof (*new_filter->tags_array));
-                if (new_filter->tags_array)
-                {
-                    for (i = 0; i < new_filter->tags_count; i++)
-                    {
-                        new_filter->tags_array[i] = string_split (tags_array[i],
-                                                                  "+", 0, 0,
-                                                                  NULL);
-                    }
-                }
-                string_free_split (tags_array);
-            }
-        }
+        new_filter->tags_array = string_split_tags (new_filter->tags,
+                                                    &new_filter->tags_count);
         new_filter->regex = strdup (regex);
         new_filter->regex_prefix = regex1;
         new_filter->regex_message = regex2;
 
         /* add filter to filters list */
         new_filter->prev_filter = last_gui_filter;
-        if (gui_filters)
+        if (last_gui_filter)
             last_gui_filter->next_filter = new_filter;
         else
             gui_filters = new_filter;
@@ -390,6 +423,10 @@ gui_filter_new (int enabled, const char *name, const char *buffer_name,
         (void) hook_signal_send ("filter_added",
                                  WEECHAT_HOOK_SIGNAL_POINTER, new_filter);
     }
+    else
+    {
+        gui_filter_new_error (name, _("not enough memory"));
+    }
 
     return new_filter;
 }
@@ -424,7 +461,8 @@ gui_filter_rename (struct t_gui_filter *filter, const char *new_name)
 void
 gui_filter_free (struct t_gui_filter *filter)
 {
-    int i;
+    if (!filter)
+        return;
 
     (void) hook_signal_send ("filter_removing",
                              WEECHAT_HOOK_SIGNAL_POINTER, filter);
@@ -439,13 +477,7 @@ gui_filter_free (struct t_gui_filter *filter)
     if (filter->tags)
         free (filter->tags);
     if (filter->tags_array)
-    {
-        for (i = 0; i < filter->tags_count; i++)
-        {
-            string_free_split (filter->tags_array[i]);
-        }
-        free (filter->tags_array);
-    }
+        string_free_split_tags (filter->tags_array);
     if (filter->regex)
         free (filter->regex);
     if (filter->regex_prefix)
@@ -492,11 +524,13 @@ gui_filter_free_all ()
  */
 
 struct t_hdata *
-gui_filter_hdata_filter_cb (void *data, const char *hdata_name)
+gui_filter_hdata_filter_cb (const void *pointer, void *data,
+                            const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = hdata_new (NULL, hdata_name, "prev_filter", "next_filter",
