@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_GUI_FILTER_H
@@ -56,7 +56,7 @@ extern int gui_filters_enabled;
 extern int gui_filter_check_line (struct t_gui_line_data *line_data);
 extern void gui_filter_buffer (struct t_gui_buffer *buffer,
                                struct t_gui_line_data *line_data);
-extern void gui_filter_all_buffers ();
+extern void gui_filter_all_buffers (struct t_gui_filter *filter);
 extern void gui_filter_global_enable ();
 extern void gui_filter_global_disable ();
 extern struct t_gui_filter *gui_filter_search_by_name (const char *name);
