@@ -1,7 +1,7 @@
 /*
  * gui-key.c - keyboard functions (used by all GUI)
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -921,7 +921,7 @@ gui_key_unbind_plugin (const char *context, const char *key)
 {
     int ctxt, num_keys, area_type;
     char *area_name;
-    struct t_gui_key *ptr_key;
+    struct t_gui_key *ptr_key, *ptr_next_key;
 
     ctxt = gui_key_search_context (context);
     if (ctxt < 0)
@@ -944,8 +944,11 @@ gui_key_unbind_plugin (const char *context, const char *key)
         gui_key_set_area_type_name (key + 5, &area_type, &area_name);
         if (area_name)
         {
-            for (ptr_key = gui_keys[ctxt]; ptr_key; ptr_key = ptr_key->next_key)
+            ptr_key = gui_keys[ctxt];
+            while (ptr_key)
             {
+                ptr_next_key = ptr_key->next_key;
+
                 if (((ptr_key->area_type[0] == area_type)
                      && ptr_key->area_name[0]
                      && (strcmp (ptr_key->area_name[0], area_name) == 0))
@@ -955,6 +958,8 @@ gui_key_unbind_plugin (const char *context, const char *key)
                 {
                     num_keys += gui_key_unbind (NULL, ctxt, ptr_key->key);
                 }
+
+                ptr_key = ptr_next_key;
             }
             free (area_name);
         }
@@ -1042,7 +1047,7 @@ gui_key_focus_command (const char *key, int context,
 {
     struct t_gui_key *ptr_key;
     int i, matching, debug, rc;
-    long unsigned int value;
+    unsigned long value;
     char *command, **commands;
     const char *str_buffer;
     struct t_hashtable *hashtable;
@@ -1159,7 +1164,7 @@ gui_key_focus_command (const char *key, int context,
                                                  command,
                                                  ptr_buffer->full_name);
                             }
-                            (void) input_data (ptr_buffer, command);
+                            (void) input_data (ptr_buffer, command, NULL);
                             free (command);
                         }
                     }
@@ -1388,7 +1393,7 @@ gui_key_pressed (const char *key_str)
                     for (i = 0; commands[i]; i++)
                     {
                         (void) input_data (gui_current_window->buffer,
-                                           commands[i]);
+                                           commands[i], NULL);
                     }
                     string_free_split (commands);
                 }
@@ -1920,22 +1925,22 @@ gui_key_hdata_key_cb (const void *pointer, void *data,
                       "gui_keys%s%s",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : "_",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : gui_key_context_string[i]);
-            hdata_new_list(hdata, str_list, &gui_keys[i], 0);
+            hdata_new_list (hdata, str_list, &gui_keys[i], 0);
             snprintf (str_list, sizeof (str_list),
                       "last_gui_key%s%s",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : "_",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : gui_key_context_string[i]);
-            hdata_new_list(hdata, str_list, &last_gui_key[i], 0);
+            hdata_new_list (hdata, str_list, &last_gui_key[i], 0);
             snprintf (str_list, sizeof (str_list),
                       "gui_default_keys%s%s",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : "_",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : gui_key_context_string[i]);
-            hdata_new_list(hdata, str_list, &gui_default_keys[i], 0);
+            hdata_new_list (hdata, str_list, &gui_default_keys[i], 0);
             snprintf (str_list, sizeof (str_list),
                       "last_gui_default_key%s%s",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : "_",
                       (i == GUI_KEY_CONTEXT_DEFAULT) ? "" : gui_key_context_string[i]);
-            hdata_new_list(hdata, str_list, &last_gui_default_key[i], 0);
+            hdata_new_list (hdata, str_list, &last_gui_default_key[i], 0);
         }
     }
     return hdata;
