@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_GUI_WINDOW_H
@@ -76,7 +76,6 @@ struct t_gui_window
     /* coordinates (for focus) */
     int coords_size;                   /* size of coords (number of lines)  */
     struct t_gui_window_coords *coords;/* coords for window                 */
-    int coords_x_message;              /* start X for messages              */
 
     /* tree */
     struct t_gui_window_tree *ptr_tree;/* pointer to leaf in windows tree   */
@@ -233,6 +232,7 @@ extern void gui_window_resize_delta (struct t_gui_window_tree *tree,
                                      int delta_percentage);
 extern int gui_window_merge (struct t_gui_window *window);
 extern void gui_window_merge_all (struct t_gui_window *window);
+extern int gui_window_close (struct t_gui_window *window);
 extern void gui_window_switch_up (struct t_gui_window *window);
 extern void gui_window_switch_down (struct t_gui_window *window);
 extern void gui_window_switch_left (struct t_gui_window *window);
