@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_BUFLIST_BAR_ITEM_H
@@ -24,10 +24,14 @@
 
 #define BUFLIST_BAR_NUM_ITEMS 3
 
+struct t_gui_bar_item;
+
+extern struct t_gui_bar_item *buflist_bar_item_buflist[BUFLIST_BAR_NUM_ITEMS];
 extern struct t_arraylist *buflist_list_buffers[BUFLIST_BAR_NUM_ITEMS];
 
 extern const char *buflist_bar_item_get_name (int index);
 extern int buflist_bar_item_get_index (const char *item_name);
+extern int buflist_bar_item_get_index_with_pointer (struct t_gui_bar_item *item);
 extern void buflist_bar_item_update (int force);
 extern int buflist_bar_item_init ();
 extern void buflist_bar_item_end ();
