@@ -1,7 +1,7 @@
 /*
  * exec-command.c - exec command
  *
- * Copyright (C) 2014-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -87,7 +87,7 @@ exec_command_list ()
             /* running command */
             weechat_printf (NULL,
                             /* TRANSLATORS: %s before "ago" is elapsed time, for example: "3m59" */
-                            _("  %s%s%s %d%s%s%s: %s\"%s%s%s\"%s (pid: %d, "
+                            _("  %s%s%s %ld%s%s%s: %s\"%s%s%s\"%s (pid: %d, "
                               "started %s ago)"),
                             weechat_color (weechat_config_string (exec_config_color_flag_running)),
                             ">>",
@@ -116,7 +116,7 @@ exec_command_list ()
                           "%Y-%m-%d %H:%M:%S", local_time) == 0)
                 str_time2[0] = '\0';
             weechat_printf (NULL,
-                            "  %s%s%s %d%s%s%s: %s\"%s%s%s\"%s (%s -> %s, %s)",
+                            "  %s%s%s %ld%s%s%s: %s\"%s%s%s\"%s (%s -> %s, %s)",
                             weechat_color (weechat_config_string (exec_config_color_flag_finished)),
                             "[]",
                             weechat_color ("reset"),
@@ -244,6 +244,10 @@ exec_command_parse_options (struct t_exec_cmd_options *cmd_options,
             cmd_options->output_to_buffer_exec_cmd = 1;
             cmd_options->new_buffer = 0;
         }
+        else if (weechat_strcasecmp (argv[i], "-oerr") == 0)
+        {
+            cmd_options->output_to_buffer_stderr = 1;
+        }
         else if (weechat_strcasecmp (argv[i], "-n") == 0)
         {
             cmd_options->output_to_buffer = 0;
@@ -335,7 +339,7 @@ exec_command_parse_options (struct t_exec_cmd_options *cmd_options,
             if (argv[i][0] == '"')
             {
                 /* search the ending double quote */
-                length_total = 1;
+                length_total = 2;
                 end = i;
                 while (end < argc)
                 {
@@ -405,13 +409,17 @@ int
 exec_command_run (struct t_gui_buffer *buffer,
                   int argc, char **argv, char **argv_eol, int start_arg)
 {
-    char str_buffer[512];
+    char str_buffer[512], *shell, *default_shell = "sh";
     struct t_exec_cmd *new_exec_cmd;
     struct t_exec_cmd_options cmd_options;
     struct t_hashtable *process_options;
     struct t_infolist *ptr_infolist;
     struct t_gui_buffer *ptr_new_buffer;
 
+    shell = NULL;
+    new_exec_cmd = NULL;
+    process_options = NULL;
+
     /* parse command options */
     cmd_options.command_index = -1;
     cmd_options.use_shell = 0;
@@ -422,6 +430,7 @@ exec_command_run (struct t_gui_buffer *buffer,
     cmd_options.ptr_buffer = buffer;
     cmd_options.output_to_buffer = 0;
     cmd_options.output_to_buffer_exec_cmd = 0;
+    cmd_options.output_to_buffer_stderr = 0;
     cmd_options.new_buffer = 0;
     cmd_options.new_buffer_clear = 0;
     cmd_options.switch_to_buffer = 1;
@@ -443,29 +452,33 @@ exec_command_run (struct t_gui_buffer *buffer,
                         _("%s%s: invalid options in option "
                           "exec.command.default_options"),
                         weechat_prefix ("error"), EXEC_PLUGIN_NAME);
-        return WEECHAT_RC_ERROR;
+        goto error;
     }
     if (!exec_command_parse_options (&cmd_options, argc, argv, start_arg, 1))
-        return WEECHAT_RC_ERROR;
+        goto error;
 
     /* options "-bg" and "-o"/"-oc"/"-n" are incompatible */
     if (cmd_options.detached
         && (cmd_options.output_to_buffer || cmd_options.new_buffer))
-        return WEECHAT_RC_ERROR;
+    {
+        goto error;
+    }
 
     /* options "-pipe" and "-bg"/"-o"/"-oc"/"-n" are incompatible */
     if (cmd_options.pipe_command
         && (cmd_options.detached || cmd_options.output_to_buffer
             || cmd_options.new_buffer))
-        return WEECHAT_RC_ERROR;
+    {
+        goto error;
+    }
 
     /* command not found? */
     if (cmd_options.command_index < 0)
-        return WEECHAT_RC_ERROR;
+        goto error;
 
     new_exec_cmd = exec_add ();
     if (!new_exec_cmd)
-        return WEECHAT_RC_ERROR;
+        goto error;
 
     /* create hashtable for weechat_hook_process_hashtable() */
     process_options = weechat_hashtable_new (32,
@@ -473,13 +486,26 @@ exec_command_run (struct t_gui_buffer *buffer,
                                              WEECHAT_HASHTABLE_STRING,
                                              NULL, NULL);
     if (!process_options)
-    {
-        exec_free (new_exec_cmd);
-        return WEECHAT_RC_ERROR;
-    }
+        goto error;
+
     /* automatically disable shell if we are downloading an URL */
     if (strncmp (argv_eol[cmd_options.command_index], "url:", 4) == 0)
         cmd_options.use_shell = 0;
+
+    /* get default shell */
+    if (cmd_options.use_shell)
+    {
+        shell = weechat_string_eval_expression (
+            weechat_config_string (exec_config_command_shell),
+            NULL, NULL, NULL);
+        if (!shell || !shell[0])
+        {
+            if (shell)
+                free (shell);
+            shell = strdup (default_shell);
+        }
+    }
+
     if (cmd_options.use_shell)
     {
         /* command will be: sh -c "command arguments..." */
@@ -532,7 +558,7 @@ exec_command_run (struct t_gui_buffer *buffer,
             else
             {
                 snprintf (str_buffer, sizeof (str_buffer),
-                          "exec.%d", new_exec_cmd->number);
+                          "exec.%ld", new_exec_cmd->number);
             }
             ptr_new_buffer = exec_buffer_new (str_buffer,
                                               (cmd_options.new_buffer == 2),
@@ -564,6 +590,7 @@ exec_command_run (struct t_gui_buffer *buffer,
     }
     new_exec_cmd->output_to_buffer = cmd_options.output_to_buffer;
     new_exec_cmd->output_to_buffer_exec_cmd = cmd_options.output_to_buffer_exec_cmd;
+    new_exec_cmd->output_to_buffer_stderr = cmd_options.output_to_buffer_stderr;
     new_exec_cmd->line_numbers = (cmd_options.line_numbers < 0) ?
         cmd_options.new_buffer : cmd_options.line_numbers;
     new_exec_cmd->color = cmd_options.color;
@@ -574,14 +601,15 @@ exec_command_run (struct t_gui_buffer *buffer,
     /* execute the command */
     if (weechat_exec_plugin->debug >= 1)
     {
-        weechat_printf (NULL, "%s: executing command: \"%s%s%s\"",
+        weechat_printf (NULL, "%s: executing command: \"%s%s%s%s\"",
                         EXEC_PLUGIN_NAME,
-                        (cmd_options.use_shell) ? "" : "sh -c '",
+                        (cmd_options.use_shell) ? shell : "",
+                        (cmd_options.use_shell) ? " -c '" : "",
                         argv_eol[cmd_options.command_index],
-                        (cmd_options.use_shell) ? "" : "'");
+                        (cmd_options.use_shell) ? "'" : "");
     }
     new_exec_cmd->hook = weechat_hook_process_hashtable (
-        (cmd_options.use_shell) ? "sh" : argv_eol[cmd_options.command_index],
+        (cmd_options.use_shell) ? shell : argv_eol[cmd_options.command_index],
         process_options,
         cmd_options.timeout * 1000,
         &exec_process_cb,
@@ -611,9 +639,21 @@ exec_command_run (struct t_gui_buffer *buffer,
                         argv_eol[cmd_options.command_index]);
     }
 
+    if (shell)
+        free (shell);
     weechat_hashtable_free (process_options);
 
     return WEECHAT_RC_OK;
+
+error:
+    if (shell)
+        free (shell);
+    if (new_exec_cmd)
+        exec_free (new_exec_cmd);
+    if (process_options)
+        weechat_hashtable_free (process_options);
+
+    return WEECHAT_RC_ERROR;
 }
 
 /*
@@ -797,7 +837,7 @@ exec_command_init ()
         N_("execute external commands"),
         N_("-list"
            " || [-sh|-nosh] [-bg|-nobg] [-stdin|-nostdin] [-buffer <name>] "
-           "[-l|-o|-n|-nf] [-cl|-nocl] [-sw|-nosw] [-ln|-noln] "
+           "[-l|-o|-oc|-n|-nf] [-oerr] [-cl|-nocl] [-sw|-nosw] [-ln|-noln] "
            "[-flush|-noflush] [-color ansi|auto|irc|weechat|strip] [-rc|-norc] "
            "[-timeout <timeout>] [-name <name>] [-pipe <command>] "
            "[-hsignal <name>] <command>"
@@ -809,8 +849,9 @@ exec_command_init ()
            " || -set <id> <property> <value>"
            " || -del <id>|-all [<id>...]"),
         N_("   -list: list commands\n"
-           "     -sh: use the shell to execute the command (WARNING: use this "
-           "option ONLY if all arguments are safe, see option -nosh)\n"
+           "     -sh: use the shell to execute the command, many commands can "
+           "be piped (WARNING: use this option ONLY if all arguments are "
+           "safe, see option -nosh)\n"
            "   -nosh: do not use the shell to execute the command (required if "
            "the command has some unsafe data, for example the content of a "
             "message from another user) (default)\n"
@@ -835,6 +876,8 @@ exec_command_init ()
            "     -nf: display output of command in a new buffer with free "
            "content (no word-wrap, no limit on number of lines) (not compatible "
            "with options -bg/-pipe/-hsignal)\n"
+           "   -oerr: also send stderr (error output) to the buffer (can be "
+           "used only with options -o and -oc)\n"
            "     -cl: clear the new buffer before displaying output\n"
            "   -nocl: append to the new buffer without clear (default)\n"
            "     -sw: switch to the output buffer (default)\n"
@@ -885,7 +928,7 @@ exec_command_init ()
            "Examples:\n"
            "  /exec -n ls -l /tmp\n"
            "  /exec -sh -n ps xu | grep weechat\n"
-           "  /exec -n -norc url:http://pastebin.com/raw.php?i=xxxxxxxx\n"
+           "  /exec -n -norc url:https://pastebin.com/raw.php?i=xxxxxxxx\n"
            "  /exec -nf -noln links -dump "
            "https://weechat.org/files/doc/devel/weechat_user.en.html\n"
            "  /exec -o uptime\n"
