@@ -1,7 +1,7 @@
 /*
  * exec-config.c - exec configuration options (file exec.conf)
  *
- * Copyright (C) 2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -34,11 +34,12 @@ struct t_config_file *exec_config_file = NULL;
 
 struct t_config_option *exec_config_command_default_options;
 struct t_config_option *exec_config_command_purge_delay;
+struct t_config_option *exec_config_command_shell;
 
 /* exec config, color section */
 
-struct t_config_option *exec_config_color_flag_running;
 struct t_config_option *exec_config_color_flag_finished;
+struct t_config_option *exec_config_color_flag_running;
 
 char **exec_config_cmd_options = NULL;
 int exec_config_cmd_num_options = 0;
@@ -49,10 +50,11 @@ int exec_config_cmd_num_options = 0;
  */
 
 void
-exec_config_change_command_default_options (void *data,
+exec_config_change_command_default_options (const void *pointer, void *data,
                                             struct t_config_option *option)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) option;
 
@@ -61,7 +63,13 @@ exec_config_change_command_default_options (void *data,
 
     exec_config_cmd_options = weechat_string_split (
         weechat_config_string (exec_config_command_default_options),
-        " ", 0, 0, &exec_config_cmd_num_options);
+        " ",
+        NULL,
+        WEECHAT_STRING_SPLIT_STRIP_LEFT
+        | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+        | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+        0,
+        &exec_config_cmd_num_options);
 }
 
 /*
@@ -69,9 +77,11 @@ exec_config_change_command_default_options (void *data,
  */
 
 int
-exec_config_reload_cb (void *data, struct t_config_file *config_file)
+exec_config_reload_cb (const void *pointer, void *data,
+                       struct t_config_file *config_file)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     return weechat_config_reload (config_file);
@@ -91,19 +101,22 @@ exec_config_init ()
     struct t_config_section *ptr_section;
 
     exec_config_file = weechat_config_new (EXEC_CONFIG_NAME,
-                                           &exec_config_reload_cb, NULL);
+                                           &exec_config_reload_cb, NULL, NULL);
     if (!exec_config_file)
         return 0;
 
     /* command */
     ptr_section = weechat_config_new_section (exec_config_file, "command",
                                               0, 0,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL);
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL);
     if (!ptr_section)
     {
         weechat_config_free (exec_config_file);
+        exec_config_file = NULL;
         return 0;
     }
 
@@ -113,40 +126,56 @@ exec_config_init ()
         N_("default options for command /exec (see /help exec); example: "
            "\"-nosh -bg\" to run all commands in background (no output), and "
            "without using the shell"),
-        NULL, 0, 0, "", NULL, 0, NULL, NULL,
-        &exec_config_change_command_default_options, NULL, NULL, NULL);
+        NULL, 0, 0, "", NULL, 0,
+        NULL, NULL, NULL,
+        &exec_config_change_command_default_options, NULL, NULL,
+        NULL, NULL, NULL);
     exec_config_command_purge_delay = weechat_config_new_option (
         exec_config_file, ptr_section,
         "purge_delay", "integer",
         N_("delay for purging finished commands (in seconds, 0 = purge "
            "commands immediately, -1 = never purge)"),
         NULL, -1, 36000 * 24 * 30, "0", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    exec_config_command_shell = weechat_config_new_option (
+        exec_config_file, ptr_section,
+        "shell", "string",
+        N_("shell to use with command \"/exec -sh\"; it can be just the name "
+           "of shell if it is in PATH (for example \"bash\") or the absolute "
+           "path to the shell (for example \"/bin/bash\"); if value is empty, "
+           "\"sh\" is used (note: content is evaluated, see /help eval)"),
+        NULL, 0, 0, "${env:SHELL}", NULL, 0,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL,
+        NULL, NULL, NULL);
 
     /* color */
     ptr_section = weechat_config_new_section (exec_config_file, "color",
                                               0, 0,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL, NULL, NULL,
-                                              NULL, NULL);
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL,
+                                              NULL, NULL, NULL);
     if (!ptr_section)
     {
         weechat_config_free (exec_config_file);
+        exec_config_file = NULL;
         return 0;
     }
 
-    exec_config_color_flag_running = weechat_config_new_option (
-        exec_config_file, ptr_section,
-        "flag_running", "color",
-        N_("text color for a running command flag in list of commands"),
-        NULL, 0, 0, "lightgreen", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
     exec_config_color_flag_finished = weechat_config_new_option (
         exec_config_file, ptr_section,
         "flag_finished", "color",
         N_("text color for a finished command flag in list of commands"),
         NULL, 0, 0, "lightred", NULL, 0,
-        NULL, NULL, NULL, NULL, NULL, NULL);
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    exec_config_color_flag_running = weechat_config_new_option (
+        exec_config_file, ptr_section,
+        "flag_running", "color",
+        N_("text color for a running command flag in list of commands"),
+        NULL, 0, 0, "lightgreen", NULL, 0,
+        NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
 
     return 1;
 }
