@@ -102,15 +102,17 @@ fifo_command_init ()
            "\n"
            "FIFO pipe is used as remote control of WeeChat: you can send "
            "commands or text to the FIFO pipe from your shell.\n"
-           "By default the FIFO pipe is in ~/.weechat/weechat_fifo\n"
+           "By default the FIFO pipe is called weechat_fifo_xxx (where xxx is "
+           "the WeeChat process id) and located in the WeeChat runtime "
+           "directory (see /debug dirs).\n"
            "\n"
            "The expected format is one of:\n"
            "  plugin.buffer *text or command here\n"
            "  *text or command here\n"
            "\n"
-           "For example to change your freenode nick:\n"
-           "  echo 'irc.server.freenode */nick newnick' "
-           ">~/.weechat/weechat_fifo\n"
+           "For example to change your libera nick:\n"
+           "  echo 'irc.server.libera */nick newnick' "
+           ">/run/user/1000/weechat/weechat_fifo_12345\n"
            "\n"
            "Please read the user's guide for more info and examples.\n"
            "\n"
