@@ -1,7 +1,7 @@
 /*
  * fset-command.c - Fast Set command
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -469,7 +469,11 @@ fset_command_run_set_cb (const void *pointer, void *data,
 
     rc = WEECHAT_RC_OK;
 
-    argv = weechat_string_split (command, " ", 0, 0, &argc);
+    argv = weechat_string_split (command, " ", NULL,
+                                 WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                 | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                 | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                 0, &argc);
 
     if (argc > 2)
         goto end;
@@ -642,6 +646,10 @@ fset_command_init ()
            "value\n"
            "               d==xxx  show only changed options with exact value "
            "\"xxx\"\n"
+           "               h=xxx   show only options with \"xxx\" in "
+           "description (translated)\n"
+           "               he=xxx  show only options with \"xxx\" in "
+           "description (in English)\n"
            "               =xxx    show only options with \"xxx\" in value\n"
            "               ==xxx   show only options with exact value \"xxx\"\n"
            "               c:xxx   show only options matching the evaluated "
