@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -24,6 +24,7 @@
 
 extern struct t_config_file *fset_config_file;
 
+extern struct t_config_option *fset_config_look_auto_refresh;
 extern struct t_config_option *fset_config_look_auto_unmark;
 extern struct t_config_option *fset_config_look_condition_catch_set;
 extern struct t_config_option *fset_config_look_export_help_default;
@@ -37,10 +38,10 @@ extern struct t_config_option *fset_config_look_use_color_value;
 extern struct t_config_option *fset_config_look_use_keys;
 extern struct t_config_option *fset_config_look_use_mute;
 
-extern struct t_config_option *fset_config_format_option[2];
 extern struct t_config_option *fset_config_format_export_help;
 extern struct t_config_option *fset_config_format_export_option;
 extern struct t_config_option *fset_config_format_export_option_null;
+extern struct t_config_option *fset_config_format_option[2];
 
 extern struct t_config_option *fset_config_color_default_value[2];
 extern struct t_config_option *fset_config_color_description[2];
@@ -79,6 +80,7 @@ extern struct t_config_option *fset_config_color_value[2];
 extern struct t_config_option *fset_config_color_value_changed[2];
 extern struct t_config_option *fset_config_color_value_undef[2];
 
+extern char **fset_config_auto_refresh;
 extern char **fset_config_sort_fields;
 extern int fset_config_sort_fields_count;
 extern int fset_config_format_option_num_lines[2];
