@@ -1,7 +1,7 @@
 /*
  * irc-channel.c - channel and private chat management for IRC plugin
  *
- * Copyright (C) 2003-2018 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -40,6 +40,13 @@
 #include "irc-input.h"
 
 
+char *irc_channel_typing_state_string[IRC_CHANNEL_NUM_TYPING_STATES] =
+{ "off", "active", "paused", "done" };
+
+/* default CHANTYPES */
+char *irc_channel_default_chantypes = "#&";
+
+
 /*
  * Checks if a channel pointer is valid for a server.
  *
@@ -247,6 +254,18 @@ irc_channel_create_buffer (struct t_irc_server *server,
 
     ptr_buffer = irc_channel_search_buffer (server, channel_type,
                                             channel_name);
+    if (!ptr_buffer && (channel_type == IRC_CHANNEL_TYPE_PRIVATE))
+    {
+        /*
+         * in case of private buffer, we reuse a buffer which has wrong type
+         * "channel" (opened by a manual /join or autojoin)
+         */
+        ptr_buffer = irc_channel_search_buffer (server,
+                                                IRC_CHANNEL_TYPE_CHANNEL,
+                                                channel_name);
+        if (ptr_buffer)
+            weechat_bar_item_update ("buffer_name");
+    }
     if (ptr_buffer)
     {
         weechat_nicklist_remove_all (ptr_buffer);
@@ -328,6 +347,7 @@ irc_channel_create_buffer (struct t_irc_server *server,
     weechat_buffer_set (ptr_buffer, "localvar_set_type",
                         (channel_type == IRC_CHANNEL_TYPE_CHANNEL) ? "channel" : "private");
     weechat_buffer_set (ptr_buffer, "localvar_set_nick", server->nick);
+    weechat_buffer_set (ptr_buffer, "localvar_set_host", server->host);
     weechat_buffer_set (ptr_buffer, "localvar_set_server", server->name);
     weechat_buffer_set (ptr_buffer, "localvar_set_channel", channel_name);
     if (server->is_away && server->away_message)
@@ -491,6 +511,8 @@ irc_channel_new (struct t_irc_server *server, int channel_type,
             irc_modelist_new (new_channel, ptr_chanmode[0]);
     }
     new_channel->join_smart_filtered = NULL;
+    new_channel->typing_state = IRC_CHANNEL_TYPING_STATE_OFF;
+    new_channel->typing_status_sent = 0;
     new_channel->buffer = ptr_buffer;
     new_channel->buffer_as_string = NULL;
 
@@ -556,9 +578,13 @@ irc_channel_set_buffer_title (struct t_irc_channel *channel)
             channel->topic,
             (weechat_config_boolean (irc_config_look_topic_strip_colors)) ? 0 : 1);
         weechat_buffer_set (channel->buffer, "title", title_color);
+        if (title_color)
+            free (title_color);
     }
     else
+    {
         weechat_buffer_set (channel->buffer, "title", "");
+    }
 }
 
 /*
@@ -568,11 +594,57 @@ irc_channel_set_buffer_title (struct t_irc_channel *channel)
 void
 irc_channel_set_topic (struct t_irc_channel *channel, const char *topic)
 {
+    int display_warning;
+
+    /*
+     * display a warning in the private buffer if the address of remote
+     * nick has changed (that means you may talk to someone else!)
+     */
+    display_warning = (
+        (channel->type == IRC_CHANNEL_TYPE_PRIVATE)
+        && weechat_config_boolean (irc_config_look_display_pv_warning_address)
+        && channel->topic && channel->topic[0]
+        && topic && topic[0]
+        && (strcmp (channel->topic, topic) != 0));
+
     if (channel->topic)
         free (channel->topic);
     channel->topic = (topic) ? strdup (topic) : NULL;
 
     irc_channel_set_buffer_title (channel);
+
+    if (display_warning)
+    {
+        weechat_printf_date_tags (
+                    channel->buffer,
+                    0,
+                    "no_log,warning_nick_address",
+                    _("%sWarning: the address of remote nick has changed"),
+                    weechat_prefix ("error"));
+    }
+}
+
+/*
+ * Sets topic of all private buffers with a nick.
+ */
+
+void
+irc_channel_set_topic_private_buffers (struct t_irc_server *server,
+                                       struct t_irc_nick *nick,
+                                       const char *nickname,
+                                       const char *topic)
+{
+    struct t_irc_channel *ptr_channel;
+
+    for (ptr_channel = server->channels; ptr_channel;
+         ptr_channel = ptr_channel->next_channel)
+    {
+        if ((ptr_channel->type == IRC_CHANNEL_TYPE_PRIVATE)
+            && (irc_server_strcasecmp (server, ptr_channel->name, (nick) ? nick->name : nickname) == 0))
+        {
+            irc_channel_set_topic (ptr_channel, topic);
+        }
+    }
 }
 
 /*
@@ -600,16 +672,17 @@ int
 irc_channel_is_channel (struct t_irc_server *server, const char *string)
 {
     char first_char[2];
+    const char *ptr_chantypes;
 
     if (!string)
         return 0;
 
     first_char[0] = string[0];
     first_char[1] = '\0';
-    return (strpbrk (first_char,
-                     (server && server->chantypes) ?
-                     server->chantypes : IRC_CHANNEL_DEFAULT_CHANTYPES)) ?
-        1 : 0;
+
+    ptr_chantypes = irc_server_get_chantypes (server);
+
+    return (strpbrk (first_char, ptr_chantypes)) ? 1 : 0;
 }
 
 /*
@@ -625,21 +698,25 @@ irc_channel_get_auto_chantype (struct t_irc_server *server,
                                const char *channel_name)
 {
     static char chantype[2];
+    const char *ptr_chantypes;
 
     chantype[0] = '\0';
     chantype[1] = '\0';
 
     if (weechat_config_boolean (irc_config_look_join_auto_add_chantype)
-        && !irc_channel_is_channel (server, channel_name)
-        && server->chantypes
-        && server->chantypes[0])
+        && !irc_channel_is_channel (server, channel_name))
     {
-        /*
-         * use '#' if it's in chantypes (anywhere in the string), because it is
-         * the most common channel type, and fallback on first channel type
-         */
-        chantype[0] = (strchr (server->chantypes, '#')) ?
-            '#' : server->chantypes[0];
+        ptr_chantypes = irc_server_get_chantypes (server);
+        if (ptr_chantypes && ptr_chantypes[0])
+        {
+            /*
+             * use '#' if it's in chantypes (anywhere in the string), because
+             * it is the most common channel type, and fallback on first
+             * channel type
+             */
+            chantype[0] = (strchr (ptr_chantypes, '#')) ?
+                '#' : ptr_chantypes[0];
+        }
     }
 
     return chantype;
@@ -1095,8 +1172,8 @@ void
 irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
                                         const char *nick)
 {
-    int i, unmask_delay, length_tags, nick_found, join, chghost;
-    int nick_changed, smart_filtered, remove_smart_filter;
+    int i, unmask_delay, length_tags, nick_found, join, account;
+    int chghost, nick_changed, smart_filtered, remove_smart_filter;
     time_t *ptr_time, date_min;
     struct t_hdata *hdata_line, *hdata_line_data;
     struct t_gui_line *own_lines;
@@ -1172,6 +1249,8 @@ irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
             length_tags = 0;
             nick_found = 0;
             join = 0;
+            account = 0;
+            chghost = 0;
             nick_changed = 0;
             irc_nick1 = NULL;
             irc_nick2 = NULL;
@@ -1185,6 +1264,8 @@ irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
                 }
                 else if (strcmp (tags[i], "irc_join") == 0)
                     join = 1;
+                else if (strcmp (tags[i], "irc_account") == 0)
+                    account = 1;
                 else if (strcmp (tags[i], "irc_chghost") == 0)
                     chghost = 1;
                 else if (strcmp (tags[i], "irc_nick") == 0)
@@ -1210,7 +1291,7 @@ irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
                     break;
                 remove_smart_filter = 1;
             }
-            else if (nick_found && (join || chghost) && smart_filtered)
+            else if (nick_found && (join || account || chghost) && smart_filtered)
             {
                 remove_smart_filter = 1;
             }
@@ -1270,7 +1351,6 @@ irc_channel_join_smart_filtered_unmask (struct t_irc_channel *channel,
     weechat_hashtable_remove (channel->join_smart_filtered, nick);
 }
 
-
 /*
  * Rejoins a channel (for example after kick).
  */
@@ -1509,6 +1589,8 @@ irc_channel_hdata_channel_cb (const void *pointer, void *data,
         WEECHAT_HDATA_VAR(struct t_irc_channel, modelists, POINTER, 0, NULL, "irc_modelist");
         WEECHAT_HDATA_VAR(struct t_irc_channel, last_modelist, POINTER, 0, NULL, "irc_modelist");
         WEECHAT_HDATA_VAR(struct t_irc_channel, join_smart_filtered, HASHTABLE, 0, NULL, NULL);
+        WEECHAT_HDATA_VAR(struct t_irc_channel, typing_state, INTEGER, 0, NULL, NULL);
+        WEECHAT_HDATA_VAR(struct t_irc_channel, typing_status_sent, TIME, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_channel, buffer, POINTER, 0, NULL, "buffer");
         WEECHAT_HDATA_VAR(struct t_irc_channel, buffer_as_string, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_channel, prev_channel, POINTER, 0, NULL, hdata_name);
@@ -1699,6 +1781,8 @@ irc_channel_print_log (struct t_irc_channel *channel)
                         channel->join_smart_filtered,
                         weechat_hashtable_get_string (channel->join_smart_filtered,
                                                       "keys_values"));
+    weechat_log_printf ("       typing_state . . . . . . : %d",    channel->typing_state);
+    weechat_log_printf ("       typing_status_sent . . . : %lld",  (long long)channel->typing_status_sent);
     weechat_log_printf ("       buffer . . . . . . . . . : 0x%lx", channel->buffer);
     weechat_log_printf ("       buffer_as_string . . . . : '%s'",  channel->buffer_as_string);
     weechat_log_printf ("       prev_channel . . . . . . : 0x%lx", channel->prev_channel);
