@@ -1,7 +1,7 @@
 /*
  * irc-ctcp.c - IRC CTCP protocol
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -150,19 +150,25 @@ irc_ctcp_display_request (struct t_irc_server *server,
 void
 irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
                                   const char *command, const char *nick,
-                                  const char *address, char *arguments)
+                                  const char *address, const char *arguments)
 {
-    char *pos_end, *pos_space, *pos_args, *pos_usec;
+    char *dup_arguments, *ptr_args, *pos_end, *pos_space, *pos_args, *pos_usec;
     struct timeval tv;
     long sec1, usec1, sec2, usec2, difftime;
 
-    while (arguments && arguments[0])
+    dup_arguments = strdup (arguments);
+    if (!dup_arguments)
+        return;
+
+    ptr_args = dup_arguments;
+
+    while (ptr_args && ptr_args[0])
     {
-        pos_end = strrchr (arguments + 1, '\01');
+        pos_end = strrchr (ptr_args + 1, '\01');
         if (pos_end)
             pos_end[0] = '\0';
 
-        pos_space = strchr (arguments + 1, ' ');
+        pos_space = strchr (ptr_args + 1, ' ');
         if (pos_space)
         {
             pos_space[0] = '\0';
@@ -171,7 +177,7 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
             {
                 pos_args++;
             }
-            if (strcmp (arguments + 1, "PING") == 0)
+            if (weechat_strcasecmp (ptr_args + 1, "PING") == 0)
             {
                 pos_usec = strchr (pos_args, ' ');
                 if (pos_usec)
@@ -198,11 +204,9 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
                         nick,
                         IRC_COLOR_RESET,
                         IRC_COLOR_CHAT_CHANNEL,
-                        arguments + 1,
+                        ptr_args + 1,
                         IRC_COLOR_RESET,
                         (float)difftime / 1000000.0);
-
-                    pos_usec[0] = ' ';
                 }
             }
             else
@@ -218,12 +222,11 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
                     nick,
                     IRC_COLOR_RESET,
                     IRC_COLOR_CHAT_CHANNEL,
-                    arguments + 1,
+                    ptr_args + 1,
                     IRC_COLOR_RESET,
                     " ",
                     pos_args);
             }
-            pos_space[0] = ' ';
         }
         else
         {
@@ -238,17 +241,16 @@ irc_ctcp_display_reply_from_nick (struct t_irc_server *server, time_t date,
                 nick,
                 IRC_COLOR_RESET,
                 IRC_COLOR_CHAT_CHANNEL,
-                arguments + 1,
+                ptr_args + 1,
                 "",
                 "",
                 "");
         }
 
-        if (pos_end)
-            pos_end[0] = '\01';
-
-        arguments = (pos_end) ? pos_end + 1 : NULL;
+        ptr_args = (pos_end) ? pos_end + 1 : NULL;
     }
+
+    free (dup_arguments);
 }
 
 /*
@@ -266,14 +268,21 @@ irc_ctcp_reply_to_nick (struct t_irc_server *server,
     int number;
     char hash_key[32];
     const char *str_args;
-    char *str_args_color;
+    char *str_args_color, *ctcp_upper;
+
+    ctcp_upper = strdup (ctcp);
+    if (!ctcp_upper)
+        return;
+
+    weechat_string_toupper (ctcp_upper);
 
     hashtable = irc_server_sendf (
         server,
         IRC_SERVER_SEND_OUTQ_PRIO_LOW | IRC_SERVER_SEND_RETURN_HASHTABLE,
         NULL,
         "NOTICE %s :\01%s%s%s\01",
-        nick, ctcp,
+        nick,
+        ctcp_upper,
         (arguments) ? " " : "",
         (arguments) ? arguments : "");
 
@@ -307,7 +316,7 @@ irc_ctcp_reply_to_nick (struct t_irc_server *server,
                     nick,
                     IRC_COLOR_RESET,
                     IRC_COLOR_CHAT_CHANNEL,
-                    ctcp,
+                    ctcp_upper,
                     (str_args_color[0]) ? IRC_COLOR_RESET : "",
                     (str_args_color[0]) ? " " : "",
                     str_args_color);
@@ -317,6 +326,8 @@ irc_ctcp_reply_to_nick (struct t_irc_server *server,
         }
         weechat_hashtable_free (hashtable);
     }
+
+    free (ctcp_upper);
 }
 
 /*
@@ -328,11 +339,10 @@ irc_ctcp_reply_to_nick (struct t_irc_server *server,
 char *
 irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
 {
-    char *res, *temp, *username, *realname;
-    const char *info;
+    char *res, *temp, *username, *realname, *info, *info2;
     time_t now;
     struct tm *local_time;
-    char buf[1024];
+    char buf[4096];
     struct utsname *buf_uname;
 
     /*
@@ -354,6 +364,8 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     info = weechat_info_get ("version_git", "");
     temp = weechat_string_replace (res, "$git", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -365,13 +377,18 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
      *   0.4.0-dev (git: v0.3.9-104-g7eb5cc4)
      */
     info = weechat_info_get ("version_git", "");
+    info2 = weechat_info_get ("version", "");
     snprintf (buf, sizeof (buf), "%s%s%s%s",
-              weechat_info_get ("version", ""),
+              info2,
               (info && info[0]) ? " (git: " : "",
               (info && info[0]) ? info : "",
               (info && info[0]) ? ")" : "");
     temp = weechat_string_replace (res, "$versiongit", buf);
     free (res);
+    if (info)
+        free (info);
+    if (info2)
+        free (info2);
     if (!temp)
         return NULL;
     res = temp;
@@ -384,6 +401,8 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     info = weechat_info_get ("version", "");
     temp = weechat_string_replace (res, "$version", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -395,6 +414,8 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     info = weechat_info_get ("date", "");
     temp = weechat_string_replace (res, "$compilation", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -424,12 +445,14 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     }
 
     /*
-     * $site: WeeChat web site, example:
+     * $site: WeeChat website, example:
      *   https://weechat.org/
      */
     info = weechat_info_get ("weechat_site", "");
     temp = weechat_string_replace (res, "$site", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -441,6 +464,8 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
     info = weechat_info_get ("weechat_site_download", "");
     temp = weechat_string_replace (res, "$download", info);
     free (res);
+    if (info)
+        free (info);
     if (!temp)
         return NULL;
     res = temp;
@@ -467,9 +492,9 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
      * $username: user name, example:
      *   name
      */
-    username = weechat_string_eval_expression (
-        IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_USERNAME),
-        NULL, NULL, NULL);
+    username = irc_server_eval_expression (
+        server,
+        IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_USERNAME));
     if (username)
     {
         temp = weechat_string_replace (res, "$username", username);
@@ -484,9 +509,9 @@ irc_ctcp_replace_variables (struct t_irc_server *server, const char *format)
      * $realname: real name, example:
      *   John doe
      */
-    realname = weechat_string_eval_expression (
-        IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_REALNAME),
-        NULL, NULL, NULL);
+    realname = irc_server_eval_expression (
+        server,
+        IRC_SERVER_OPTION_STRING(server, IRC_SERVER_OPTION_REALNAME));
     if (realname)
     {
         temp = weechat_string_replace (res, "$realname", realname);
@@ -527,7 +552,7 @@ irc_ctcp_dcc_filename_without_quotes (const char *filename)
 
 void
 irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
-                   const char *arguments, char *message)
+                   const char *arguments, const char *message)
 {
     char *dcc_args, *pos, *pos_file, *pos_addr, *pos_port, *pos_size;
     char *pos_start_resume, *filename;
@@ -655,7 +680,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 
         (void) weechat_hook_signal_send ("irc_dcc",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
         if (filename)
             free (filename);
@@ -751,7 +776,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 
         (void) weechat_hook_signal_send ("irc_dcc",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
         if (filename)
             free (filename);
@@ -847,7 +872,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 
         (void) weechat_hook_signal_send ("irc_dcc",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
         if (filename)
             free (filename);
@@ -958,7 +983,7 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 
         (void) weechat_hook_signal_send ("irc_dcc",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
         free (dcc_args);
     }
@@ -971,24 +996,30 @@ irc_ctcp_recv_dcc (struct t_irc_server *server, const char *nick,
 void
 irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                struct t_irc_channel *channel, const char *address,
-               const char *nick, const char *remote_nick, char *arguments,
-               char *message)
+               const char *nick, const char *remote_nick,
+               const char *arguments, const char *message)
 {
-    char *pos_end, *pos_space, *pos_args;
+    char *dup_arguments, *ptr_args, *pos_end, *pos_space, *pos_args;
+    char *nick_color, *decoded_reply;
     const char *reply;
-    char *decoded_reply;
     struct t_irc_channel *ptr_channel;
     struct t_irc_nick *ptr_nick;
     int nick_is_me;
 
-    while (arguments && arguments[0])
+    dup_arguments = strdup (arguments);
+    if (!dup_arguments)
+        return;
+
+    ptr_args = dup_arguments;
+
+    while (ptr_args && ptr_args[0])
     {
-        pos_end = strrchr (arguments + 1, '\01');
+        pos_end = strrchr (ptr_args + 1, '\01');
         if (pos_end)
             pos_end[0] = '\0';
 
         pos_args = NULL;
-        pos_space = strchr (arguments + 1, ' ');
+        pos_space = strchr (ptr_args + 1, ' ');
         if (pos_space)
         {
             pos_space[0] = '\0';
@@ -1000,7 +1031,7 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
         }
 
         /* CTCP ACTION */
-        if (strcmp (arguments + 1, "ACTION") == 0)
+        if (weechat_strcasecmp (ptr_args + 1, "ACTION") == 0)
         {
             nick_is_me = (irc_server_strcasecmp (server, server->nick, nick) == 0);
             if (channel)
@@ -1014,6 +1045,12 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                 irc_channel_nick_speaking_time_remove_old (channel);
                 irc_channel_nick_speaking_time_add (server, channel, nick,
                                                     time (NULL));
+                if (ptr_nick)
+                    nick_color = strdup (ptr_nick->color);
+                else if (nick)
+                    nick_color = irc_nick_find_color (nick);
+                else
+                    nick_color = strdup (IRC_COLOR_CHAT_NICK);
                 weechat_printf_date_tags (
                     channel->buffer,
                     date,
@@ -1026,11 +1063,13 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                     "%s%s%s%s%s%s%s",
                     weechat_prefix ("action"),
                     irc_nick_mode_for_display (server, ptr_nick, 0),
-                    (ptr_nick) ? ptr_nick->color : ((nick) ? irc_nick_find_color (nick) : IRC_COLOR_CHAT_NICK),
+                    nick_color,
                     nick,
                     (pos_args) ? IRC_COLOR_RESET : "",
                     (pos_args) ? " " : "",
                     (pos_args) ? pos_args : "");
+                if (nick_color)
+                    free (nick_color);
             }
             else
             {
@@ -1073,16 +1112,16 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                         (pos_args) ? pos_args : "");
                     (void) weechat_hook_signal_send ("irc_pv",
                                                      WEECHAT_HOOK_SIGNAL_STRING,
-                                                     message);
+                                                     (void *)message);
                 }
             }
         }
         /* CTCP PING */
-        else if (strcmp (arguments + 1, "PING") == 0)
+        else if (weechat_strcasecmp (ptr_args + 1, "PING") == 0)
         {
-            reply = irc_ctcp_get_reply (server, arguments + 1);
+            reply = irc_ctcp_get_reply (server, ptr_args + 1);
             irc_ctcp_display_request (server, date, command, channel, nick,
-                                      address, arguments + 1, pos_args, reply);
+                                      address, ptr_args + 1, pos_args, reply);
             if (!reply || reply[0])
             {
                 if (reply)
@@ -1091,30 +1130,30 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                     if (decoded_reply)
                     {
                         irc_ctcp_reply_to_nick (server, command, channel, nick,
-                                                arguments + 1, decoded_reply);
+                                                ptr_args + 1, decoded_reply);
                         free (decoded_reply);
                     }
                 }
                 else
                 {
                     irc_ctcp_reply_to_nick (server, command, channel, nick,
-                                            arguments + 1, pos_args);
+                                            ptr_args + 1, pos_args);
                 }
             }
         }
         /* CTCP DCC */
-        else if (strcmp (arguments + 1, "DCC") == 0)
+        else if (weechat_strcasecmp (ptr_args + 1, "DCC") == 0)
         {
             irc_ctcp_recv_dcc (server, nick, pos_args, message);
         }
         /* other CTCP */
         else
         {
-            reply = irc_ctcp_get_reply (server, arguments + 1);
+            reply = irc_ctcp_get_reply (server, ptr_args + 1);
             if (reply)
             {
                 irc_ctcp_display_request (server, date, command, channel, nick,
-                                          address, arguments + 1, pos_args,
+                                          address, ptr_args + 1, pos_args,
                                           reply);
 
                 if (reply[0])
@@ -1123,7 +1162,7 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                     if (decoded_reply)
                     {
                         irc_ctcp_reply_to_nick (server, command, channel, nick,
-                                                arguments + 1, decoded_reply);
+                                                ptr_args + 1, decoded_reply);
                         free (decoded_reply);
                     }
                 }
@@ -1144,7 +1183,7 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
                         nick,
                         IRC_COLOR_RESET,
                         IRC_COLOR_CHAT_CHANNEL,
-                        arguments + 1,
+                        ptr_args + 1,
                         (pos_args) ? IRC_COLOR_RESET : "",
                         (pos_args) ? " " : "",
                         (pos_args) ? pos_args : "");
@@ -1154,14 +1193,10 @@ irc_ctcp_recv (struct t_irc_server *server, time_t date, const char *command,
 
         (void) weechat_hook_signal_send ("irc_ctcp",
                                          WEECHAT_HOOK_SIGNAL_STRING,
-                                         message);
+                                         (void *)message);
 
-        if (pos_space)
-            pos_space[0] = ' ';
-
-        if (pos_end)
-            pos_end[0] = '\01';
-
-        arguments = (pos_end) ? pos_end + 1 : NULL;
+        ptr_args = (pos_end) ? pos_end + 1 : NULL;
     }
+
+    free (dup_arguments);
 }
