@@ -1,7 +1,7 @@
 /*
  * irc-input.c - input data management for IRC buffers
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -82,7 +82,7 @@ irc_input_user_message_display (struct t_gui_buffer *buffer, int action,
         if (action)
         {
             snprintf (str_tags, sizeof (str_tags),
-                      "irc_action,notify_none,no_highlight");
+                      "irc_action,self_msg,notify_none,no_highlight");
         }
         else
         {
@@ -90,7 +90,7 @@ irc_input_user_message_display (struct t_gui_buffer *buffer, int action,
                 weechat_config_color (
                     weechat_config_get ("weechat.color.chat_nick_self")));
             snprintf (str_tags, sizeof (str_tags),
-                      "notify_none,no_highlight,prefix_nick_%s",
+                      "notify_none,self_msg,no_highlight,prefix_nick_%s",
                       (str_color) ? str_color : "default");
             if (str_color)
                 free (str_color);
@@ -98,8 +98,9 @@ irc_input_user_message_display (struct t_gui_buffer *buffer, int action,
         ptr_text = (text_decoded) ? text_decoded : ((text2) ? text2 : text);
         if (action)
         {
-            weechat_printf_tags (
+            weechat_printf_date_tags (
                 buffer,
+                0,
                 irc_protocol_tags (
                     "privmsg", str_tags,
                     (ptr_nick) ? ptr_nick->name : ptr_server->nick,
@@ -114,8 +115,9 @@ irc_input_user_message_display (struct t_gui_buffer *buffer, int action,
         }
         else
         {
-            weechat_printf_tags (
+            weechat_printf_date_tags (
                 buffer,
+                0,
                 irc_protocol_tags (
                     "privmsg", str_tags,
                     (ptr_nick) ? ptr_nick->name : ptr_server->nick,
@@ -137,7 +139,8 @@ irc_input_user_message_display (struct t_gui_buffer *buffer, int action,
 }
 
 /*
- * Sends a PRIVMSG message, and split it if message size is > 512 bytes.
+ * Sends a PRIVMSG message, and split it if message size is > 512 bytes
+ * (by default).
  *
  * Warning: this function makes temporary changes in "message".
  */
@@ -189,7 +192,8 @@ irc_input_send_user_message (struct t_gui_buffer *buffer, int flags,
  */
 
 int
-irc_input_data (struct t_gui_buffer *buffer, const char *input_data, int flags)
+irc_input_data (struct t_gui_buffer *buffer, const char *input_data, int flags,
+                int force_user_message)
 {
     const char *ptr_data;
     char *data_with_colors, *msg;
@@ -200,6 +204,8 @@ irc_input_data (struct t_gui_buffer *buffer, const char *input_data, int flags)
     {
         if (weechat_strcasecmp (input_data, "q") == 0)
             weechat_buffer_close (buffer);
+        else
+            irc_raw_filter_options (input_data);
     }
     else
     {
@@ -207,7 +213,8 @@ irc_input_data (struct t_gui_buffer *buffer, const char *input_data, int flags)
          * if send unknown commands is enabled and that input data is a
          * command, then send this command to IRC server
          */
-        if (weechat_config_boolean (irc_config_network_send_unknown_commands)
+        if (!force_user_message
+            && weechat_config_boolean (irc_config_network_send_unknown_commands)
             && !weechat_string_input_for_buffer (input_data))
         {
             if (ptr_server)
@@ -220,9 +227,13 @@ irc_input_data (struct t_gui_buffer *buffer, const char *input_data, int flags)
 
         if (ptr_channel)
         {
-            ptr_data = weechat_string_input_for_buffer (input_data);
-            if (!ptr_data)
-                ptr_data = input_data;
+            ptr_data = input_data;
+            if (!force_user_message)
+            {
+                ptr_data = weechat_string_input_for_buffer (input_data);
+                if (!ptr_data)
+                    ptr_data = input_data;
+            }
             data_with_colors = irc_color_encode (
                 ptr_data,
                 weechat_config_boolean (irc_config_network_colors_send));
@@ -253,13 +264,16 @@ irc_input_data (struct t_gui_buffer *buffer, const char *input_data, int flags)
  */
 
 int
-irc_input_data_cb (void *data, struct t_gui_buffer *buffer,
+irc_input_data_cb (const void *pointer, void *data,
+                   struct t_gui_buffer *buffer,
                    const char *input_data)
 {
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
-    return irc_input_data (buffer, input_data, IRC_SERVER_SEND_OUTQ_PRIO_HIGH);
+    return irc_input_data (buffer, input_data,
+                           IRC_SERVER_SEND_OUTQ_PRIO_HIGH, 0);
 }
 
 /*
@@ -268,28 +282,32 @@ irc_input_data_cb (void *data, struct t_gui_buffer *buffer,
  * This signal can be used by other plugins/scripts, it simulates input or
  * command from user on an IRC buffer (it is used for example by Relay plugin).
  *
- * Format of signal_data (string) is "server;channel;flags;tags;text"
+ * Format of signal_data (string) is "server;channel;options;tags;text"
  *   server: server name (required)
  *   channel: channel name (optional)
- *   flags: flags for irc_server_sendf() (optional)
+ *   options: comma-separated list of options (optional):
+ *     "priority_high": send with high priority (default)
+ *     "priority_low": send with low priority
+ *     "user_message": force user message (don't execute a command)
  *   tags: tags for irc_server_sendf() (optional)
  *   text: text or command (required).
  */
 
 int
-irc_input_send_cb (void *data, const char *signal,
+irc_input_send_cb (const void *pointer, void *data,
+                   const char *signal,
                    const char *type_data, void *signal_data)
 {
     const char *ptr_string, *ptr_message;
-    char *pos_semicol1, *pos_semicol2, *pos_semicol3, *pos_semicol4, *error;
-    char *server, *channel, *flags, *tags;
-    long flags_value;
-    char *data_with_colors;
+    char *pos_semicol1, *pos_semicol2, *pos_semicol3, *pos_semicol4;
+    char *server, *channel, *options, *tags, *data_with_colors, **list_options;
+    int i, num_options, flags, force_user_message;
     struct t_irc_server *ptr_server;
     struct t_irc_channel *ptr_channel;
     struct t_gui_buffer *ptr_buffer;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) signal;
     (void) type_data;
@@ -298,7 +316,9 @@ irc_input_send_cb (void *data, const char *signal,
 
     server = NULL;
     channel = NULL;
-    flags = NULL;
+    options = NULL;
+    flags = IRC_SERVER_SEND_OUTQ_PRIO_HIGH;
+    force_user_message = 0;
     tags = NULL;
     ptr_message = NULL;
     ptr_server = NULL;
@@ -324,8 +344,8 @@ irc_input_send_cb (void *data, const char *signal,
             {
                 if (pos_semicol3 > pos_semicol2 + 1)
                 {
-                    flags = weechat_strndup (pos_semicol2 + 1,
-                                             pos_semicol3 - pos_semicol2 - 1);
+                    options = weechat_strndup (pos_semicol2 + 1,
+                                               pos_semicol3 - pos_semicol2 - 1);
                 }
                 pos_semicol4 = strchr (pos_semicol3 + 1, ';');
                 if (pos_semicol4)
@@ -341,13 +361,30 @@ irc_input_send_cb (void *data, const char *signal,
         }
     }
 
-    flags_value = IRC_SERVER_SEND_OUTQ_PRIO_HIGH;
-    if (flags)
+    if (options && options[0])
     {
-        error = NULL;
-        flags_value = strtol (flags, &error, 10);
-        if (flags_value < 0)
-            flags_value = IRC_SERVER_SEND_OUTQ_PRIO_HIGH;
+        list_options = weechat_string_split (
+            options,
+            ",",
+            NULL,
+            WEECHAT_STRING_SPLIT_STRIP_LEFT
+            | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+            | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+            0,
+            &num_options);
+        if (list_options)
+        {
+            for (i = 0; i < num_options; i++)
+            {
+                if (strcmp (list_options[i], "priority_high") == 0)
+                    flags = IRC_SERVER_SEND_OUTQ_PRIO_HIGH;
+                else if (strcmp (list_options[i], "priority_low") == 0)
+                    flags = IRC_SERVER_SEND_OUTQ_PRIO_LOW;
+                else if (strcmp (list_options[i], "user_message") == 0)
+                    force_user_message = 1;
+            }
+            weechat_string_free_split (list_options);
+        }
     }
 
     if (server && ptr_message)
@@ -367,10 +404,11 @@ irc_input_send_cb (void *data, const char *signal,
             irc_server_set_send_default_tags (tags);
 
             /* send text to buffer, or execute command */
-            if (weechat_string_input_for_buffer (ptr_message))
+            if (force_user_message
+                || weechat_string_input_for_buffer (ptr_message))
             {
                 /* text as input */
-                irc_input_data (ptr_buffer, ptr_message, flags_value);
+                irc_input_data (ptr_buffer, ptr_message, flags, 1);
             }
             else
             {
@@ -394,8 +432,8 @@ irc_input_send_cb (void *data, const char *signal,
         free (server);
     if (channel)
         free (channel);
-    if (flags)
-        free (flags);
+    if (options)
+        free (options);
     if (tags)
         free (tags);
 
