@@ -1,7 +1,7 @@
 /*
  * irc-message.c - functions for IRC messages
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,35 +16,65 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
+#include <limits.h>
 #include <stdio.h>
 #include <string.h>
 
 #include "../weechat-plugin.h"
 #include "irc.h"
-#include "irc-server.h"
 #include "irc-channel.h"
+#include "irc-color.h"
+#include "irc-config.h"
+#include "irc-ignore.h"
+#include "irc-server.h"
+#include "irc-tag.h"
 
 
 /*
- * Parses an IRC message and returns pointers to:
- *   - tags
- *   - message without tags
- *   - host
- *   - command
- *   - channel
- *   - target nick
- *   - arguments (if any)
+ * Parses an IRC message and returns:
+ *   - tags (string)
+ *   - message without tags (string)
+ *   - nick (string)
+ *   - host (string)
+ *   - command (string)
+ *   - channel (string)
+ *   - arguments (string)
+ *   - text (string)
+ *   - pos_command (integer: command index in message)
+ *   - pos_arguments (integer: arguments index in message)
+ *   - pos_channel (integer: channel index in message)
+ *   - pos_text (integer: text index in message)
+ *
+ * Example:
+ *   @time=2015-06-27T16:40:35.000Z :nick!user@host PRIVMSG #weechat :hello!
+ *
+ * Result:
+ *               tags: "time=2015-06-27T16:40:35.000Z"
+ *   msg_without_tags: ":nick!user@host PRIVMSG #weechat :hello!"
+ *               nick: "nick"
+ *               user: "user"
+ *               host: "nick!user@host"
+ *            command: "PRIVMSG"
+ *            channel: "#weechat"
+ *          arguments: "#weechat :hello!"
+ *               text: "hello!"
+ *        pos_command: 47
+ *      pos_arguments: 55
+ *        pos_channel: 55
+ *           pos_text: 65
  */
 
 void
 irc_message_parse (struct t_irc_server *server, const char *message,
                    char **tags, char **message_without_tags, char **nick,
-                   char **host, char **command, char **channel,
-                   char **arguments)
+                   char **user, char **host, char **command, char **channel,
+                   char **arguments, char **text,
+                   int *pos_command, int *pos_arguments, int *pos_channel,
+                   int *pos_text)
 {
     const char *ptr_message, *pos, *pos2, *pos3, *pos4;
 
@@ -54,6 +84,8 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         *message_without_tags = NULL;
     if (nick)
         *nick = NULL;
+    if (user)
+        *user = NULL;
     if (host)
         *host = NULL;
     if (command)
@@ -62,6 +94,16 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         *channel = NULL;
     if (arguments)
         *arguments = NULL;
+    if (text)
+        *text = NULL;
+    if (pos_command)
+        *pos_command = -1;
+    if (pos_arguments)
+        *pos_arguments = -1;
+    if (pos_channel)
+        *pos_channel = -1;
+    if (pos_text)
+        *pos_text = -1;
 
     if (!message)
         return;
@@ -71,21 +113,24 @@ irc_message_parse (struct t_irc_server *server, const char *message,
     /*
      * we will use this message as example:
      *
-     *   @tags :FlashCode!n=flash@host.com PRIVMSG #channel :hello!
+     *   @time=2015-06-27T16:40:35.000Z :nick!user@host PRIVMSG #weechat :hello!
      */
 
     if (ptr_message[0] == '@')
     {
         /*
-         * read tags (they are optional and enabled only if client enabled
-         * a server capability, see:
-         * http://ircv3.atheme.org/specification/message-tags-3.2)
+         * Read tags: they are optional and enabled only if client enabled
+         * a server capability.
+         * See: https://ircv3.net/specs/core/message-tags-3.2.html
          */
         pos = strchr (ptr_message, ' ');
         if (pos)
         {
             if (tags)
-                *tags = weechat_strndup (message + 1, pos - (message + 1));
+            {
+                *tags = weechat_strndup (ptr_message + 1,
+                                         pos - (ptr_message + 1));
+            }
             ptr_message = pos + 1;
             while (ptr_message[0] == ' ')
             {
@@ -97,7 +142,7 @@ irc_message_parse (struct t_irc_server *server, const char *message,
     if (message_without_tags)
         *message_without_tags = strdup (ptr_message);
 
-    /* now we have: ptr_message --> ":FlashCode!n=flash@host.com PRIVMSG #channel :hello!" */
+    /* now we have: ptr_message --> ":nick!user@host PRIVMSG #weechat :hello!" */
     if (ptr_message[0] == ':')
     {
         /* read host/nick */
@@ -107,6 +152,11 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         /* if the prefix doesn't contain a '!', split the nick at '@' */
         if (!pos2 || (pos && pos2 > pos))
             pos2 = pos3;
+        if (pos2 && pos3 && (pos3 > pos2))
+        {
+            if (user)
+                *user = weechat_strndup (pos2 + 1, pos3 - pos2 - 1);
+        }
         if (pos2 && (!pos || pos > pos2))
         {
             if (nick)
@@ -135,7 +185,7 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         }
     }
 
-    /* now we have: ptr_message --> "PRIVMSG #channel :hello!" */
+    /* now we have: ptr_message --> "PRIVMSG #weechat :hello!" */
     if (ptr_message[0])
     {
         pos = strchr (ptr_message, ' ');
@@ -143,21 +193,32 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         {
             if (command)
                 *command = weechat_strndup (ptr_message, pos - ptr_message);
+            if (pos_command)
+                *pos_command = ptr_message - message;
             pos++;
             while (pos[0] == ' ')
             {
                 pos++;
             }
-            /* now we have: pos --> "#channel :hello!" */
+            /* now we have: pos --> "#weechat :hello!" */
             if (arguments)
                 *arguments = strdup (pos);
+            if (pos_arguments)
+                *pos_arguments = pos - message;
             if ((pos[0] == ':')
                 && ((strncmp (ptr_message, "JOIN ", 5) == 0)
                     || (strncmp (ptr_message, "PART ", 5) == 0)))
             {
                 pos++;
             }
-            if (pos[0] != ':')
+            if (pos[0] == ':')
+            {
+                if (text)
+                    *text = strdup (pos + 1);
+                if (pos_text)
+                    *pos_text = pos - message + 1;
+            }
+            else
             {
                 if (irc_channel_is_channel (server, pos))
                 {
@@ -169,6 +230,21 @@ irc_message_parse (struct t_irc_server *server, const char *message,
                         else
                             *channel = strdup (pos);
                     }
+                    if (pos_channel)
+                        *pos_channel = pos - message;
+                    if (pos2)
+                    {
+                        while (pos2[0] == ' ')
+                        {
+                            pos2++;
+                        }
+                        if (pos2[0] == ':')
+                            pos2++;
+                        if (text)
+                            *text = strdup (pos2);
+                        if (pos_text)
+                            *pos_text = pos2 - message;
+                    }
                 }
                 else
                 {
@@ -198,10 +274,42 @@ irc_message_parse (struct t_irc_server *server, const char *message,
                                 else
                                     *channel = strdup (pos2);
                             }
+                            if (pos_channel)
+                                *pos_channel = pos2 - message;
+                            if (pos4)
+                            {
+                                while (pos4[0] == ' ')
+                                {
+                                    pos4++;
+                                }
+                                if (pos4[0] == ':')
+                                    pos4++;
+                                if (text)
+                                    *text = strdup (pos4);
+                                if (pos_text)
+                                    *pos_text = pos4 - message;
+                            }
                         }
-                        else if (channel && !*channel)
+                        else
                         {
-                            *channel = weechat_strndup (pos, pos3 - pos);
+                            if (channel)
+                                *channel = weechat_strndup (pos, pos3 - pos);
+                            if (pos_channel)
+                                *pos_channel = pos - message;
+                            pos4 = strchr (pos3, ' ');
+                            if (pos4)
+                            {
+                                while (pos4[0] == ' ')
+                                {
+                                    pos4++;
+                                }
+                                if (pos4[0] == ':')
+                                    pos4++;
+                                if (text)
+                                    *text = strdup (pos4);
+                                if (pos_text)
+                                    *pos_text = pos4 - message;
+                            }
                         }
                     }
                 }
@@ -211,6 +319,8 @@ irc_message_parse (struct t_irc_server *server, const char *message,
         {
             if (command)
                 *command = strdup (ptr_message);
+            if (pos_command)
+                *pos_command = ptr_message - message;
         }
     }
 }
@@ -218,12 +328,18 @@ irc_message_parse (struct t_irc_server *server, const char *message,
 /*
  * Parses an IRC message and returns hashtable with keys:
  *   - tags
+ *   - tag_xxx (one key per tag, with unescaped value)
  *   - message_without_tags
  *   - nick
  *   - host
  *   - command
  *   - channel
  *   - arguments
+ *   - text
+ *   - pos_command
+ *   - pos_arguments
+ *   - pos_channel
+ *   - pos_text
  *
  * Note: hashtable must be freed after use.
  */
@@ -232,28 +348,32 @@ struct t_hashtable *
 irc_message_parse_to_hashtable (struct t_irc_server *server,
                                 const char *message)
 {
-    char *tags,*message_without_tags, *nick, *host, *command, *channel;
-    char *arguments;
+    char *tags, *message_without_tags, *nick, *user, *host, *command, *channel;
+    char *arguments, *text, str_pos[32];
     char empty_str[1] = { '\0' };
+    int pos_command, pos_arguments, pos_channel, pos_text;
     struct t_hashtable *hashtable;
 
     irc_message_parse (server, message, &tags, &message_without_tags, &nick,
-                       &host, &command, &channel, &arguments);
+                       &user, &host, &command, &channel, &arguments, &text,
+                       &pos_command, &pos_arguments, &pos_channel, &pos_text);
 
     hashtable = weechat_hashtable_new (32,
                                        WEECHAT_HASHTABLE_STRING,
                                        WEECHAT_HASHTABLE_STRING,
-                                       NULL,
-                                       NULL);
+                                       NULL, NULL);
     if (!hashtable)
         return NULL;
 
     weechat_hashtable_set (hashtable, "tags",
                            (tags) ? tags : empty_str);
+    irc_tag_parse (tags, hashtable, "tag_");
     weechat_hashtable_set (hashtable, "message_without_tags",
                            (message_without_tags) ? message_without_tags : empty_str);
     weechat_hashtable_set (hashtable, "nick",
                            (nick) ? nick : empty_str);
+    weechat_hashtable_set (hashtable, "user",
+                           (user) ? user : empty_str);
     weechat_hashtable_set (hashtable, "host",
                            (host) ? host : empty_str);
     weechat_hashtable_set (hashtable, "command",
@@ -262,6 +382,16 @@ irc_message_parse_to_hashtable (struct t_irc_server *server,
                            (channel) ? channel : empty_str);
     weechat_hashtable_set (hashtable, "arguments",
                            (arguments) ? arguments : empty_str);
+    weechat_hashtable_set (hashtable, "text",
+                           (text) ? text : empty_str);
+    snprintf (str_pos, sizeof (str_pos), "%d", pos_command);
+    weechat_hashtable_set (hashtable, "pos_command", str_pos);
+    snprintf (str_pos, sizeof (str_pos), "%d", pos_arguments);
+    weechat_hashtable_set (hashtable, "pos_arguments", str_pos);
+    snprintf (str_pos, sizeof (str_pos), "%d", pos_channel);
+    weechat_hashtable_set (hashtable, "pos_channel", str_pos);
+    snprintf (str_pos, sizeof (str_pos), "%d", pos_text);
+    weechat_hashtable_set (hashtable, "pos_text", str_pos);
 
     if (tags)
         free (tags);
@@ -269,6 +399,8 @@ irc_message_parse_to_hashtable (struct t_irc_server *server,
         free (message_without_tags);
     if (nick)
         free (nick);
+    if (user)
+        free (user);
     if (host)
         free (host);
     if (command)
@@ -277,10 +409,48 @@ irc_message_parse_to_hashtable (struct t_irc_server *server,
         free (channel);
     if (arguments)
         free (arguments);
+    if (text)
+        free (text);
 
     return hashtable;
 }
 
+/*
+ * Encodes/decodes an IRC message using a charset.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+irc_message_convert_charset (const char *message, int pos_start,
+                             const char *modifier, const char *modifier_data)
+{
+    char *text, *msg_result;
+    int length;
+
+    text = weechat_hook_modifier_exec (modifier, modifier_data,
+                                       message + pos_start);
+    if (!text)
+        return NULL;
+
+    length = pos_start + strlen (text) + 1;
+    msg_result = malloc (length);
+    if (msg_result)
+    {
+        msg_result[0] = '\0';
+        if (pos_start > 0)
+        {
+            memcpy (msg_result, message, pos_start);
+            msg_result[pos_start] = '\0';
+        }
+        strcat (msg_result, text);
+    }
+
+    free (text);
+
+    return msg_result;
+}
+
 /*
  * Gets nick from host in an IRC message.
  */
@@ -341,6 +511,9 @@ irc_message_get_address_from_host (const char *host)
     char host2[256], *pos_space, *pos;
     const char *ptr_host;
 
+    if (!host)
+        return NULL;
+
     address[0] = '\0';
     if (host)
     {
@@ -370,6 +543,56 @@ irc_message_get_address_from_host (const char *host)
     return address;
 }
 
+/*
+ * Checks if a raw message is ignored (nick ignored on this server/channel).
+ *
+ * Returns:
+ *   0: message not ignored (displayed)
+ *   1: message ignored (not displayed)
+ */
+
+int
+irc_message_ignored (struct t_irc_server *server, const char *message)
+{
+    char *nick, *host, *host_no_color, *channel;
+    struct t_irc_channel *ptr_channel;
+    int ignored;
+
+    if (!server || !message)
+        return 0;
+
+    /* parse raw message */
+    irc_message_parse (server, message,
+                       NULL, NULL, &nick, NULL, &host,
+                       NULL, &channel, NULL,
+                       NULL, NULL, NULL,
+                       NULL, NULL);
+
+    /* remove colors from host */
+    host_no_color = (host) ? irc_color_decode (host, 0) : NULL;
+
+    /* search channel */
+    ptr_channel = (channel) ? irc_channel_search (server, channel) : NULL;
+
+    /* check if message is ignored or not */
+    ignored = irc_ignore_check (
+        server,
+        (ptr_channel) ? ptr_channel->name : channel,
+        nick,
+        host_no_color);
+
+    if (nick)
+        free (nick);
+    if (host)
+        free (host);
+    if (host_no_color)
+        free (host_no_color);
+    if (channel)
+        free (channel);
+
+    return ignored;
+}
+
 /*
  * Replaces special IRC vars ($nick, $channel, $server) in a string.
  *
@@ -501,15 +724,16 @@ irc_message_split_string (struct t_hashtable *hashtable,
                           const char *arguments,
                           const char *suffix,
                           const char delimiter,
-                          int max_length_host)
+                          int max_length_nick_user_host,
+                          int max_length)
 {
     const char *pos, *pos_max, *pos_next, *pos_last_delim;
-    char message[1024], *dup_arguments;
-    int max_length, number;
+    char message[8192], *dup_arguments;
+    int number;
 
-    max_length = 510;
-    if (max_length_host >= 0)
-        max_length -= max_length_host;
+    max_length -= 2;  /* by default: 512 - 2 = 510 bytes */
+    if (max_length_nick_user_host >= 0)
+        max_length -= max_length_nick_user_host;
     else
         max_length -= (host) ? strlen (host) + 1 : 0;
     max_length -= strlen (command) + 1;
@@ -555,7 +779,7 @@ irc_message_split_string (struct t_hashtable *hashtable,
         pos = arguments;
         pos_max = pos + max_length;
         pos_last_delim = NULL;
-        while (pos && pos[0])
+        while (pos[0])
         {
             if (pos[0] == delimiter)
                 pos_last_delim = pos;
@@ -589,6 +813,62 @@ irc_message_split_string (struct t_hashtable *hashtable,
     return 1;
 }
 
+/*
+ * Splits a AUTHENTICATE message: in 400-byte chunks, and adds an extra
+ * message "AUTHENTICATE +" if the last message has exactly 400 bytes.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+irc_message_split_authenticate (struct t_hashtable *hashtable,
+                                const char *tags, const char *host,
+                                const char *command, const char *arguments)
+{
+    int number, length;
+    char message[8192], *args;
+    const char *ptr_args;
+
+    number = 1;
+
+    length = 0;
+    ptr_args = arguments;
+    while (ptr_args && ptr_args[0])
+    {
+        length = strlen (ptr_args);
+        if (length == 0)
+            break;
+        if (length > 400)
+            length = 400;
+        args = weechat_strndup (ptr_args, length);
+        if (!args)
+            return 0;
+        snprintf (message, sizeof (message), "%s%s%s %s",
+                  (host) ? host : "",
+                  (host) ? " " : "",
+                  command,
+                  args);
+        irc_message_split_add (hashtable, number, tags, message, args);
+        free (args);
+        number++;
+        ptr_args += length;
+    }
+
+    if ((length == 0) || (length == 400))
+    {
+        snprintf (message, sizeof (message), "%s%s%s +",
+                  (host) ? host : "",
+                  (host) ? " " : "",
+                  command);
+        irc_message_split_add (hashtable, number, tags, message, "+");
+        number++;
+    }
+
+    return 1;
+}
+
 /*
  * Splits a JOIN message, taking care of keeping channel keys with channel
  * names.
@@ -601,12 +881,15 @@ irc_message_split_string (struct t_hashtable *hashtable,
 int
 irc_message_split_join (struct t_hashtable *hashtable,
                         const char *tags, const char *host,
-                        const char *arguments)
+                        const char *arguments,
+                        int max_length)
 {
     int number, channels_count, keys_count, length, length_no_channel;
     int length_to_add, index_channel;
     char **channels, **keys, *pos, *str;
-    char msg_to_send[2048], keys_to_add[2048];
+    char msg_to_send[16384], keys_to_add[16384];
+
+    max_length -= 2;  /* by default: 512 - 2 = 510 bytes */
 
     number = 1;
 
@@ -621,18 +904,30 @@ irc_message_split_join (struct t_hashtable *hashtable,
         str = weechat_strndup (arguments, pos - arguments);
         if (!str)
             return 0;
-        channels = weechat_string_split (str, ",", 0, 0, &channels_count);
+        channels = weechat_string_split (str, ",", NULL,
+                                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                         0, &channels_count);
         free (str);
         while (pos[0] == ' ')
         {
             pos++;
         }
         if (pos[0])
-            keys = weechat_string_split (pos, ",", 0, 0, &keys_count);
+            keys = weechat_string_split (pos, ",", NULL,
+                                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                         0, &keys_count);
     }
     else
     {
-        channels = weechat_string_split (arguments, ",", 0, 0, &channels_count);
+        channels = weechat_string_split (arguments, ",", NULL,
+                                         WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                         | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                         | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                         0, &channels_count);
     }
 
     snprintf (msg_to_send, sizeof (msg_to_send), "%s%sJOIN",
@@ -647,7 +942,8 @@ irc_message_split_join (struct t_hashtable *hashtable,
         length_to_add = 1 + strlen (channels[index_channel]);
         if (index_channel < keys_count)
             length_to_add += 1 + strlen (keys[index_channel]);
-        if ((length + length_to_add < 510) || (length == length_no_channel))
+        if ((length + length_to_add < max_length)
+            || (length == length_no_channel))
         {
             if (length + length_to_add < (int)sizeof (msg_to_send))
             {
@@ -710,11 +1006,14 @@ irc_message_split_join (struct t_hashtable *hashtable,
 
 int
 irc_message_split_privmsg_notice (struct t_hashtable *hashtable,
-                                  char *tags, char *host, char *command,
-                                  char *target, char *arguments,
-                                  int max_length_host)
+                                  const char *tags, const char *host,
+                                  const char *command, const char *target,
+                                  const char *arguments,
+                                  int max_length_nick_user_host,
+                                  int max_length)
 {
-    char prefix[512], suffix[2], *pos, saved_char;
+    char *arguments2, prefix[4096], suffix[2], *pos, saved_char;
+    const char *ptr_args;
     int length, rc;
 
     /*
@@ -725,23 +1024,29 @@ irc_message_split_privmsg_notice (struct t_hashtable *hashtable,
      *   :nick!user@host.com PRIVMSG #channel :hello world!
      */
 
+    arguments2 = strdup (arguments);
+    if (!arguments2)
+        return 0;
+
+    ptr_args = arguments2;
+
     /* for CTCP, prefix will be ":\01xxxx " and suffix "\01" */
     prefix[0] = '\0';
     suffix[0] = '\0';
-    length = strlen (arguments);
-    if ((arguments[0] == '\01')
-        && (arguments[length - 1] == '\01'))
+    length = strlen (arguments2);
+    if ((arguments2[0] == '\01')
+        && (arguments2[length - 1] == '\01'))
     {
-        pos = strchr (arguments, ' ');
+        pos = strchr (arguments2, ' ');
         if (pos)
         {
             pos++;
             saved_char = pos[0];
             pos[0] = '\0';
-            snprintf (prefix, sizeof (prefix), ":%s", arguments);
+            snprintf (prefix, sizeof (prefix), ":%s", arguments2);
             pos[0] = saved_char;
-            arguments[length - 1] = '\0';
-            arguments = pos;
+            arguments2[length - 1] = '\0';
+            ptr_args = pos;
             suffix[0] = '\01';
             suffix[1] = '\0';
         }
@@ -750,8 +1055,10 @@ irc_message_split_privmsg_notice (struct t_hashtable *hashtable,
         strcpy (prefix, ":");
 
     rc = irc_message_split_string (hashtable, tags, host, command, target,
-                                   prefix, arguments, suffix,
-                                   ' ', max_length_host);
+                                   prefix, ptr_args, suffix,
+                                   ' ', max_length_nick_user_host, max_length);
+
+    free (arguments2);
 
     return rc;
 }
@@ -766,10 +1073,11 @@ irc_message_split_privmsg_notice (struct t_hashtable *hashtable,
 
 int
 irc_message_split_005 (struct t_hashtable *hashtable,
-                       char *tags, char *host, char *command, char *target,
-                       char *arguments)
+                       const char *tags, const char *host, const char *command,
+                       const char *target, const char *arguments,
+                       int max_length)
 {
-    char *pos, suffix[512];
+    char *pos, suffix[4096];
 
     /*
      * 005 message looks like:
@@ -789,7 +1097,8 @@ irc_message_split_005 (struct t_hashtable *hashtable,
     }
 
     return irc_message_split_string (hashtable, tags, host, command, target,
-                                     NULL, arguments, suffix, ' ', -1);
+                                     NULL, arguments, suffix, ' ', -1,
+                                     max_length);
 }
 
 /*
@@ -799,6 +1108,10 @@ irc_message_split_005 (struct t_hashtable *hashtable,
  * "\r\n", so full size is 512 bytes (the user data does not include the
  * optional tags before the host).
  *
+ * The 512 max length is the default (recommended) and can be changed with the
+ * server option called "split_msg_max_length" (0 to disable completely the
+ * split).
+ *
  * The split takes care about type of message to do a split at best place in
  * message.
  *
@@ -820,28 +1133,47 @@ struct t_hashtable *
 irc_message_split (struct t_irc_server *server, const char *message)
 {
     struct t_hashtable *hashtable;
-    char **argv, **argv_eol, *tags, *host, *command, *arguments, target[512];
+    char **argv, **argv_eol, *tags, *host, *command, *arguments, target[4096];
     char *pos, monitor_action[3];
-    int split_ok, argc, index_args, max_length_nick, max_length_host;
+    int split_ok, argc, index_args, max_length_nick, max_length_user;
+    int max_length_host, max_length_nick_user_host, split_msg_max_length;
 
     split_ok = 0;
     tags = NULL;
     host = NULL;
     command = NULL;
     arguments = NULL;
-    index_args = 0;
     argv = NULL;
     argv_eol = NULL;
 
+    if (server)
+    {
+        split_msg_max_length = IRC_SERVER_OPTION_INTEGER(
+            server, IRC_SERVER_OPTION_SPLIT_MSG_MAX_LENGTH);
+
+        /*
+         * split disabled? use a very high max_length so the message should
+         * never be split
+         */
+        if (split_msg_max_length == 0)
+            split_msg_max_length = INT_MAX - 16;
+    }
+    else
+    {
+        split_msg_max_length = 512;  /* max length by default */
+    }
+
     /* debug message */
     if (weechat_irc_plugin->debug >= 2)
-        weechat_printf (NULL, "irc_message_split: message='%s'", message);
+    {
+        weechat_printf (NULL, "irc_message_split: message='%s', max length=%d",
+                        message, split_msg_max_length);
+    }
 
     hashtable = weechat_hashtable_new (32,
                                        WEECHAT_HASHTABLE_STRING,
                                        WEECHAT_HASHTABLE_STRING,
-                                       NULL,
-                                       NULL);
+                                       NULL, NULL);
     if (!hashtable)
         return NULL;
 
@@ -858,8 +1190,16 @@ irc_message_split (struct t_irc_server *server, const char *message)
         }
     }
 
-    argv = weechat_string_split (message, " ", 0, 0, &argc);
-    argv_eol = weechat_string_split (message, " ", 2, 0, NULL);
+    argv = weechat_string_split (message, " ", NULL,
+                                 WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                 | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                 | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                 0, &argc);
+    argv_eol = weechat_string_split (message, " ", NULL,
+                                     WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                     | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS
+                                     | WEECHAT_STRING_SPLIT_KEEP_EOL,
+                                     0, NULL);
 
     if (argc < 2)
         goto end;
@@ -882,13 +1222,26 @@ irc_message_split (struct t_irc_server *server, const char *message)
 
     max_length_nick = (server && (server->nick_max_length > 0)) ?
         server->nick_max_length : 16;
-    max_length_host = 1 + /* ":"  */
-        max_length_nick + /* nick */
-        1 +               /* "!"  */
-        63 +              /* host */
-        1;                /* " "  */
+    max_length_user = (server && (server->user_max_length > 0)) ?
+        server->user_max_length : 10;
+    max_length_host = (server && (server->host_max_length > 0)) ?
+        server->host_max_length : 63;
 
-    if ((weechat_strcasecmp (command, "ison") == 0)
+    max_length_nick_user_host = 1 +  /* ":"  */
+        max_length_nick +            /* nick */
+        1 +                          /* "!"  */
+        max_length_user +            /* user */
+        1 +                          /* @    */
+        max_length_host +            /* host */
+        1;                           /* " "  */
+
+    if (weechat_strcasecmp (command, "authenticate") == 0)
+    {
+        /* AUTHENTICATE UzXAmVffxuzFy77XWBGwABBQAgdinelBrKZaR3wE7nsIETuTVY= */
+        split_ok = irc_message_split_authenticate (
+            hashtable, tags, host, command, arguments);
+    }
+    else if ((weechat_strcasecmp (command, "ison") == 0)
         || (weechat_strcasecmp (command, "wallops") == 0))
     {
         /*
@@ -899,7 +1252,7 @@ irc_message_split (struct t_irc_server *server, const char *message)
             hashtable, tags, host, command, NULL, ":",
             (argv_eol[index_args][0] == ':') ?
             argv_eol[index_args] + 1 : argv_eol[index_args],
-            NULL, ' ', max_length_host);
+            NULL, ' ', max_length_nick_user_host, split_msg_max_length);
     }
     else if (weechat_strcasecmp (command, "monitor") == 0)
     {
@@ -914,7 +1267,8 @@ irc_message_split (struct t_irc_server *server, const char *message)
                       "%c ", argv_eol[index_args][0]);
             split_ok = irc_message_split_string (
                 hashtable, tags, host, command, NULL, monitor_action,
-                argv_eol[index_args] + 2, NULL, ',', max_length_host);
+                argv_eol[index_args] + 2, NULL, ',', max_length_nick_user_host,
+                split_msg_max_length);
         }
         else
         {
@@ -922,17 +1276,17 @@ irc_message_split (struct t_irc_server *server, const char *message)
                 hashtable, tags, host, command, NULL, ":",
                 (argv_eol[index_args][0] == ':') ?
                 argv_eol[index_args] + 1 : argv_eol[index_args],
-                NULL, ',', max_length_host);
+                NULL, ',', max_length_nick_user_host, split_msg_max_length);
         }
     }
     else if (weechat_strcasecmp (command, "join") == 0)
     {
         /* JOIN #channel1,#channel2,#channel3 key1,key2 */
-        if (strlen (message) > 510)
+        if ((int)strlen (message) > split_msg_max_length - 2)
         {
-            /* split join if it's more than 510 bytes */
+            /* split join if it's too long */
             split_ok = irc_message_split_join (hashtable, tags, host,
-                                               arguments);
+                                               arguments, split_msg_max_length);
         }
     }
     else if ((weechat_strcasecmp (command, "privmsg") == 0)
@@ -948,7 +1302,7 @@ irc_message_split (struct t_irc_server *server, const char *message)
                 hashtable, tags, host, command, argv[index_args],
                 (argv_eol[index_args + 1][0] == ':') ?
                 argv_eol[index_args + 1] + 1 : argv_eol[index_args + 1],
-                max_length_host);
+                max_length_nick_user_host, split_msg_max_length);
         }
     }
     else if (weechat_strcasecmp (command, "005") == 0)
@@ -959,7 +1313,8 @@ irc_message_split (struct t_irc_server *server, const char *message)
             split_ok = irc_message_split_005 (
                 hashtable, tags, host, command, argv[index_args],
                 (argv_eol[index_args + 1][0] == ':') ?
-                argv_eol[index_args + 1] + 1 : argv_eol[index_args + 1]);
+                argv_eol[index_args + 1] + 1 : argv_eol[index_args + 1],
+                split_msg_max_length);
         }
     }
     else if (weechat_strcasecmp (command, "353") == 0)
@@ -978,7 +1333,7 @@ irc_message_split (struct t_irc_server *server, const char *message)
                     hashtable, tags, host, command, target, ":",
                     (argv_eol[index_args + 2][0] == ':') ?
                     argv_eol[index_args + 2] + 1 : argv_eol[index_args + 2],
-                    NULL, ' ', -1);
+                    NULL, ' ', -1, split_msg_max_length);
             }
             else
             {
@@ -991,7 +1346,7 @@ irc_message_split (struct t_irc_server *server, const char *message)
                         hashtable, tags, host, command, target, ":",
                         (argv_eol[index_args + 3][0] == ':') ?
                         argv_eol[index_args + 3] + 1 : argv_eol[index_args + 3],
-                        NULL, ' ', -1);
+                        NULL, ' ', -1, split_msg_max_length);
                 }
             }
         }
@@ -1001,7 +1356,11 @@ end:
     if (!split_ok
         || (weechat_hashtable_get_integer (hashtable, "items_count") == 0))
     {
-        irc_message_split_add (hashtable, 1, tags, message, arguments);
+        irc_message_split_add (hashtable,
+                               (message) ? 1 : 0,
+                               tags,
+                               message,
+                               arguments);
     }
 
     if (tags)
