@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_IRC_MESSAGE_H
@@ -25,8 +25,9 @@ struct t_irc_channel;
 
 extern void irc_message_parse (struct t_irc_server *server, const char *message,
                                char **tags, char **message_without_tags,
-                               char **nick, char **host, char **command,
-                               char **channel, char **arguments, char **text,
+                               char **nick, char **user, char **host,
+                               char **command, char **channel,
+                               char **arguments, char **text,
                                int *pos_command, int *pos_arguments,
                                int *pos_channel, int *pos_text);
 extern struct t_hashtable *irc_message_parse_to_hashtable (struct t_irc_server *server,
@@ -37,6 +38,8 @@ extern char *irc_message_convert_charset (const char *message,
                                           const char *modifier_data);
 extern const char *irc_message_get_nick_from_host (const char *host);
 extern const char *irc_message_get_address_from_host (const char *host);
+extern int irc_message_ignored (struct t_irc_server *server,
+                                const char *message);
 extern char *irc_message_replace_vars (struct t_irc_server *server,
                                        const char *channel_name,
                                        const char *string);
