@@ -1,7 +1,7 @@
 /*
  * irc-nick.c - nick management for IRC plugin
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -62,195 +62,86 @@ irc_nick_valid (struct t_irc_channel *channel, struct t_irc_nick *nick)
 }
 
 /*
- * Checks if string is a valid nick string (RFC 1459).
+ * Checks if string is a valid nick string, using server UTF8MAPPING.
  *
  * Returns:
- *   1: string is a valid nick
- *   0: string is not a valid nick
+ *   1: string is a valid nick on this server
+ *   0: string is not a valid nick on this server
  */
 
 int
-irc_nick_is_nick (const char *string)
+irc_nick_is_nick (struct t_irc_server *server, const char *string)
 {
-    const char *ptr;
+    const char *ptr_string, *ptr_prefix_chars, *ptr_chantypes;
+    int utf8mapping;
 
     if (!string || !string[0])
         return 0;
 
-    /* first char must not be a number or hyphen */
-    ptr = string;
-    if (strchr ("0123456789-", *ptr))
-        return 0;
+    utf8mapping = (server) ? server->utf8mapping : IRC_SERVER_UTF8MAPPING_NONE;
+    ptr_prefix_chars = (server && server->prefix_chars) ?
+        server->prefix_chars : irc_server_prefix_chars_default;
+    ptr_chantypes = irc_server_get_chantypes (server);
 
-    while (ptr && ptr[0])
+    /* check length of nick in bytes (if we have a limit in the server) */
+    if (server && (server->nick_max_length > 0)
+        && (int)strlen (string) > server->nick_max_length)
     {
-        if (!strchr (IRC_NICK_VALID_CHARS, *ptr))
+        /* nick is too long */
+        return 0;
+    }
+
+    /* check if nick is UTF-8 valid */
+    if ((utf8mapping == IRC_SERVER_UTF8MAPPING_RFC8265)
+        && !weechat_utf8_is_valid (string, -1, NULL))
+    {
+        /* invalid UTF-8 */
+        return 0;
+    }
+
+    /* check the first char in the nick */
+    if ((utf8mapping == IRC_SERVER_UTF8MAPPING_NONE)
+        && strchr ("0123456789-", string[0]))
+    {
+        /* first char is invalid */
+        return 0;
+    }
+    if (strchr (ptr_prefix_chars, string[0])
+        || strchr (ptr_chantypes, string[0]))
+    {
+        return 0;
+    }
+
+    /* check if there are forbidden chars in nick */
+    ptr_string = string;
+    while (ptr_string && ptr_string[0])
+    {
+        if ((utf8mapping == IRC_SERVER_UTF8MAPPING_NONE)
+            && !strchr (IRC_NICK_VALID_CHARS_RFC1459, ptr_string[0]))
+        {
             return 0;
-        ptr++;
+        }
+        if ((utf8mapping == IRC_SERVER_UTF8MAPPING_RFC8265)
+            && strchr (IRC_NICK_INVALID_CHARS_RFC8265, ptr_string[0]))
+        {
+            return 0;
+        }
+        ptr_string = weechat_utf8_next_char (ptr_string);
     }
 
     return 1;
 }
 
-/*
- * Duplicates a nick and stops at first char in list (using option
- * irc.look.nick_color_stop_chars).
- */
-
-char *
-irc_nick_strdup_for_color (const char *nickname)
-{
-    int char_size, other_char_seen;
-    char *result, *pos, utf_char[16];
-
-    result = malloc (strlen (nickname) + 1);
-    pos = result;
-    other_char_seen = 0;
-    while (nickname[0])
-    {
-        char_size = weechat_utf8_char_size (nickname);
-        memcpy (utf_char, nickname, char_size);
-        utf_char[char_size] = '\0';
-
-        if (strstr (weechat_config_string (irc_config_look_nick_color_stop_chars),
-                    utf_char))
-        {
-            if (other_char_seen)
-            {
-                pos[0] = '\0';
-                return result;
-            }
-        }
-        else
-        {
-            other_char_seen = 1;
-        }
-        memcpy (pos, utf_char, char_size);
-        pos += char_size;
-
-        nickname += char_size;
-    }
-    pos[0] = '\0';
-    return result;
-}
-
-/*
- * Hashes a nickname to find color.
- *
- * Returns a number which is the index of color in the nicks colors of option
- * "weechat.color.chat_nick_colors".
- */
-
-int
-irc_nick_hash_color (const char *nickname)
-{
-    unsigned long color;
-    const char *ptr_nick;
-
-    if (!irc_config_nick_colors)
-        irc_config_set_nick_colors ();
-
-    if (irc_config_num_nick_colors == 0)
-        return 0;
-
-    ptr_nick = nickname;
-    color = 0;
-
-    switch (weechat_config_integer (irc_config_look_nick_color_hash))
-    {
-        case IRC_CONFIG_LOOK_NICK_COLOR_HASH_DJB2:
-            /* variant of djb2 hash */
-            color = 5381;
-            while (ptr_nick && ptr_nick[0])
-            {
-                color ^= (color << 5) + (color >> 2) + weechat_utf8_char_int (ptr_nick);
-                ptr_nick = weechat_utf8_next_char (ptr_nick);
-            }
-            break;
-        case IRC_CONFIG_LOOK_NICK_COLOR_HASH_SUM:
-            /* sum of letters */
-            color = 0;
-            while (ptr_nick && ptr_nick[0])
-            {
-                color += weechat_utf8_char_int (ptr_nick);
-                ptr_nick = weechat_utf8_next_char (ptr_nick);
-            }
-            break;
-    }
-
-    return (color % irc_config_num_nick_colors);
-}
-
-/*
- * Gets forced color for a nick.
- *
- * Returns the name of color (for example: "green"), NULL if no color is forced
- * for nick.
- */
-
-const char *
-irc_nick_get_forced_color (const char *nickname)
-{
-    const char *forced_color;
-    char *nick_lower;
-
-    if (!nickname)
-        return NULL;
-
-    forced_color = weechat_hashtable_get (irc_config_hashtable_nick_color_force,
-                                          nickname);
-    if (forced_color)
-        return forced_color;
-
-    nick_lower = strdup (nickname);
-    if (nick_lower)
-    {
-        weechat_string_tolower (nick_lower);
-        forced_color = weechat_hashtable_get (irc_config_hashtable_nick_color_force,
-                                              nick_lower);
-        free (nick_lower);
-    }
-
-    return forced_color;
-}
-
 /*
  * Finds a color code for a nick (according to nick letters).
  *
  * Returns a WeeChat color code (that can be used for display).
  */
 
-const char *
+char *
 irc_nick_find_color (const char *nickname)
 {
-    int color;
-    char *nickname2;
-    const char *forced_color, *str_color;
-
-    if (!irc_config_nick_colors)
-        irc_config_set_nick_colors ();
-
-    if (irc_config_num_nick_colors == 0)
-        return weechat_color ("default");
-
-    /* look if color is forced */
-    forced_color = irc_nick_get_forced_color (nickname);
-    if (forced_color)
-    {
-        forced_color = weechat_color (forced_color);
-        if (forced_color && forced_color[0])
-            return forced_color;
-    }
-
-    /* hash nickname to get color */
-    nickname2 = irc_nick_strdup_for_color (nickname);
-    color = irc_nick_hash_color ((nickname2) ? nickname2 : nickname);
-    if (nickname2)
-        free (nickname2);
-
-    /* return color */
-    str_color = weechat_color (irc_config_nick_colors[color]);
-    return (str_color[0]) ? str_color : weechat_color("default");
+    return weechat_info_get ("nick_color", nickname);
 }
 
 /*
@@ -259,33 +150,10 @@ irc_nick_find_color (const char *nickname)
  * Returns the name of a color (for example: "green").
  */
 
-const char *
+char *
 irc_nick_find_color_name (const char *nickname)
 {
-    int color;
-    char *nickname2;
-    const char *forced_color;
-    static char *default_color = "default";
-
-    if (!irc_config_nick_colors)
-        irc_config_set_nick_colors ();
-
-    if (irc_config_num_nick_colors == 0)
-        return default_color;
-
-    /* look if color is forced */
-    forced_color = irc_nick_get_forced_color (nickname);
-    if (forced_color)
-        return forced_color;
-
-    /* hash nickname to get color */
-    nickname2 = irc_nick_strdup_for_color (nickname);
-    color = irc_nick_hash_color ((nickname2) ? nickname2 : nickname);
-    if (nickname2)
-        free (nickname2);
-
-    /* return color name */
-    return irc_config_nick_colors[color];
+    return weechat_info_get ("nick_color_name", nickname);
 }
 
 /*
@@ -297,6 +165,9 @@ irc_nick_set_current_prefix (struct t_irc_nick *nick)
 {
     char *ptr_prefixes;
 
+    if (!nick)
+        return;
+
     nick->prefix[0] = ' ';
     for (ptr_prefixes = nick->prefixes; ptr_prefixes[0]; ptr_prefixes++)
     {
@@ -321,6 +192,9 @@ irc_nick_set_prefix (struct t_irc_server *server, struct t_irc_nick *nick,
 {
     int index;
 
+    if (!nick)
+        return;
+
     index = irc_server_get_prefix_char_index (server, prefix);
     if (index >= 0)
     {
@@ -339,6 +213,9 @@ irc_nick_set_prefixes (struct t_irc_server *server, struct t_irc_nick *nick,
 {
     const char *ptr_prefixes;
 
+    if (!nick)
+        return;
+
     /* reset all prefixes in nick */
     memset (nick->prefixes, ' ', strlen (nick->prefixes));
 
@@ -355,6 +232,29 @@ irc_nick_set_prefixes (struct t_irc_server *server, struct t_irc_nick *nick,
     irc_nick_set_current_prefix (nick);
 }
 
+/*
+ * Sets host for nick.
+ */
+
+void
+irc_nick_set_host (struct t_irc_nick *nick, const char *host)
+{
+    if (!nick)
+        return;
+
+    /* if host is the same, just return */
+    if ((!nick->host && !host)
+        || (nick->host && host && strcmp (nick->host, host) == 0))
+    {
+        return;
+    }
+
+    /* update the host in nick */
+    if (nick->host)
+        free (nick->host);
+    nick->host = (host) ? strdup (host) : NULL;
+}
+
 /*
  * Checks if nick is "op" (or better than "op", for example channel admin or
  * channel owner).
@@ -481,7 +381,7 @@ irc_nick_get_prefix_color_name (struct t_irc_server *server, char prefix)
  * Gets nick color for nicklist.
  */
 
-const char *
+char *
 irc_nick_get_color_for_nicklist (struct t_irc_server *server,
                                  struct t_irc_nick *nick)
 {
@@ -490,17 +390,17 @@ irc_nick_get_color_for_nicklist (struct t_irc_server *server,
     static char *nick_color_away = "weechat.color.nicklist_away";
 
     if (nick->away)
-        return nick_color_away;
+        return strdup (nick_color_away);
 
-    if (weechat_config_boolean(irc_config_look_color_nicks_in_nicklist))
+    if (weechat_config_boolean (irc_config_look_color_nicks_in_nicklist))
     {
         if (irc_server_strcasecmp (server, nick->name, server->nick) == 0)
-            return nick_color_self;
+            return strdup (nick_color_self);
         else
             return irc_nick_find_color_name (nick->name);
     }
 
-    return nick_color_bar_fg;
+    return strdup (nick_color_bar_fg);
 }
 
 /*
@@ -513,14 +413,18 @@ irc_nick_nicklist_add (struct t_irc_server *server,
                        struct t_irc_nick *nick)
 {
     struct t_gui_nick_group *ptr_group;
+    char *color;
 
     ptr_group = irc_nick_get_nicklist_group (server, channel->buffer, nick);
+    color = irc_nick_get_color_for_nicklist (server, nick);
     weechat_nicklist_add_nick (channel->buffer, ptr_group,
                                nick->name,
-                               irc_nick_get_color_for_nicklist (server, nick),
+                               color,
                                nick->prefix,
                                irc_nick_get_prefix_color_name (server, nick->prefix[0]),
                                1);
+    if (color)
+        free (color);
 }
 
 /*
@@ -597,6 +501,7 @@ irc_nick_nicklist_set_color_all ()
     struct t_irc_server *ptr_server;
     struct t_irc_channel *ptr_channel;
     struct t_irc_nick *ptr_nick;
+    char *color;
 
     for (ptr_server = irc_servers; ptr_server;
          ptr_server = ptr_server->next_server)
@@ -607,9 +512,10 @@ irc_nick_nicklist_set_color_all ()
             for (ptr_nick = ptr_channel->nicks; ptr_nick;
                  ptr_nick = ptr_nick->next_nick)
             {
-                irc_nick_nicklist_set (ptr_channel, ptr_nick, "color",
-                                       irc_nick_get_color_for_nicklist (ptr_server,
-                                                                        ptr_nick));
+                color = irc_nick_get_color_for_nicklist (ptr_server, ptr_nick);
+                irc_nick_nicklist_set (ptr_channel, ptr_nick, "color", color);
+                if (color)
+                    free (color);
             }
         }
     }
@@ -624,7 +530,7 @@ irc_nick_nicklist_set_color_all ()
 struct t_irc_nick *
 irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
               const char *nickname, const char *host, const char *prefixes,
-              int away)
+              int away, const char *account, const char *realname)
 {
     struct t_irc_nick *new_nick, *ptr_nick;
     int length;
@@ -639,15 +545,11 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
     ptr_nick = irc_nick_search (server, channel, nickname);
     if (ptr_nick)
     {
-        /* save away status from existing nick (before removing it) */
-        away = ptr_nick->away;
-
         /* remove old nick from nicklist */
         irc_nick_nicklist_remove (server, channel, ptr_nick);
 
-        /* update nick */
+        /* update nick prefixes */
         irc_nick_set_prefixes (server, ptr_nick, prefixes);
-        ptr_nick->away = away;
 
         /* add new nick in nicklist */
         irc_nick_nicklist_add (server, channel, ptr_nick);
@@ -662,24 +564,30 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
     /* initialize new nick */
     new_nick->name = strdup (nickname);
     new_nick->host = (host) ? strdup (host) : NULL;
+    new_nick->account = (account) ? strdup (account) : NULL;
+    new_nick->realname = (realname) ? strdup (realname) : NULL;
     length = strlen (irc_server_get_prefix_chars (server));
     new_nick->prefixes = malloc (length + 1);
-    if (!new_nick->name || !new_nick->prefixes)
+    new_nick->prefix = malloc (2);
+    if (!new_nick->name || !new_nick->prefixes || !new_nick->prefix)
     {
         if (new_nick->name)
             free (new_nick->name);
         if (new_nick->host)
             free (new_nick->host);
+        if (new_nick->account)
+            free (new_nick->account);
+        if (new_nick->realname)
+            free (new_nick->realname);
         if (new_nick->prefixes)
             free (new_nick->prefixes);
+        if (new_nick->prefix)
+            free (new_nick->prefix);
         free (new_nick);
         return NULL;
     }
-    if (new_nick->prefixes)
-    {
-        memset (new_nick->prefixes, ' ', length);
-        new_nick->prefixes[length] = '\0';
-    }
+    memset (new_nick->prefixes, ' ', length);
+    new_nick->prefixes[length] = '\0';
     new_nick->prefix[0] = ' ';
     new_nick->prefix[1] = '\0';
     irc_nick_set_prefixes (server, new_nick, prefixes);
@@ -687,11 +595,11 @@ irc_nick_new (struct t_irc_server *server, struct t_irc_channel *channel,
     if (irc_server_strcasecmp (server, new_nick->name, server->nick) == 0)
         new_nick->color = strdup (IRC_COLOR_CHAT_NICK_SELF);
     else
-        new_nick->color = strdup (irc_nick_find_color (new_nick->name));
+        new_nick->color = irc_nick_find_color (new_nick->name);
 
     /* add nick to end of list */
     new_nick->prev_nick = channel->last_nick;
-    if (channel->nicks)
+    if (channel->last_nick)
         channel->last_nick->next_nick = new_nick;
     else
         channel->nicks = new_nick;
@@ -736,7 +644,7 @@ irc_nick_change (struct t_irc_server *server, struct t_irc_channel *channel,
     if (nick_is_me)
         nick->color = strdup (IRC_COLOR_CHAT_NICK_SELF);
     else
-        nick->color = strdup (irc_nick_find_color (nick->name));
+        nick->color = irc_nick_find_color (nick->name);
 
     /* add nick in nicklist */
     irc_nick_nicklist_add (server, channel, nick);
@@ -768,7 +676,58 @@ irc_nick_set_mode (struct t_irc_server *server, struct t_irc_channel *channel,
     irc_nick_nicklist_add (server, channel, nick);
 
     if (irc_server_strcasecmp (server, nick->name, server->nick) == 0)
+    {
         weechat_bar_item_update ("input_prompt");
+        weechat_bar_item_update ("irc_nick");
+        weechat_bar_item_update ("irc_nick_host");
+    }
+}
+
+/*
+ * Reallocates the "prefixes" string in all nicks of all channels on the server
+ * (after 005 has been received).
+ */
+
+void
+irc_nick_realloc_prefixes (struct t_irc_server *server,
+                           int old_length, int new_length)
+{
+    struct t_irc_channel *ptr_channel;
+    struct t_irc_nick *ptr_nick;
+    char *new_prefixes;
+
+    for (ptr_channel = server->channels; ptr_channel;
+         ptr_channel = ptr_channel->next_channel)
+    {
+        for (ptr_nick = ptr_channel->nicks; ptr_nick;
+             ptr_nick = ptr_nick->next_nick)
+        {
+            if (ptr_nick->prefixes)
+            {
+                new_prefixes = realloc (ptr_nick->prefixes, new_length + 1);
+                if (new_prefixes)
+                {
+                    ptr_nick->prefixes = new_prefixes;
+                    if (new_length > old_length)
+                    {
+                        memset (ptr_nick->prefixes + old_length,
+                                ' ',
+                                new_length - old_length);
+                    }
+                    ptr_nick->prefixes[new_length] = '\0';
+                }
+            }
+            else
+            {
+                ptr_nick->prefixes = malloc (new_length + 1);
+                if (ptr_nick->prefixes)
+                {
+                    memset (ptr_nick->prefixes, ' ', new_length);
+                    ptr_nick->prefixes[new_length] = '\0';
+                }
+            }
+        }
+    }
 }
 
 /*
@@ -810,6 +769,12 @@ irc_nick_free (struct t_irc_server *server, struct t_irc_channel *channel,
         free (nick->host);
     if (nick->prefixes)
         free (nick->prefixes);
+    if (nick->prefix)
+        free (nick->prefix);
+    if (nick->account)
+        free (nick->account);
+    if (nick->realname)
+        free (nick->realname);
     if (nick->color)
         free (nick->color);
 
@@ -913,18 +878,15 @@ void
 irc_nick_set_away (struct t_irc_server *server, struct t_irc_channel *channel,
                    struct t_irc_nick *nick, int is_away)
 {
-    if (!is_away
-        || server->cap_away_notify
-        || ((IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK) > 0)
-            && ((IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK_MAX_NICKS) == 0)
-                || (channel->nicks_count <= IRC_SERVER_OPTION_INTEGER(server, IRC_SERVER_OPTION_AWAY_CHECK_MAX_NICKS)))))
+    char *color;
+
+    if (is_away != nick->away)
     {
-        if ((is_away && !nick->away) || (!is_away && nick->away))
-        {
-            nick->away = is_away;
-            irc_nick_nicklist_set (channel, nick, "color",
-                                   irc_nick_get_color_for_nicklist (server, nick));
-        }
+        nick->away = is_away;
+        color = irc_nick_get_color_for_nicklist (server, nick);
+        irc_nick_nicklist_set (channel, nick, "color", color);
+        if (color)
+            free (color);
     }
 }
 
@@ -991,12 +953,25 @@ irc_nick_as_prefix (struct t_irc_server *server, struct t_irc_nick *nick,
                     const char *nickname, const char *force_color)
 {
     static char result[256];
+    char *color;
+
+    if (force_color)
+        color = strdup (force_color);
+    else if (nick)
+        color = strdup (nick->color);
+    else if (nickname)
+        color = irc_nick_find_color (nickname);
+    else
+        color = strdup (IRC_COLOR_CHAT_NICK);
 
     snprintf (result, sizeof (result), "%s%s%s\t",
               irc_nick_mode_for_display (server, nick, 1),
-              (force_color) ? force_color : ((nick) ? nick->color : ((nickname) ? irc_nick_find_color (nickname) : IRC_COLOR_CHAT_NICK)),
+              color,
               (nick) ? nick->name : nickname);
 
+    if (color)
+        free (color);
+
     return result;
 }
 
@@ -1008,6 +983,10 @@ const char *
 irc_nick_color_for_msg (struct t_irc_server *server, int server_message,
                         struct t_irc_nick *nick, const char *nickname)
 {
+    static char color[16][64];
+    static int index_color = 0;
+    char *color_found;
+
     if (server_message
         && !weechat_config_boolean (irc_config_look_color_nicks_in_server_messages))
     {
@@ -1024,7 +1003,14 @@ irc_nick_color_for_msg (struct t_irc_server *server, int server_message,
         {
             return IRC_COLOR_CHAT_NICK_SELF;
         }
-        return irc_nick_find_color (nickname);
+        color_found = irc_nick_find_color (nickname);
+        index_color = (index_color + 1) % 16;
+        snprintf (color[index_color], sizeof (color[index_color]),
+                  "%s",
+                  color_found);
+        if (color_found)
+            free (color_found);
+        return color[index_color];
     }
 
     return IRC_COLOR_CHAT_NICK;
@@ -1040,7 +1026,7 @@ irc_nick_color_for_pv (struct t_irc_channel *channel, const char *nickname)
     if (weechat_config_boolean (irc_config_look_color_pv_nick_like_channel))
     {
         if (!channel->pv_remote_nick_color)
-            channel->pv_remote_nick_color = strdup (irc_nick_find_color (nickname));
+            channel->pv_remote_nick_color = irc_nick_find_color (nickname);
         if (channel->pv_remote_nick_color)
             return channel->pv_remote_nick_color;
     }
@@ -1113,11 +1099,13 @@ irc_nick_default_ban_mask (struct t_irc_nick *nick)
  */
 
 struct t_hdata *
-irc_nick_hdata_nick_cb (void *data, const char *hdata_name)
+irc_nick_hdata_nick_cb (const void *pointer, void *data,
+                        const char *hdata_name)
 {
     struct t_hdata *hdata;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
 
     hdata = weechat_hdata_new (hdata_name, "prev_nick", "next_nick",
@@ -1129,6 +1117,8 @@ irc_nick_hdata_nick_cb (void *data, const char *hdata_name)
         WEECHAT_HDATA_VAR(struct t_irc_nick, prefixes, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_nick, prefix, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_nick, away, INTEGER, 0, NULL, NULL);
+        WEECHAT_HDATA_VAR(struct t_irc_nick, account, STRING, 0, NULL, NULL);
+        WEECHAT_HDATA_VAR(struct t_irc_nick, realname, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_nick, color, STRING, 0, NULL, NULL);
         WEECHAT_HDATA_VAR(struct t_irc_nick, prev_nick, POINTER, 0, NULL, hdata_name);
         WEECHAT_HDATA_VAR(struct t_irc_nick, next_nick, POINTER, 0, NULL, hdata_name);
@@ -1167,6 +1157,10 @@ irc_nick_add_to_infolist (struct t_infolist *infolist,
         return 0;
     if (!weechat_infolist_new_var_integer (ptr_item, "away", nick->away))
         return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "account", nick->account))
+        return 0;
+    if (!weechat_infolist_new_var_string (ptr_item, "realname", nick->realname))
+        return 0;
     if (!weechat_infolist_new_var_string (ptr_item, "color", nick->color))
         return 0;
 
@@ -1186,6 +1180,8 @@ irc_nick_print_log (struct t_irc_nick *nick)
     weechat_log_printf ("         prefixes . . . : '%s'",  nick->prefixes);
     weechat_log_printf ("         prefix . . . . : '%s'",  nick->prefix);
     weechat_log_printf ("         away . . . . . : %d",    nick->away);
+    weechat_log_printf ("         account. . . . : '%s'",  nick->account);
+    weechat_log_printf ("         realname . . . : '%s'",  nick->realname);
     weechat_log_printf ("         color. . . . . : '%s'",  nick->color);
     weechat_log_printf ("         prev_nick. . . : 0x%lx", nick->prev_nick);
     weechat_log_printf ("         next_nick. . . : 0x%lx", nick->next_nick);
