@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_IRC_RAW_H
@@ -30,16 +30,19 @@
 #define IRC_RAW_PREFIX_SEND          "<--"
 #define IRC_RAW_PREFIX_SEND_MODIFIED "<=="
 
-#define IRC_RAW_FLAG_RECV     1
-#define IRC_RAW_FLAG_SEND     2
-#define IRC_RAW_FLAG_MODIFIED 4
-#define IRC_RAW_FLAG_REDIRECT 8
-#define IRC_RAW_FLAG_BINARY   16
+#define IRC_RAW_FLAG_RECV     (1 << 0)
+#define IRC_RAW_FLAG_SEND     (1 << 1)
+#define IRC_RAW_FLAG_MODIFIED (1 << 2)
+#define IRC_RAW_FLAG_REDIRECT (1 << 3)
+#define IRC_RAW_FLAG_BINARY   (1 << 4)
+
+struct t_irc_server;
 
 struct t_irc_raw_message
 {
     time_t date;                       /* date/time of message              */
-    char *prefix;                      /* prefix                            */
+    struct t_irc_server *server;       /* server                            */
+    int flags;                         /* flags                             */
     char *message;                     /* message                           */
     struct t_irc_raw_message *prev_message; /* pointer to previous message  */
     struct t_irc_raw_message *next_message; /* pointer to next message      */
@@ -51,14 +54,19 @@ extern struct t_gui_buffer *irc_raw_buffer;
 extern int irc_raw_messages_count;
 extern struct t_irc_raw_message *irc_raw_messages, *last_irc_raw_message;
 
+extern void irc_raw_refresh (int clear);
 extern void irc_raw_open (int switch_to_buffer);
+extern void irc_raw_set_filter (const char *filter);
+extern void irc_raw_filter_options (const char *filter);
 extern struct t_irc_raw_message *irc_raw_message_add_to_list (time_t date,
-                                                              const char *prefix,
+                                                              struct t_irc_server *server,
+                                                              int flags,
                                                               const char *message);
 extern void irc_raw_print (struct t_irc_server *server, int flags,
                            const char *message);
-extern void irc_raw_message_free_all ();
 extern int irc_raw_add_to_infolist (struct t_infolist *infolist,
                                     struct t_irc_raw_message *raw_message);
+extern void irc_raw_init ();
+extern void irc_raw_end ();
 
 #endif /* WEECHAT_PLUGIN_IRC_RAW_H */
