@@ -1,7 +1,7 @@
 /*
  * irc-upgrade.c - save/restore IRC plugin data when upgrading WeeChat
  *
- * Copyright (C) 2003-2015 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -30,6 +30,7 @@
 #include "irc-channel.h"
 #include "irc-config.h"
 #include "irc-input.h"
+#include "irc-modelist.h"
 #include "irc-nick.h"
 #include "irc-notify.h"
 #include "irc-raw.h"
@@ -39,6 +40,7 @@
 
 struct t_irc_server *irc_upgrade_current_server = NULL;
 struct t_irc_channel *irc_upgrade_current_channel = NULL;
+struct t_irc_modelist *irc_upgrade_current_modelist = NULL;
 
 
 /*
@@ -59,6 +61,8 @@ irc_upgrade_save_all_data (struct t_upgrade_file *upgrade_file)
     struct t_irc_redirect *ptr_redirect;
     struct t_irc_redirect_pattern *ptr_redirect_pattern;
     struct t_irc_notify *ptr_notify;
+    struct t_irc_modelist *ptr_modelist;
+    struct t_irc_modelist_item *ptr_item;
     struct t_irc_raw_message *ptr_raw_message;
     int rc;
 
@@ -120,6 +124,46 @@ irc_upgrade_save_all_data (struct t_upgrade_file *upgrade_file)
                 if (!rc)
                     return 0;
             }
+
+            for (ptr_modelist = ptr_channel->modelists; ptr_modelist;
+                 ptr_modelist = ptr_modelist->next_modelist)
+            {
+                /* save modelist */
+                infolist = weechat_infolist_new ();
+                if (!infolist)
+                    return 0;
+                if (!irc_modelist_add_to_infolist (infolist, ptr_modelist))
+                {
+                    weechat_infolist_free (infolist);
+                    return 0;
+                }
+                rc = weechat_upgrade_write_object (upgrade_file,
+                                                   IRC_UPGRADE_TYPE_MODELIST,
+                                                   infolist);
+                weechat_infolist_free (infolist);
+                if (!rc)
+                    return 0;
+
+                for (ptr_item = ptr_modelist->items; ptr_item;
+                     ptr_item = ptr_item->next_item)
+                {
+                    /* save modelist item */
+                    infolist = weechat_infolist_new ();
+                    if (!infolist)
+                        return 0;
+                    if (!irc_modelist_item_add_to_infolist (infolist, ptr_item))
+                    {
+                        weechat_infolist_free (infolist);
+                        return 0;
+                    }
+                    rc = weechat_upgrade_write_object (upgrade_file,
+                                                       IRC_UPGRADE_TYPE_MODELIST_ITEM,
+                                                       infolist);
+                    weechat_infolist_free (infolist);
+                    if (!rc)
+                        return 0;
+                }
+            }
         }
 
         /* save server redirects */
@@ -224,7 +268,8 @@ irc_upgrade_save ()
     int rc;
     struct t_upgrade_file *upgrade_file;
 
-    upgrade_file = weechat_upgrade_new (IRC_UPGRADE_FILENAME, 1);
+    upgrade_file = weechat_upgrade_new (IRC_UPGRADE_FILENAME,
+                                        NULL, NULL, NULL);
     if (!upgrade_file)
         return 0;
 
@@ -245,6 +290,7 @@ irc_upgrade_set_buffer_callbacks ()
 {
     struct t_infolist *infolist;
     struct t_gui_buffer *ptr_buffer;
+    struct t_irc_server *ptr_server;
     const char *type;
 
     infolist = weechat_infolist_get ("buffer", NULL, NULL);
@@ -260,8 +306,17 @@ irc_upgrade_set_buffer_callbacks ()
                 type = weechat_buffer_get_string (ptr_buffer, "localvar_type");
                 if (type && (strcmp (type, "channel") == 0))
                 {
+                    ptr_server = irc_server_search (
+                        weechat_buffer_get_string (ptr_buffer,
+                                                   "localvar_server"));
                     weechat_buffer_set_pointer (ptr_buffer, "nickcmp_callback",
                                                 &irc_buffer_nickcmp_cb);
+                    if (ptr_server)
+                    {
+                        weechat_buffer_set_pointer (ptr_buffer,
+                                                    "nickcmp_callback_pointer",
+                                                    ptr_server);
+                    }
                 }
                 if (strcmp (weechat_infolist_string (infolist, "name"),
                             IRC_RAW_BUFFER_NAME) == 0)
@@ -279,23 +334,26 @@ irc_upgrade_set_buffer_callbacks ()
  */
 
 int
-irc_upgrade_read_cb (void *data,
+irc_upgrade_read_cb (const void *pointer, void *data,
                      struct t_upgrade_file *upgrade_file,
                      int object_id,
                      struct t_infolist *infolist)
 {
-    int flags, sock, size, i, index, nicks_count, num_items;
+    int flags, sock, size, i, index, nicks_count, num_items, utf8mapping;
     long number;
     time_t join_time;
     char *buf, option_name[64], **nicks, *nick_join, *pos, *error;
     char **items;
     const char *buffer_name, *str, *nick;
+    struct t_irc_server *ptr_server;
     struct t_irc_nick *ptr_nick;
     struct t_irc_redirect *ptr_redirect;
     struct t_irc_notify *ptr_notify;
+    struct t_irc_modelist_item *ptr_item;
     struct t_gui_buffer *ptr_buffer;
 
     /* make C compiler happy */
+    (void) pointer;
     (void) data;
     (void) upgrade_file;
 
@@ -310,6 +368,8 @@ irc_upgrade_read_cb (void *data,
                 {
                     irc_upgrade_current_server->temp_server =
                         weechat_infolist_integer (infolist, "temp_server");
+                    irc_upgrade_current_server->fake_server =
+                        weechat_infolist_integer (infolist, "fake_server");
                     irc_upgrade_current_server->buffer = NULL;
                     buffer_name = weechat_infolist_string (infolist, "buffer_name");
                     if (buffer_name && buffer_name[0])
@@ -344,10 +404,12 @@ irc_upgrade_read_cb (void *data,
                     if (sock >= 0)
                     {
                         irc_upgrade_current_server->sock = sock;
-                        irc_upgrade_current_server->hook_fd = weechat_hook_fd (irc_upgrade_current_server->sock,
-                                                                               1, 0, 0,
-                                                                               &irc_server_recv_cb,
-                                                                               irc_upgrade_current_server);
+                        irc_upgrade_current_server->hook_fd = weechat_hook_fd (
+                            irc_upgrade_current_server->sock,
+                            1, 0, 0,
+                            &irc_server_recv_cb,
+                            irc_upgrade_current_server,
+                            NULL);
                     }
                     irc_upgrade_current_server->is_connected = weechat_infolist_integer (infolist, "is_connected");
                     irc_upgrade_current_server->ssl_connected = weechat_infolist_integer (infolist, "ssl_connected");
@@ -361,7 +423,34 @@ irc_upgrade_read_cb (void *data,
                     str = weechat_infolist_string (infolist, "nick_modes");
                     if (str)
                         irc_upgrade_current_server->nick_modes = strdup (str);
-                    irc_upgrade_current_server->cap_away_notify = weechat_infolist_integer (infolist, "cap_away_notify");
+                    str = weechat_infolist_string (infolist, "host");
+                    if (str)
+                        irc_upgrade_current_server->host = strdup (str);
+                    /*
+                     * "cap_ls" and "cap_list" replace "cap_away_notify",
+                     * "cap_account_notify" and "cap_extended_join"
+                     * in WeeChat 2.2
+                     */
+                    if (weechat_infolist_integer (infolist, "cap_away_notify"))
+                    {
+                        weechat_hashtable_set (irc_upgrade_current_server->cap_ls, "away-notify", NULL);
+                        weechat_hashtable_set (irc_upgrade_current_server->cap_list, "away-notify", NULL);
+                    }
+                    if (weechat_infolist_integer (infolist, "cap_account_notify"))
+                    {
+                        weechat_hashtable_set (irc_upgrade_current_server->cap_ls, "account-notify", NULL);
+                        weechat_hashtable_set (irc_upgrade_current_server->cap_list, "account-notify", NULL);
+                    }
+                    if (weechat_infolist_integer (infolist, "cap_extended_join"))
+                    {
+                        weechat_hashtable_set (irc_upgrade_current_server->cap_ls, "extended-join", NULL);
+                        weechat_hashtable_set (irc_upgrade_current_server->cap_list, "extended-join", NULL);
+                    }
+                    weechat_hashtable_add_from_infolist (
+                        irc_upgrade_current_server->cap_ls, infolist, "cap_ls");
+                    weechat_hashtable_add_from_infolist (
+                        irc_upgrade_current_server->cap_list, infolist, "cap_list");
+
                     str = weechat_infolist_string (infolist, "isupport");
                     if (str)
                         irc_upgrade_current_server->isupport = strdup (str);
@@ -390,7 +479,60 @@ irc_upgrade_read_cb (void *data,
                         irc_upgrade_current_server->prefix_chars = strdup (str);
                     }
                     irc_upgrade_current_server->nick_max_length = weechat_infolist_integer (infolist, "nick_max_length");
+                    /* "user_max_length" is new in WeeChat 2.6 */
+                    if (weechat_infolist_search_var (infolist, "user_max_length"))
+                    {
+                        irc_upgrade_current_server->user_max_length = weechat_infolist_integer (infolist, "user_max_length");
+                    }
+                    else
+                    {
+                        /* WeeChat <= 2.5 */
+                        str = irc_server_get_isupport_value (irc_upgrade_current_server,
+                                                             "USERLEN");
+                        if (str)
+                        {
+                            error = NULL;
+                            number = strtol (str, &error, 10);
+                            if (error && !error[0])
+                                irc_upgrade_current_server->user_max_length = (int)number;
+                        }
+                    }
+                    /* "host_max_length" is new in WeeChat 2.6 */
+                    if (weechat_infolist_search_var (infolist, "host_max_length"))
+                    {
+                        irc_upgrade_current_server->host_max_length = weechat_infolist_integer (infolist, "host_max_length");
+                    }
+                    else
+                    {
+                        /* WeeChat <= 2.5 */
+                        str = irc_server_get_isupport_value (irc_upgrade_current_server,
+                                                             "HOSTLEN");
+                        if (str)
+                        {
+                            error = NULL;
+                            number = strtol (str, &error, 10);
+                            if (error && !error[0])
+                                irc_upgrade_current_server->host_max_length = (int)number;
+                        }
+                    }
                     irc_upgrade_current_server->casemapping = weechat_infolist_integer (infolist, "casemapping");
+                    /* "utf8mapping" is new in WeeChat 2.9 */
+                    if (weechat_infolist_search_var (infolist, "utf8mapping"))
+                    {
+                        irc_upgrade_current_server->utf8mapping = weechat_infolist_integer (infolist, "utf8mapping");
+                    }
+                    else
+                    {
+                        /* WeeChat <= 2.8 */
+                        str = irc_server_get_isupport_value (irc_upgrade_current_server,
+                                                             "UTF8MAPPING");
+                        if (str)
+                        {
+                            utf8mapping = irc_server_search_utf8mapping (str);
+                            if (utf8mapping >= 0)
+                                irc_upgrade_current_server->utf8mapping = utf8mapping;
+                        }
+                    }
                     str = weechat_infolist_string (infolist, "chantypes");
                     if (str)
                         irc_upgrade_current_server->chantypes = strdup (str);
@@ -422,6 +564,23 @@ irc_upgrade_read_cb (void *data,
                                 irc_upgrade_current_server->monitor = (int)number;
                         }
                     }
+                    /* "clienttagdeny" is new in WeeChat 3.3 */
+                    if (weechat_infolist_search_var (infolist, "clienttagdeny"))
+                    {
+                        irc_server_set_clienttagdeny (irc_upgrade_current_server,
+                                                      weechat_infolist_string (infolist, "clienttagdeny"));
+                    }
+                    else
+                    {
+                        /* WeeChat <= 3.2 */
+                        str = irc_server_get_isupport_value (irc_upgrade_current_server,
+                                                             "CLIENTTAGDENY");
+                        if (str)
+                        {
+                            irc_server_set_clienttagdeny (irc_upgrade_current_server,
+                                                          str);
+                        }
+                    }
                     irc_upgrade_current_server->reconnect_delay = weechat_infolist_integer (infolist, "reconnect_delay");
                     irc_upgrade_current_server->reconnect_start = weechat_infolist_time (infolist, "reconnect_start");
                     irc_upgrade_current_server->command_time = weechat_infolist_time (infolist, "command_time");
@@ -433,6 +592,7 @@ irc_upgrade_read_cb (void *data,
                         irc_upgrade_current_server->away_message = strdup (str);
                     irc_upgrade_current_server->away_time = weechat_infolist_time (infolist, "away_time");
                     irc_upgrade_current_server->lag = weechat_infolist_integer (infolist, "lag");
+                    irc_upgrade_current_server->lag_displayed = weechat_infolist_integer (infolist, "lag_displayed");
                     buf = weechat_infolist_buffer (infolist, "lag_check_time", &size);
                     if (buf)
                         memcpy (&(irc_upgrade_current_server->lag_check_time), buf, size);
@@ -465,8 +625,15 @@ irc_upgrade_read_cb (void *data,
                         str = weechat_infolist_string (infolist, "join_msg_received");
                         if (str)
                         {
-                            items = weechat_string_split (str, ",", 0, 0,
-                                                          &num_items);
+                            items = weechat_string_split (
+                                str,
+                                ",",
+                                NULL,
+                                WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                0,
+                                &num_items);
                             if (items)
                             {
                                 for (i = 0; i < num_items; i++)
@@ -477,7 +644,7 @@ irc_upgrade_read_cb (void *data,
                                 weechat_string_free_split (items);
                             }
                         }
-                        irc_upgrade_current_channel->checking_away = weechat_infolist_integer (infolist, "checking_away");
+                        irc_upgrade_current_channel->checking_whox = weechat_infolist_integer (infolist, "checking_whox");
                         str = weechat_infolist_string (infolist, "away_message");
                         if (str)
                             irc_upgrade_current_channel->away_message = strdup (str);
@@ -521,8 +688,15 @@ irc_upgrade_read_cb (void *data,
                         str = weechat_infolist_string (infolist, "join_smart_filtered");
                         if (str)
                         {
-                            nicks = weechat_string_split (str, ",", 0, 0,
-                                                          &nicks_count);
+                            nicks = weechat_string_split (
+                                str,
+                                ",",
+                                NULL,
+                                WEECHAT_STRING_SPLIT_STRIP_LEFT
+                                | WEECHAT_STRING_SPLIT_STRIP_RIGHT
+                                | WEECHAT_STRING_SPLIT_COLLAPSE_SEPS,
+                                0,
+                                &nicks_count);
                             if (nicks)
                             {
                                 for (i = 0; i < nicks_count; i++)
@@ -561,7 +735,9 @@ irc_upgrade_read_cb (void *data,
                                              weechat_infolist_string (infolist, "name"),
                                              weechat_infolist_string (infolist, "host"),
                                              weechat_infolist_string (infolist, "prefixes"),
-                                             weechat_infolist_integer (infolist, "away"));
+                                             weechat_infolist_integer (infolist, "away"),
+                                             weechat_infolist_string (infolist, "account"),
+                                             weechat_infolist_string (infolist, "realname"));
                     if (ptr_nick)
                     {
                         /*
@@ -638,6 +814,33 @@ irc_upgrade_read_cb (void *data,
                     }
                 }
                 break;
+            case IRC_UPGRADE_TYPE_MODELIST:
+                if (irc_upgrade_current_server && irc_upgrade_current_channel)
+                {
+                    /* modelists are already created by the channel */
+                    irc_upgrade_current_modelist = irc_modelist_search (
+                        irc_upgrade_current_channel,
+                        weechat_infolist_string (infolist, "type")[0]);
+                    if (irc_upgrade_current_modelist)
+                    {
+                        irc_upgrade_current_modelist->state = weechat_infolist_integer (infolist, "state");
+                    }
+                }
+                break;
+            case IRC_UPGRADE_TYPE_MODELIST_ITEM:
+                if (irc_upgrade_current_server && irc_upgrade_current_channel && irc_upgrade_current_modelist)
+                {
+                    ptr_item = irc_modelist_item_new (
+                        irc_upgrade_current_modelist,
+                        weechat_infolist_string (infolist, "mask"),
+                        weechat_infolist_string (infolist, "setter"),
+                        weechat_infolist_time (infolist, "datetime"));
+                    if (ptr_item)
+                    {
+                        ptr_item->number = weechat_infolist_integer (infolist, "number");
+                    }
+                }
+                break;
             case IRC_UPGRADE_TYPE_REDIRECT:
                 if (irc_upgrade_current_server)
                 {
@@ -693,9 +896,20 @@ irc_upgrade_read_cb (void *data,
                 }
                 break;
             case IRC_UPGRADE_TYPE_RAW_MESSAGE:
-                irc_raw_message_add_to_list (weechat_infolist_time (infolist, "date"),
-                                             weechat_infolist_string (infolist, "prefix"),
-                                             weechat_infolist_string (infolist, "message"));
+                /* "server" and "flags" are new in WeeChat 2.7  */
+                str = weechat_infolist_string (infolist, "server");
+                if (str && str[0])
+                {
+                    ptr_server = irc_server_search (str);
+                    if (ptr_server)
+                    {
+                        irc_raw_message_add_to_list (
+                            weechat_infolist_time (infolist, "date"),
+                            ptr_server,
+                            weechat_infolist_integer (infolist, "flags"),
+                            weechat_infolist_string (infolist, "message"));
+                    }
+                }
                 break;
         }
     }
@@ -716,15 +930,26 @@ irc_upgrade_load ()
 {
     int rc;
     struct t_upgrade_file *upgrade_file;
+    const char *ptr_filter;
 
     irc_upgrade_set_buffer_callbacks ();
 
-    upgrade_file = weechat_upgrade_new (IRC_UPGRADE_FILENAME, 0);
+    upgrade_file = weechat_upgrade_new (IRC_UPGRADE_FILENAME,
+                                        &irc_upgrade_read_cb, NULL, NULL);
     if (!upgrade_file)
         return 0;
-    rc = weechat_upgrade_read (upgrade_file, &irc_upgrade_read_cb, NULL);
+
+    rc = weechat_upgrade_read (upgrade_file);
+
     weechat_upgrade_close (upgrade_file);
 
+    if (irc_raw_buffer)
+    {
+        ptr_filter = weechat_buffer_get_string (irc_raw_buffer,
+                                                "localvar_filter");
+        irc_raw_filter_options (
+            (ptr_filter && ptr_filter[0]) ? ptr_filter : "*");
+    }
+
     return rc;
 }
-
