@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_IRC_UPGRADE_H
@@ -33,6 +33,8 @@ enum t_irc_upgrade_type
     IRC_UPGRADE_TYPE_REDIRECT_PATTERN,
     IRC_UPGRADE_TYPE_REDIRECT,
     IRC_UPGRADE_TYPE_NOTIFY,
+    IRC_UPGRADE_TYPE_MODELIST,
+    IRC_UPGRADE_TYPE_MODELIST_ITEM,
 };
 
 extern int irc_upgrade_save ();
