@@ -2,7 +2,7 @@
  * weechat-lua.c - lua plugin for WeeChat
  *
  * Copyright (C) 2006-2007 Emmanuel Bouthenot <kolter@openics.org>
- * Copyright (C) 2006-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2006-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -17,7 +17,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #undef _
@@ -42,15 +42,41 @@ WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
 WEECHAT_PLUGIN_PRIORITY(4000);
 
-struct t_weechat_plugin *weechat_lua_plugin;
+struct t_weechat_plugin *weechat_lua_plugin = NULL;
+
+struct t_plugin_script_data lua_data;
+
+struct t_config_file *lua_config_file = NULL;
+struct t_config_option *lua_config_look_check_license = NULL;
+struct t_config_option *lua_config_look_eval_keep_context = NULL;
 
 int lua_quiet = 0;
+
+struct t_plugin_script *lua_script_eval = NULL;
+int lua_eval_mode = 0;
+int lua_eval_send_input = 0;
+int lua_eval_exec_commands = 0;
+struct t_gui_buffer *lua_eval_buffer = NULL;
+#if LUA_VERSION_NUM >= 502
+#define LUA_LOAD "load"
+#else
+#define LUA_LOAD "loadstring"
+#endif /* LUA_VERSION_NUM >= 502 */
+#define LUA_EVAL_SCRIPT                                                 \
+    "function script_lua_eval(code)\n"                                  \
+    "    assert(" LUA_LOAD "(code))()\n"                                \
+    "end\n"                                                             \
+    "\n"                                                                \
+    "weechat.register('" WEECHAT_SCRIPT_EVAL_NAME "', '', '1.0', "      \
+    "'" WEECHAT_LICENSE "', 'Evaluation of source code', '', '')\n"
+
 struct t_plugin_script *lua_scripts = NULL;
 struct t_plugin_script *last_lua_script = NULL;
 struct t_plugin_script *lua_current_script = NULL;
 struct t_plugin_script *lua_registered_script = NULL;
 const char *lua_current_script_filename = NULL;
 lua_State *lua_current_interpreter = NULL;
+char **lua_buffer_output = NULL;
 
 /*
  * string used to execute action "install":
@@ -155,6 +181,102 @@ weechat_lua_tohashtable (lua_State *interpreter, int index, int size,
     return hashtable;
 }
 
+/*
+ * Flushes output.
+ */
+
+void
+weechat_lua_output_flush ()
+{
+    const char *ptr_command;
+    char *temp_buffer, *command;
+    int length;
+
+    if (!*lua_buffer_output[0])
+        return;
+
+    /* if there's no buffer, we catch the output, so there's no flush */
+    if (lua_eval_mode && !lua_eval_buffer)
+        return;
+
+    temp_buffer = strdup (*lua_buffer_output);
+    if (!temp_buffer)
+        return;
+
+    weechat_string_dyn_copy (lua_buffer_output, NULL);
+
+    if (lua_eval_mode)
+    {
+        if (lua_eval_send_input)
+        {
+            if (lua_eval_exec_commands)
+                ptr_command = temp_buffer;
+            else
+                ptr_command = weechat_string_input_for_buffer (temp_buffer);
+            if (ptr_command)
+            {
+                weechat_command (lua_eval_buffer, temp_buffer);
+            }
+            else
+            {
+                length = 1 + strlen (temp_buffer) + 1;
+                command = malloc (length);
+                if (command)
+                {
+                    snprintf (command, length, "%c%s",
+                              temp_buffer[0], temp_buffer);
+                    weechat_command (lua_eval_buffer,
+                                     (command[0]) ? command : " ");
+                    free (command);
+                }
+            }
+        }
+        else
+        {
+            weechat_printf (lua_eval_buffer, "%s", temp_buffer);
+        }
+    }
+    else
+    {
+        /* script (no eval mode) */
+        weechat_printf (
+            NULL,
+            weechat_gettext ("%s: stdout/stderr (%s): %s"),
+            LUA_PLUGIN_NAME,
+            (lua_current_script) ? lua_current_script->name : "?",
+            temp_buffer);
+    }
+
+    free (temp_buffer);
+}
+
+/*
+ * Redirection for stdout and stderr.
+ */
+
+int
+weechat_lua_output (lua_State *L)
+{
+    const char *msg, *ptr_msg, *ptr_newline;
+
+    if (lua_gettop (L) < 1)
+        return 0;
+
+    msg = lua_tostring (L, -1);
+    ptr_msg = msg;
+    while ((ptr_newline = strchr (ptr_msg, '\n')) != NULL)
+    {
+        weechat_string_dyn_concat (lua_buffer_output,
+                                   ptr_msg,
+                                   ptr_newline - ptr_msg);
+        weechat_lua_output_flush ();
+        ptr_msg = ++ptr_newline;
+    }
+    weechat_string_dyn_concat (lua_buffer_output, ptr_msg, -1);
+
+    return 0;
+}
+
 /*
  * Executes a lua function.
  */
@@ -164,7 +286,7 @@ weechat_lua_exec (struct t_plugin_script *script, int ret_type,
                   const char *function, const char *format, void **argv)
 {
     void *ret_value;
-    int argc, i, *ret_i;
+    int argc, i, *ret_i, rc;
     lua_State *old_lua_current_interpreter;
     struct t_plugin_script *old_lua_current_script;
 
@@ -205,7 +327,11 @@ weechat_lua_exec (struct t_plugin_script *script, int ret_type,
 
     ret_value = NULL;
 
-    if (lua_pcall (lua_current_interpreter, argc, 1, 0) == 0)
+    rc = lua_pcall (lua_current_interpreter, argc, 1, 0);
+
+    weechat_lua_output_flush ();
+
+    if (rc == 0)
     {
         if (ret_type == WEECHAT_SCRIPT_EXEC_STRING)
         {
@@ -223,6 +349,24 @@ weechat_lua_exec (struct t_plugin_script *script, int ret_type,
                                 function);
             }
         }
+        else if (ret_type == WEECHAT_SCRIPT_EXEC_POINTER)
+        {
+            ret_value = (char *) lua_tostring (lua_current_interpreter, -1);
+            if (ret_value)
+            {
+                ret_value = plugin_script_str2ptr (weechat_lua_plugin,
+                                                   script->name, function,
+                                                   ret_value);
+            }
+            else
+            {
+                weechat_printf (NULL,
+                                weechat_gettext ("%s%s: function \"%s\" must "
+                                                 "return a valid value"),
+                                weechat_prefix ("error"), LUA_PLUGIN_NAME,
+                                function);
+            }
+        }
         else if (ret_type == WEECHAT_SCRIPT_EXEC_INT)
         {
             ret_i = malloc (sizeof (*ret_i));
@@ -239,7 +383,14 @@ weechat_lua_exec (struct t_plugin_script *script, int ret_type,
         }
         else
         {
-            WEECHAT_SCRIPT_MSG_WRONG_ARGS(LUA_CURRENT_SCRIPT_NAME, function);
+            if (ret_type != WEECHAT_SCRIPT_EXEC_IGNORE)
+            {
+                weechat_printf (NULL,
+                                weechat_gettext ("%s%s: function \"%s\" must "
+                                                 "return a valid value"),
+                                weechat_prefix ("error"), LUA_PLUGIN_NAME,
+                                function);
+            }
         }
     }
     else
@@ -253,6 +404,13 @@ weechat_lua_exec (struct t_plugin_script *script, int ret_type,
                         lua_tostring (lua_current_interpreter, -1));
     }
 
+    if ((ret_type != WEECHAT_SCRIPT_EXEC_IGNORE) && !ret_value)
+    {
+        weechat_printf (NULL,
+                        weechat_gettext ("%s%s: error in function \"%s\""),
+                        weechat_prefix ("error"), LUA_PLUGIN_NAME, function);
+    }
+
     lua_pop (lua_current_interpreter, 1);
 
     lua_current_script = old_lua_current_script;
@@ -277,7 +435,7 @@ weechat_lua_add_constant (lua_State *L, struct t_lua_const *ptr_const)
 #else
         lua_pushnumber (L, ptr_const->int_value);
 #endif /* LUA_VERSION_NUM >= 503 */
-    lua_settable(L, -3);
+    lua_settable (L, -3);
 }
 
 /*
@@ -287,7 +445,7 @@ weechat_lua_add_constant (lua_State *L, struct t_lua_const *ptr_const)
 int
 weechat_lua_newindex (lua_State *L)
 {
-    luaL_error(L, "Error: read-only constant");
+    luaL_error (L, "Error: read-only constant");
 
     return 0;
 }
@@ -338,31 +496,42 @@ weechat_lua_register_lib (lua_State *L, const char *libname,
 /*
  * Loads a lua script.
  *
- * Returns:
- *   1: OK
- *   0: error
+ * If code is NULL, the content of filename is read and executed.
+ * If code is not NULL, it is executed (the file is not read).
+ *
+ * Returns pointer to new registered script, NULL if error.
  */
 
-int
-weechat_lua_load (const char *filename)
+struct t_plugin_script *
+weechat_lua_load (const char *filename, const char *code)
 {
     FILE *fp;
-    char *weechat_lua_code = {
+    char *lua_redirect_output = {
+        "function weechat_output_string(str)\n"
+        "    weechat.__output__(str)\n"
+        "end\n"
         "weechat_outputs = {\n"
-        "    write = function (self, str)\n"
-        "        weechat.print(\"\", \"lua: stdout/stderr: \" .. str)\n"
-        "    end\n"
+        "    write = weechat_output_string\n"
         "}\n"
         "io.stdout = weechat_outputs\n"
         "io.stderr = weechat_outputs\n"
+        "io.write = weechat_output_string\n"
+        "print = weechat_output_string\n"
     };
 
-    if ((fp = fopen (filename, "r")) == NULL)
+    fp = NULL;
+
+    if (!code)
     {
-        weechat_printf (NULL,
-                        weechat_gettext ("%s%s: script \"%s\" not found"),
-                        weechat_prefix ("error"), LUA_PLUGIN_NAME, filename);
-        return 0;
+        fp = fopen (filename, "r");
+        if (!fp)
+        {
+            weechat_printf (NULL,
+                            weechat_gettext ("%s%s: script \"%s\" not found"),
+                            weechat_prefix ("error"), LUA_PLUGIN_NAME,
+                            filename);
+            return NULL;
+        }
     }
 
     if ((weechat_lua_plugin->debug >= 2) || !lua_quiet)
@@ -375,7 +544,7 @@ weechat_lua_load (const char *filename)
     lua_current_script = NULL;
     lua_registered_script = NULL;
 
-    lua_current_interpreter = luaL_newstate();
+    lua_current_interpreter = luaL_newstate ();
 
     if (lua_current_interpreter == NULL)
     {
@@ -383,8 +552,9 @@ weechat_lua_load (const char *filename)
                         weechat_gettext ("%s%s: unable to create new "
                                          "sub-interpreter"),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME);
-        fclose (fp);
-        return 0;
+        if (fp)
+            fclose (fp);
+        return NULL;
     }
 
 #ifdef LUA_VERSION_NUM /* LUA_VERSION_NUM is defined only in lua >= 5.1.0 */
@@ -403,9 +573,9 @@ weechat_lua_load (const char *filename)
                               weechat_lua_api_consts);
 
 #ifdef LUA_VERSION_NUM
-    if (luaL_dostring (lua_current_interpreter, weechat_lua_code) != 0)
+    if (luaL_dostring (lua_current_interpreter, lua_redirect_output) != 0)
 #else
-    if (lua_dostring (lua_current_interpreter, weechat_lua_code) != 0)
+    if (lua_dostring (lua_current_interpreter, lua_redirect_output) != 0)
 #endif /* LUA_VERSION_NUM */
     {
         weechat_printf (NULL,
@@ -416,32 +586,65 @@ weechat_lua_load (const char *filename)
 
     lua_current_script_filename = filename;
 
-    if (luaL_loadfile (lua_current_interpreter, filename) != 0)
+    if (code)
     {
-        weechat_printf (NULL,
-                        weechat_gettext ("%s%s: unable to load file \"%s\""),
-                        weechat_prefix ("error"), LUA_PLUGIN_NAME, filename);
-        weechat_printf (NULL,
-                        weechat_gettext ("%s%s: error: %s"),
-                        weechat_prefix ("error"), LUA_PLUGIN_NAME,
-                        lua_tostring (lua_current_interpreter, -1));
-        lua_close (lua_current_interpreter);
-        fclose (fp);
-        return 0;
+        /* execute code without reading file */
+        if (luaL_loadstring (lua_current_interpreter, code) != 0)
+        {
+            weechat_printf (NULL,
+                            weechat_gettext ("%s%s: unable to load source "
+                                             "code"),
+                            weechat_prefix ("error"), LUA_PLUGIN_NAME);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s%s: error: %s"),
+                            weechat_prefix ("error"), LUA_PLUGIN_NAME,
+                            lua_tostring (lua_current_interpreter, -1));
+            lua_close (lua_current_interpreter);
+            return NULL;
+        }
+    }
+    else
+    {
+        /* read and execute code from file */
+        if (luaL_loadfile (lua_current_interpreter, filename) != 0)
+        {
+            weechat_printf (NULL,
+                            weechat_gettext ("%s%s: unable to load file \"%s\""),
+                            weechat_prefix ("error"), LUA_PLUGIN_NAME, filename);
+            weechat_printf (NULL,
+                            weechat_gettext ("%s%s: error: %s"),
+                            weechat_prefix ("error"), LUA_PLUGIN_NAME,
+                            lua_tostring (lua_current_interpreter, -1));
+            lua_close (lua_current_interpreter);
+            fclose (fp);
+            return NULL;
+        }
     }
 
     if (lua_pcall (lua_current_interpreter, 0, 0, 0) != 0)
     {
-        weechat_printf (NULL,
-                        weechat_gettext ("%s%s: unable to execute file "
-                                         "\"%s\""),
-                        weechat_prefix ("error"), LUA_PLUGIN_NAME, filename);
+        if (code)
+        {
+            weechat_printf (NULL,
+                            weechat_gettext ("%s%s: unable to execute source "
+                                             "code"),
+                            weechat_prefix ("error"), LUA_PLUGIN_NAME);
+        }
+        else
+        {
+            weechat_printf (NULL,
+                            weechat_gettext ("%s%s: unable to execute file "
+                                             "\"%s\""),
+                            weechat_prefix ("error"), LUA_PLUGIN_NAME,
+                            filename);
+        }
         weechat_printf (NULL,
                         weechat_gettext ("%s%s: error: %s"),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME,
                         lua_tostring (lua_current_interpreter, -1));
         lua_close (lua_current_interpreter);
-        fclose (fp);
+        if (fp)
+            fclose (fp);
 
         /* if script was registered, remove it from list */
         if (lua_current_script)
@@ -452,9 +655,11 @@ weechat_lua_load (const char *filename)
             lua_current_script = NULL;
         }
 
-        return 0;
+        return NULL;
     }
-    fclose (fp);
+
+    if (fp)
+        fclose (fp);
 
     if (!lua_registered_script)
     {
@@ -463,7 +668,7 @@ weechat_lua_load (const char *filename)
                                          "found (or failed) in file \"%s\""),
                         weechat_prefix ("error"), LUA_PLUGIN_NAME, filename);
         lua_close (lua_current_interpreter);
-        return 0;
+        return NULL;
     }
     lua_current_script = lua_registered_script;
 
@@ -481,7 +686,7 @@ weechat_lua_load (const char *filename)
                                      WEECHAT_HOOK_SIGNAL_STRING,
                                      lua_current_script->filename);
 
-    return 1;
+    return lua_current_script;
 }
 
 /*
@@ -494,7 +699,7 @@ weechat_lua_load_cb (void *data, const char *filename)
     /* make C compiler happy */
     (void) data;
 
-    weechat_lua_load (filename);
+    weechat_lua_load (filename, NULL);
 }
 
 /*
@@ -597,7 +802,7 @@ weechat_lua_reload_name (const char *name)
                                 weechat_gettext ("%s: script \"%s\" unloaded"),
                                 LUA_PLUGIN_NAME, name);
             }
-            weechat_lua_load (filename);
+            weechat_lua_load (filename, NULL);
             free (filename);
         }
     }
@@ -622,6 +827,64 @@ weechat_lua_unload_all ()
     }
 }
 
+/*
+ * Evaluates lua source code.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+weechat_lua_eval (struct t_gui_buffer *buffer, int send_to_buffer_as_input,
+                  int exec_commands, const char *code)
+{
+    void *func_argv[1], *result;
+
+    if (!lua_script_eval)
+    {
+        lua_quiet = 1;
+        lua_script_eval = weechat_lua_load (WEECHAT_SCRIPT_EVAL_NAME,
+                                            LUA_EVAL_SCRIPT);
+        lua_quiet = 0;
+        if (!lua_script_eval)
+            return 0;
+    }
+
+    weechat_lua_output_flush ();
+
+    lua_eval_mode = 1;
+    lua_eval_send_input = send_to_buffer_as_input;
+    lua_eval_exec_commands = exec_commands;
+    lua_eval_buffer = buffer;
+
+    func_argv[0] = (char *)code;
+    result = weechat_lua_exec (lua_script_eval,
+                               WEECHAT_SCRIPT_EXEC_IGNORE,
+                               "script_lua_eval",
+                               "s", func_argv);
+    /* result is ignored */
+    if (result)
+        free (result);
+
+    weechat_lua_output_flush ();
+
+    lua_eval_mode = 0;
+    lua_eval_send_input = 0;
+    lua_eval_exec_commands = 0;
+    lua_eval_buffer = NULL;
+
+    if (!weechat_config_boolean (lua_config_look_eval_keep_context))
+    {
+        lua_quiet = 1;
+        weechat_lua_unload (lua_script_eval);
+        lua_quiet = 0;
+        lua_script_eval = NULL;
+    }
+
+    return 1;
+}
+
 /*
  * Callback for command "/lua".
  */
@@ -631,12 +894,12 @@ weechat_lua_command_cb (const void *pointer, void *data,
                         struct t_gui_buffer *buffer,
                         int argc, char **argv, char **argv_eol)
 {
-    char *ptr_name, *path_script;
+    char *ptr_name, *ptr_code, *path_script;
+    int i, send_to_buffer_as_input, exec_commands;
 
     /* make C compiler happy */
     (void) pointer;
     (void) data;
-    (void) buffer;
 
     if (argc == 1)
     {
@@ -668,6 +931,10 @@ weechat_lua_command_cb (const void *pointer, void *data,
         {
             weechat_lua_unload_all ();
         }
+        else if (weechat_strcasecmp (argv[1], "version") == 0)
+        {
+            plugin_script_display_interpreter (weechat_lua_plugin, 0);
+        }
         else
             WEECHAT_COMMAND_ERROR;
     }
@@ -702,7 +969,8 @@ weechat_lua_command_cb (const void *pointer, void *data,
                 /* load lua script */
                 path_script = plugin_script_search_path (weechat_lua_plugin,
                                                          ptr_name);
-                weechat_lua_load ((path_script) ? path_script : ptr_name);
+                weechat_lua_load ((path_script) ? path_script : ptr_name,
+                                  NULL);
                 if (path_script)
                     free (path_script);
             }
@@ -718,6 +986,39 @@ weechat_lua_command_cb (const void *pointer, void *data,
             }
             lua_quiet = 0;
         }
+        else if (weechat_strcasecmp (argv[1], "eval") == 0)
+        {
+            send_to_buffer_as_input = 0;
+            exec_commands = 0;
+            ptr_code = argv_eol[2];
+            for (i = 2; i < argc; i++)
+            {
+                if (argv[i][0] == '-')
+                {
+                    if (strcmp (argv[i], "-o") == 0)
+                    {
+                        if (i + 1 >= argc)
+                            WEECHAT_COMMAND_ERROR;
+                        send_to_buffer_as_input = 1;
+                        exec_commands = 0;
+                        ptr_code = argv_eol[i + 1];
+                    }
+                    else if (strcmp (argv[i], "-oc") == 0)
+                    {
+                        if (i + 1 >= argc)
+                            WEECHAT_COMMAND_ERROR;
+                        send_to_buffer_as_input = 1;
+                        exec_commands = 1;
+                        ptr_code = argv_eol[i + 1];
+                    }
+                }
+                else
+                    break;
+            }
+            if (!weechat_lua_eval (buffer, send_to_buffer_as_input,
+                                   exec_commands, ptr_code))
+                WEECHAT_COMMAND_ERROR;
+        }
         else
             WEECHAT_COMMAND_ERROR;
     }
@@ -763,6 +1064,29 @@ weechat_lua_hdata_cb (const void *pointer, void *data,
                                        hdata_name);
 }
 
+/*
+ * Returns lua info "lua_eval".
+ */
+
+char *
+weechat_lua_info_eval_cb (const void *pointer, void *data,
+                          const char *info_name,
+                          const char *arguments)
+{
+    char *output;
+
+    /* make C compiler happy */
+    (void) pointer;
+    (void) data;
+    (void) info_name;
+
+    weechat_lua_eval (NULL, 0, 0, (arguments) ? arguments : "");
+    output = strdup (*lua_buffer_output);
+    weechat_string_dyn_copy (lua_buffer_output, NULL);
+
+    return output;
+}
+
 /*
  * Returns infolist with lua scripts.
  */
@@ -813,31 +1137,6 @@ weechat_lua_signal_debug_dump_cb (const void *pointer, void *data,
     return WEECHAT_RC_OK;
 }
 
-/*
- * Display infos about external libraries used.
- */
-
-int
-weechat_lua_signal_debug_libs_cb (const void *pointer, void *data,
-                                  const char *signal,
-                                  const char *type_data, void *signal_data)
-{
-    /* make C compiler happy */
-    (void) pointer;
-    (void) data;
-    (void) signal;
-    (void) type_data;
-    (void) signal_data;
-
-#ifdef LUA_VERSION
-    weechat_printf (NULL, "  %s: %s", LUA_PLUGIN_NAME, LUA_VERSION);
-#else
-    weechat_printf (NULL, "  %s: (?)", LUA_PLUGIN_NAME);
-#endif /* LUA_VERSION */
-
-    return WEECHAT_RC_OK;
-}
-
 /*
  * Timer for executing actions.
  */
@@ -932,21 +1231,41 @@ weechat_lua_signal_script_action_cb (const void *pointer, void *data,
 int
 weechat_plugin_init (struct t_weechat_plugin *plugin, int argc, char *argv[])
 {
-    struct t_plugin_script_init init;
-
     weechat_lua_plugin = plugin;
 
-    init.callback_command = &weechat_lua_command_cb;
-    init.callback_completion = &weechat_lua_completion_cb;
-    init.callback_hdata = &weechat_lua_hdata_cb;
-    init.callback_infolist = &weechat_lua_infolist_cb;
-    init.callback_signal_debug_dump = &weechat_lua_signal_debug_dump_cb;
-    init.callback_signal_debug_libs = &weechat_lua_signal_debug_libs_cb;
-    init.callback_signal_script_action = &weechat_lua_signal_script_action_cb;
-    init.callback_load_file = &weechat_lua_load_cb;
+    /* set interpreter name and version */
+    weechat_hashtable_set (plugin->variables, "interpreter_name",
+                           plugin->name);
+#ifdef LUA_VERSION
+    weechat_hashtable_set (plugin->variables, "interpreter_version",
+                           LUA_VERSION);
+#else
+    weechat_hashtable_set (plugin->variables, "interpreter_version",
+                           "");
+#endif /* LUA_VERSION */
+
+    /* init stdout/stderr buffer */
+    lua_buffer_output = weechat_string_dyn_alloc (256);
+    if (!lua_buffer_output)
+        return WEECHAT_RC_ERROR;
+
+    lua_data.config_file = &lua_config_file;
+    lua_data.config_look_check_license = &lua_config_look_check_license;
+    lua_data.config_look_eval_keep_context = &lua_config_look_eval_keep_context;
+    lua_data.scripts = &lua_scripts;
+    lua_data.last_script = &last_lua_script;
+    lua_data.callback_command = &weechat_lua_command_cb;
+    lua_data.callback_completion = &weechat_lua_completion_cb;
+    lua_data.callback_hdata = &weechat_lua_hdata_cb;
+    lua_data.callback_info_eval = &weechat_lua_info_eval_cb;
+    lua_data.callback_infolist = &weechat_lua_infolist_cb;
+    lua_data.callback_signal_debug_dump = &weechat_lua_signal_debug_dump_cb;
+    lua_data.callback_signal_script_action = &weechat_lua_signal_script_action_cb;
+    lua_data.callback_load_file = &weechat_lua_load_cb;
+    lua_data.unload_all = &weechat_lua_unload_all;
 
     lua_quiet = 1;
-    plugin_script_init (weechat_lua_plugin, argc, argv, &init);
+    plugin_script_init (weechat_lua_plugin, argc, argv, &lua_data);
     lua_quiet = 0;
 
     plugin_script_display_short_list (weechat_lua_plugin,
@@ -965,7 +1284,12 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
 {
     /* unload all scripts */
     lua_quiet = 1;
-    plugin_script_end (plugin, &lua_scripts, &weechat_lua_unload_all);
+    if (lua_script_eval)
+    {
+        weechat_lua_unload (lua_script_eval);
+        lua_script_eval = NULL;
+    }
+    plugin_script_end (plugin, &lua_data);
     lua_quiet = 0;
 
     /* free some data */
@@ -975,6 +1299,7 @@ weechat_plugin_end (struct t_weechat_plugin *plugin)
         free (lua_action_remove_list);
     if (lua_action_autoload_list)
         free (lua_action_autoload_list);
+    weechat_string_dyn_free (lua_buffer_output, 1);
 
     return WEECHAT_RC_OK;
 }
