@@ -162,6 +162,39 @@ plugin_api_crypto_hash_pbkdf2 (const void *data, int data_size,
                                   iterations, hash, hash_size);
 }
 
+/*
+ * Computes HMAC of key + message using the given algorithm.
+ *
+ * Returns:
+ *   1: OK
+ *   0: error
+ */
+
+int
+plugin_api_crypto_hmac (const void *key, int key_size,
+                        const void *message, int message_size,
+                        const char *hash_algo,
+                        void *hash, int *hash_size)
+{
+    int algo;
+
+    if (!hash)
+        return 0;
+
+    if (hash_size)
+        *hash_size = 0;
+
+    if (!key || (key_size < 1) || !message || (message_size < 1) || !hash_algo)
+        return 0;
+
+    algo = weecrypto_get_hash_algo (hash_algo);
+    if (algo == GCRY_MD_NONE)
+        return 0;
+
+    return weecrypto_hmac (key, key_size, message, message_size,
+                           algo, hash, hash_size);
+}
+
 /*
  * Frees an option.
  */
@@ -431,13 +464,36 @@ plugin_api_modifier_eval_path_home_cb (const void *pointer, void *data,
                                        const char *modifier_data,
                                        const char *string)
 {
+    struct t_hashtable *options;
+    const char *ptr_directory;
+    char *result;
+
     /* make C compiler happy */
     (void) pointer;
     (void) data;
     (void) modifier;
-    (void) modifier_data;
 
-    return string_eval_path_home (string, NULL, NULL, NULL);
+    options = NULL;
+    ptr_directory = (modifier_data
+                     && (strncmp (modifier_data, "directory=", 10) == 0)) ?
+        modifier_data + 10 : NULL;
+    if (ptr_directory)
+    {
+        options = hashtable_new (
+            32,
+            WEECHAT_HASHTABLE_STRING,
+            WEECHAT_HASHTABLE_STRING,
+            NULL, NULL);
+        if (options)
+            hashtable_set (options, "directory", ptr_directory);
+    }
+
+    result = string_eval_path_home (string, NULL, NULL, options);
+
+    if (options)
+        hashtable_free (options);
+
+    return result;
 }
 
 /*
