@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_PLUGIN_SCRIPT_H
@@ -26,11 +26,15 @@ enum t_weechat_script_exec_type
 {
     WEECHAT_SCRIPT_EXEC_INT = 0,
     WEECHAT_SCRIPT_EXEC_STRING,
+    WEECHAT_SCRIPT_EXEC_POINTER,
     WEECHAT_SCRIPT_EXEC_HASHTABLE,
+    WEECHAT_SCRIPT_EXEC_IGNORE,
 };
 
 #define WEECHAT_SCRIPT_HASHTABLE_DEFAULT_SIZE 16
 
+#define WEECHAT_SCRIPT_EVAL_NAME "__eval__"
+
 #define WEECHAT_SCRIPT_MSG_NOT_INIT(__current_script,                   \
                                     __function)                         \
     weechat_printf (NULL,                                               \
@@ -66,44 +70,57 @@ struct t_plugin_script
     struct t_plugin_script *next_script; /* link to next script             */
 };
 
-struct t_plugin_script_init
+struct t_plugin_script_data
 {
-    int (*callback_command)(const void *pointer, void *data,
-                            struct t_gui_buffer *buffer,
-                            int argc, char **argv, char **argv_eol);
-    int (*callback_completion)(const void *pointer, void *data,
-                               const char *completion_item,
-                               struct t_gui_buffer *buffer,
-                               struct t_gui_completion *completion);
-    struct t_hdata *(*callback_hdata)(const void *pointer,
-                                      void *data,
-                                      const char *hdata_name);
-    struct t_infolist *(*callback_infolist)(const void *pointer,
-                                            void *data,
-                                            const char *infolist_name,
-                                            void *obj_pointer,
-                                            const char *arguments);
-    int (*callback_signal_debug_dump)(const void *pointer, void *data,
-                                      const char *signal,
-                                      const char *type_data,
-                                      void *signal_data);
-    int (*callback_signal_debug_libs)(const void *pointer, void *data,
-                                      const char *signal,
-                                      const char *type_data,
-                                      void *signal_data);
-    int (*callback_signal_script_action)(const void *pointer, void *data,
-                                         const char *signal,
-                                         const char *type_data,
-                                         void *signal_data);
-    void (*callback_load_file)(void *data, const char *filename);
+    /* variables */
+    struct t_config_file **config_file;
+    struct t_config_option **config_look_check_license;
+    struct t_config_option **config_look_eval_keep_context;
+    struct t_plugin_script **scripts;
+    struct t_plugin_script **last_script;
+
+    /* callbacks */
+    int (*callback_command) (const void *pointer, void *data,
+                             struct t_gui_buffer *buffer,
+                             int argc, char **argv, char **argv_eol);
+    int (*callback_completion) (const void *pointer, void *data,
+                                const char *completion_item,
+                                struct t_gui_buffer *buffer,
+                                struct t_gui_completion *completion);
+    struct t_hdata *(*callback_hdata) (const void *pointer,
+                                       void *data,
+                                       const char *hdata_name);
+    char *(*callback_info_eval) (const void *pointer,
+                                 void *data,
+                                 const char *info_name,
+                                 const char *arguments);
+    struct t_infolist *(*callback_infolist) (const void *pointer,
+                                             void *data,
+                                             const char *infolist_name,
+                                             void *obj_pointer,
+                                             const char *arguments);
+    int (*callback_signal_debug_dump) (const void *pointer, void *data,
+                                       const char *signal,
+                                       const char *type_data,
+                                       void *signal_data);
+    int (*callback_signal_script_action) (const void *pointer, void *data,
+                                          const char *signal,
+                                          const char *type_data,
+                                          void *signal_data);
+    void (*callback_load_file) (void *data, const char *filename);
+
+    /* functions */
+    void (*unload_all) ();
 };
 
+extern void plugin_script_display_interpreter (struct t_weechat_plugin *plugin,
+                                               int indent);
 extern void plugin_script_init (struct t_weechat_plugin *weechat_plugin,
                                 int argc, char *argv[],
-                                struct t_plugin_script_init *init);
+                                struct t_plugin_script_data *plugin_data);
 extern int plugin_script_valid (struct t_plugin_script *scripts,
                                 struct t_plugin_script *script);
-extern char *plugin_script_ptr2str (void *pointer);
+extern const char *plugin_script_ptr2str (void *pointer);
 extern void *plugin_script_str2ptr (struct t_weechat_plugin *weechat_plugin,
                                     const char *script_name,
                                     const char *function_name,
@@ -121,13 +138,24 @@ extern struct t_plugin_script *plugin_script_search (struct t_weechat_plugin *we
                                                      const char *name);
 extern char *plugin_script_search_path (struct t_weechat_plugin *weechat_plugin,
                                         const char *filename);
+extern struct t_plugin_script *plugin_script_alloc (const char *filename,
+                                                    const char *name,
+                                                    const char *author,
+                                                    const char *version,
+                                                    const char *license,
+                                                    const char *description,
+                                                    const char *shutdown_func,
+                                                    const char *charset);
 extern struct t_plugin_script *plugin_script_add (struct t_weechat_plugin *weechat_plugin,
-                                                  struct t_plugin_script **scripts,
-                                                  struct t_plugin_script **last_script,
-                                                  const char *filename, const char *name,
-                                                  const char *author, const char *version,
-                                                  const char *license, const char *description,
-                                                  const char *shutdown_func, const char *charset);
+                                                  struct t_plugin_script_data *plugin_data,
+                                                  const char *filename,
+                                                  const char *name,
+                                                  const char *author,
+                                                  const char *version,
+                                                  const char *license,
+                                                  const char *description,
+                                                  const char *shutdown_func,
+                                                  const char *charset);
 extern void plugin_script_set_buffer_callbacks (struct t_weechat_plugin *weechat_plugin,
                                                 struct t_plugin_script *scripts,
                                                 struct t_plugin_script *script,
@@ -138,6 +166,7 @@ extern void plugin_script_set_buffer_callbacks (struct t_weechat_plugin *weechat
                                                 int (*callback_buffer_close) (const void *pointer,
                                                                               void *data,
                                                                               struct t_gui_buffer *buffer));
+extern void plugin_script_free (struct t_plugin_script *script);
 extern void plugin_script_remove (struct t_weechat_plugin *weechat_plugin,
                                   struct t_plugin_script **scripts,
                                   struct t_plugin_script **last_script,
@@ -149,7 +178,8 @@ extern void plugin_script_action_add (char **action_list, const char *name);
 extern void plugin_script_action_install (struct t_weechat_plugin *weechat_plugin,
                                           struct t_plugin_script *scripts,
                                           void (*script_unload)(struct t_plugin_script *script),
-                                          int (*script_load)(const char *filename),
+                                          struct t_plugin_script *(*script_load)(const char *filename,
+                                                                                 const char *code),
                                           int *quiet,
                                           char **list);
 extern void plugin_script_action_remove (struct t_weechat_plugin *weechat_plugin,
@@ -177,8 +207,7 @@ extern struct t_infolist *plugin_script_infolist_list_scripts (struct t_weechat_
                                                                void *pointer,
                                                                const char *arguments);
 extern void plugin_script_end (struct t_weechat_plugin *weechat_plugin,
-                               struct t_plugin_script **scripts,
-                               void (*callback_unload_all)());
+                               struct t_plugin_script_data *plugin_data);
 extern void plugin_script_print_log (struct t_weechat_plugin *weechat_plugin,
                                      struct t_plugin_script *scripts);
 
