@@ -53,12 +53,12 @@ struct t_relay_server *last_relay_server = NULL;
  *
  *   string                    ipv4 ipv6 ssl unix protocol protocol_args
  *   ---------------------------------------------------------------
- *   irc.freenode              1    1    0   0    irc      freenode
- *   ssl.irc.freenode          1    1    1   0    irc      freenode
- *   ipv4.irc.freenode         1    0    0   0    irc      freenode
- *   ipv6.irc.freenode         0    1    0   0    irc      freenode
- *   ipv4.ipv6.irc.freenode    1    1    0   0    irc      freenode
- *   ipv6.ssl.irc.freenode     0    1    1   0    irc      freenode
+ *   irc.libera                1    1    0   0    irc      libera
+ *   ssl.irc.libera            1    1    1   0    irc      libera
+ *   ipv4.irc.libera           1    0    0   0    irc      libera
+ *   ipv6.irc.libera           0    1    0   0    irc      libera
+ *   ipv4.ipv6.irc.libera      1    1    0   0    irc      libera
+ *   ipv6.ssl.irc.libera       0    1    1   0    irc      libera
  *   weechat                   1    1    0   0    weechat
  *   ssl.weechat               1    1    1   0    weechat
  *   ipv6.ssl.weechat          0    1    1   0    weechat
@@ -276,7 +276,6 @@ relay_server_sock_cb (const void *pointer, void *data, int fd)
     (void) data;
     (void) fd;
 
-    client_fd = -1;
     relay_password = NULL;
     relay_totp_secret = NULL;
 
@@ -791,6 +790,7 @@ relay_server_new (const char *protocol_string, enum t_relay_protocol protocol,
                   int ipv4, int ipv6, int ssl, int unix_socket)
 {
     struct t_relay_server *new_server, *dup_server;
+    struct t_hashtable *options;
 
     if (!protocol_string)
         return NULL;
@@ -823,8 +823,17 @@ relay_server_new (const char *protocol_string, enum t_relay_protocol protocol,
         new_server->protocol_args =
             (protocol_args) ? strdup (protocol_args) : NULL;
         new_server->port = port;
+        options = weechat_hashtable_new (
+            32,
+            WEECHAT_HASHTABLE_STRING,
+            WEECHAT_HASHTABLE_STRING,
+            NULL, NULL);
+        if (options)
+            weechat_hashtable_set (options, "directory", "runtime");
         new_server->path = weechat_string_eval_path_home (path,
-                                                          NULL, NULL, NULL);
+                                                          NULL, NULL, options);
+        if (options)
+            weechat_hashtable_free (options);
         new_server->ipv4 = ipv4;
         new_server->ipv6 = ipv6;
         new_server->ssl = ssl;
@@ -862,8 +871,18 @@ void
 relay_server_update_path (struct t_relay_server *server, const char *path)
 {
     char *new_path;
+    struct t_hashtable *options;
 
-    new_path = weechat_string_eval_path_home (path, NULL, NULL, NULL);
+    options = weechat_hashtable_new (
+        32,
+        WEECHAT_HASHTABLE_STRING,
+        WEECHAT_HASHTABLE_STRING,
+        NULL, NULL);
+    if (options)
+        weechat_hashtable_set (options, "directory", "runtime");
+    new_path = weechat_string_eval_path_home (path, NULL, NULL, options);
+    if (options)
+        weechat_hashtable_free (options);
     if (!new_path)
         return;
 
