@@ -1,7 +1,7 @@
 /*
  * relay-websocket.c - websocket server functions for relay plugin (RFC 6455)
  *
- * Copyright (C) 2013-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2013-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,19 +16,19 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <string.h>
-#include <gcrypt.h>
 
 #include "../weechat-plugin.h"
 #include "relay.h"
 #include "relay-client.h"
 #include "relay-config.h"
+#include "relay-websocket.h"
 
 
 /*
@@ -85,10 +85,11 @@ relay_websocket_save_header (struct t_relay_client *client,
     if (!pos || (pos == message))
         return;
 
-    /* get header name */
+    /* get header name, which is case-insensitive */
     name = weechat_strndup (message, pos - message);
     if (!name)
         return;
+    weechat_string_tolower (name);
 
     /* get pointer on header value */
     ptr_value = pos + 1;
@@ -111,7 +112,7 @@ relay_websocket_save_header (struct t_relay_client *client,
  *   Upgrade: websocket
  *   Connection: Upgrade
  *   Host: myhost:5000
- *   Origin: http://example.org.org
+ *   Origin: https://example.org
  *   Pragma: no-cache
  *   Cache-Control: no-cache
  *   Sec-WebSocket-Key: fo1J9uHSsrfDP3BkwUylzQ==
@@ -142,20 +143,20 @@ relay_websocket_client_handshake_valid (struct t_relay_client *client)
     const char *value;
 
     /* check if we have header "Upgrade" with value "websocket" */
-    value = weechat_hashtable_get (client->http_headers, "Upgrade");
+    value = weechat_hashtable_get (client->http_headers, "upgrade");
     if (!value)
         return -1;
     if (weechat_strcasecmp (value, "websocket") != 0)
         return -1;
 
     /* check if we have header "Sec-WebSocket-Key" with non-empty value */
-    value = weechat_hashtable_get (client->http_headers, "Sec-WebSocket-Key");
+    value = weechat_hashtable_get (client->http_headers, "sec-websocket-key");
     if (!value || !value[0])
         return -1;
 
     if (relay_config_regex_websocket_allowed_origins)
     {
-        value = weechat_hashtable_get (client->http_headers, "Origin");
+        value = weechat_hashtable_get (client->http_headers, "origin");
         if (!value || !value[0])
             return -2;
         if (regexec (relay_config_regex_websocket_allowed_origins, value, 0,
@@ -186,13 +187,11 @@ char *
 relay_websocket_build_handshake (struct t_relay_client *client)
 {
     const char *sec_websocket_key;
-    char *key, sec_websocket_accept[128], handshake[1024];
-    unsigned char *result;
-    gcry_md_hd_t hd;
-    int length;
+    char *key, sec_websocket_accept[128], handshake[1024], hash[160 / 8];
+    int length, hash_size;
 
     sec_websocket_key = weechat_hashtable_get (client->http_headers,
-                                               "Sec-WebSocket-Key");
+                                               "sec-websocket-key");
     if (!sec_websocket_key || !sec_websocket_key[0])
         return NULL;
 
@@ -208,12 +207,16 @@ relay_websocket_build_handshake (struct t_relay_client *client)
     snprintf (key, length, "%s%s", sec_websocket_key, WEBSOCKET_GUID);
 
     /* compute 160-bit SHA1 on the key and encode it with base64 */
-    gcry_md_open (&hd, GCRY_MD_SHA1, 0);
-    length = gcry_md_get_algo_dlen (GCRY_MD_SHA1);
-    gcry_md_write (hd, key, strlen (key));
-    result = gcry_md_read (hd, GCRY_MD_SHA1);
-    weechat_string_encode_base64 ((char *)result, length, sec_websocket_accept);
-    gcry_md_close (hd);
+    if (!weechat_crypto_hash (key, strlen (key), "sha1", hash, &hash_size))
+    {
+        free (key);
+        return NULL;
+    }
+    if (weechat_string_base_encode (64, hash, hash_size,
+                                    sec_websocket_accept) < 0)
+    {
+        sec_websocket_accept[0] = '\0';
+    }
 
     free (key);
 
@@ -222,7 +225,6 @@ relay_websocket_build_handshake (struct t_relay_client *client)
               "HTTP/1.1 101 Switching Protocols\r\n"
               "Upgrade: websocket\r\n"
               "Connection: Upgrade\r\n"
-              //"Sec-WebSocket-Protocol: chat\r\n"
               "Sec-WebSocket-Accept: %s\r\n"
               "\r\n",
               sec_websocket_accept);
@@ -249,7 +251,8 @@ relay_websocket_send_http (struct t_relay_client *client,
     if (message)
     {
         snprintf (message, length, "HTTP/1.1 %s\r\n\r\n", http);
-        relay_client_send (client, message, strlen (message), NULL);
+        relay_client_send (client, RELAY_CLIENT_MSG_STANDARD,
+                           message, strlen (message), NULL);
         free (message);
     }
 }
@@ -269,13 +272,16 @@ relay_websocket_decode_frame (const unsigned char *buffer,
                               unsigned long long *decoded_length)
 {
     unsigned long long i, index_buffer, length_frame_size, length_frame;
+    unsigned char opcode;
 
     *decoded_length = 0;
     index_buffer = 0;
 
     /* loop to decode all frames in message */
-    while (index_buffer + 2 <= buffer_length)
+    while (index_buffer + 1 < buffer_length)
     {
+        opcode = buffer[index_buffer] & 15;
+
         /*
          * check if frame is masked: client MUST send a masked frame; if frame is
          * not masked, we MUST reject it and close the connection (see RFC 6455)
@@ -287,10 +293,12 @@ relay_websocket_decode_frame (const unsigned char *buffer,
         length_frame_size = 1;
         length_frame = buffer[index_buffer + 1] & 127;
         index_buffer += 2;
+        if (index_buffer >= buffer_length)
+            return 0;
         if ((length_frame == 126) || (length_frame == 127))
         {
             length_frame_size = (length_frame == 126) ? 2 : 8;
-            if (buffer_length < 1 + length_frame_size)
+            if (index_buffer + length_frame_size > buffer_length)
                 return 0;
             length_frame = 0;
             for (i = 0; i < length_frame_size; i++)
@@ -300,10 +308,9 @@ relay_websocket_decode_frame (const unsigned char *buffer,
             index_buffer += length_frame_size;
         }
 
-        if (buffer_length < 1 + length_frame_size + 4 + length_frame)
-            return 0;
-
         /* read masks (4 bytes) */
+        if (index_buffer + 4 > buffer_length)
+            return 0;
         int masks[4];
         for (i = 0; i < 4; i++)
         {
@@ -311,13 +318,33 @@ relay_websocket_decode_frame (const unsigned char *buffer,
         }
         index_buffer += 4;
 
+        /* copy opcode in decoded data */
+        switch (opcode)
+        {
+            case WEBSOCKET_FRAME_OPCODE_PING:
+                decoded[*decoded_length] = RELAY_CLIENT_MSG_PING;
+                break;
+            case WEBSOCKET_FRAME_OPCODE_CLOSE:
+                decoded[*decoded_length] = RELAY_CLIENT_MSG_CLOSE;
+                break;
+            default:
+                decoded[*decoded_length] = RELAY_CLIENT_MSG_STANDARD;
+                break;
+        }
+        *decoded_length += 1;
+
         /* decode data using masks */
+        if ((length_frame > buffer_length)
+            || (index_buffer + length_frame > buffer_length))
+        {
+            return 0;
+        }
         for (i = 0; i < length_frame; i++)
         {
             decoded[*decoded_length + i] = (int)((unsigned char)buffer[index_buffer + i]) ^ masks[i % 4];
         }
         decoded[*decoded_length + length_frame] = '\0';
-        *decoded_length += length_frame;
+        *decoded_length += length_frame + 1;
         index_buffer += length_frame;
     }
 
@@ -334,7 +361,7 @@ relay_websocket_decode_frame (const unsigned char *buffer,
  */
 
 char *
-relay_websocket_encode_frame (struct t_relay_client *client,
+relay_websocket_encode_frame (int opcode,
                               const char *buffer,
                               unsigned long long length,
                               unsigned long long *length_frame)
@@ -348,7 +375,8 @@ relay_websocket_encode_frame (struct t_relay_client *client,
     if (!frame)
         return NULL;
 
-    frame[0] = (client->send_data_type == RELAY_CLIENT_DATA_TEXT) ? 0x81 : 0x82;
+    frame[0] = 0x80;
+    frame[0] |= opcode;
 
     if (length <= 125)
     {
@@ -356,7 +384,7 @@ relay_websocket_encode_frame (struct t_relay_client *client,
         frame[1] = length;
         index = 2;
     }
-    else if ((length >= 126) && (length <= 65535))
+    else if (length <= 65535)
     {
         /* length on 2 bytes */
         frame[1] = 126;
