@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,17 +14,22 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_RELAY_WEECHAT_H
 #define WEECHAT_PLUGIN_RELAY_WEECHAT_H
 
 struct t_relay_client;
+enum t_relay_status;
 
 #define RELAY_WEECHAT_DATA(client, var)                          \
     (((struct t_relay_weechat_data *)client->protocol_data)->var)
 
+#define RELAY_WEECHAT_AUTH_OK(client)                            \
+    ((RELAY_WEECHAT_DATA(client, password_ok)                    \
+      && RELAY_WEECHAT_DATA(client, totp_ok)))
+
 enum t_relay_weechat_compression
 {
     RELAY_WEECHAT_COMPRESSION_OFF = 0, /* no compression of binary objects  */
@@ -35,7 +40,14 @@ enum t_relay_weechat_compression
 
 struct t_relay_weechat_data
 {
+    /* handshake status */
+    int handshake_done;                /* 1 if handshake has been done      */
+
+    /* authentication status (init command) */
     int password_ok;                   /* password received and OK?         */
+    int totp_ok;                       /* TOTP received and OK?             */
+
+    /* options set by client (init command) */
     enum t_relay_weechat_compression compression; /* compression type       */
 
     /* sync of buffers */
@@ -48,6 +60,8 @@ struct t_relay_weechat_data
     struct t_hook *hook_timer_nicklist;   /* timer for sending nicklist     */
 };
 
+extern char *relay_weechat_compression_string[];
+
 extern int relay_weechat_compression_search (const char *compression);
 extern void relay_weechat_hook_signals (struct t_relay_client *client);
 extern void relay_weechat_unhook_signals (struct t_relay_client *client);
@@ -58,6 +72,7 @@ extern void relay_weechat_close_connection (struct t_relay_client *client);
 extern void relay_weechat_alloc (struct t_relay_client *client);
 extern void relay_weechat_alloc_with_infolist (struct t_relay_client *client,
                                                struct t_infolist *infolist);
+extern enum t_relay_status relay_weechat_get_initial_status (struct t_relay_client *client);
 extern void relay_weechat_free (struct t_relay_client *client);
 extern int relay_weechat_add_to_infolist (struct t_infolist_item *item,
                                           struct t_relay_client *client);
