@@ -1,7 +1,7 @@
 /*
  * script-command.c - script command
  *
- * Copyright (C) 2003-2017 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -37,8 +37,9 @@
  */
 
 void
-script_command_action (struct t_gui_buffer *buffer, const char *action,
-                       const char *arguments, int need_repository)
+script_command_action (struct t_gui_buffer *buffer,
+                       const char *action, const char *arguments,
+                       int need_repository, int error_repository)
 {
     struct t_script_repo *ptr_script;
     char str_action[4096];
@@ -71,7 +72,8 @@ script_command_action (struct t_gui_buffer *buffer, const char *action,
                           (quiet) ? "-q " : "",
                           action,
                           ptr_script->name_with_extension);
-                script_action_schedule (str_action, need_repository, quiet);
+                script_action_schedule (str_action, need_repository,
+                                        error_repository, quiet);
             }
         }
         else
@@ -81,7 +83,8 @@ script_command_action (struct t_gui_buffer *buffer, const char *action,
                       (quiet) ? "-q " : "",
                       action,
                       arguments);
-            script_action_schedule (str_action, need_repository, quiet);
+            script_action_schedule (str_action, need_repository,
+                                    error_repository, quiet);
         }
     }
     else if (script_buffer && (buffer == script_buffer))
@@ -95,7 +98,8 @@ script_command_action (struct t_gui_buffer *buffer, const char *action,
             snprintf (str_action, sizeof (str_action),
                       "-q %s",
                       action);
-            script_action_schedule (str_action, need_repository, 1);
+            script_action_schedule (str_action, need_repository,
+                                    error_repository, 1);
         }
         else
         {
@@ -109,7 +113,8 @@ script_command_action (struct t_gui_buffer *buffer, const char *action,
                               "-q %s %s",
                               action,
                               ptr_script->name_with_extension);
-                    script_action_schedule (str_action, need_repository, 1);
+                    script_action_schedule (str_action, need_repository,
+                                            error_repository, 1);
                 }
             }
         }
@@ -135,7 +140,7 @@ script_command_script (const void *pointer, void *data,
 
     if (argc == 1)
     {
-        script_action_schedule ("buffer", 1, 0);
+        script_action_schedule ("buffer", 1, 1, 0);
         return WEECHAT_RC_OK;
     }
 
@@ -159,13 +164,13 @@ script_command_script (const void *pointer, void *data,
             script_repo_filter_scripts ((argc > 2) ? argv_eol[2] : NULL);
         else
             script_repo_set_filter ((argc > 2) ? argv_eol[2] : NULL);
-        script_action_schedule ("buffer", 1, 0);
+        script_action_schedule ("buffer", 1, 1, 0);
         return WEECHAT_RC_OK;
     }
 
     if (weechat_strcasecmp (argv[1], "list") == 0)
     {
-        script_action_schedule (argv_eol[1], 1, 0);
+        script_action_schedule (argv_eol[1], 1, 0, 0);
         return WEECHAT_RC_OK;
     }
 
@@ -179,6 +184,7 @@ script_command_script (const void *pointer, void *data,
         script_command_action (buffer,
                                argv[1],
                                (argc > 2) ? argv_eol[2] : NULL,
+                               0,
                                0);
         return WEECHAT_RC_OK;
     }
@@ -193,13 +199,14 @@ script_command_script (const void *pointer, void *data,
         script_command_action (buffer,
                                argv[1],
                                (argc > 2) ? argv_eol[2] : NULL,
+                               1,
                                1);
         return WEECHAT_RC_OK;
     }
 
     if (weechat_strcasecmp (argv[1], "upgrade") == 0)
     {
-        script_action_schedule ("upgrade", 1, 0);
+        script_action_schedule ("upgrade", 1, 1, 0);
         return WEECHAT_RC_OK;
     }
 
@@ -289,8 +296,8 @@ script_command_init ()
 {
     weechat_hook_command (
         "script",
-        N_("WeeChat scripts manager"),
-        N_("list [-o|-i]"
+        N_("WeeChat script manager"),
+        N_("list [-o|-ol|-i|-il]"
            " || search <text>"
            " || show <script>"
            " || load|unload|reload <script> [<script>...]"
@@ -299,9 +306,14 @@ script_command_init ()
            " || upgrade"
            " || update"),
         N_("          list: list loaded scripts (all languages)\n"
-           "            -o: send list of loaded scripts to buffer\n"
+           "            -o: send list of loaded scripts to buffer "
+           "(string in English)\n"
+           "           -ol: send list of loaded scripts to buffer "
+           "(translated string)\n"
            "            -i: copy list of loaded scripts in command line (for "
-           "sending to buffer)\n"
+           "sending to buffer) (string in English)\n"
+           "           -il: copy list of loaded scripts in command line (for "
+           "sending to buffer) (translated string)\n"
            "        search: search scripts by tags, language (python, "
            "perl, ...), filename extension (py, pl, ...) or text; result is "
            "displayed on scripts buffer\n"
@@ -335,6 +347,16 @@ script_command_init ()
            "  | installed\n"
            "  popular script\n"
            "\n"
+           "In output of /script list, the possible status for each script are:\n"
+           "  * ? i a H N\n"
+           "  | | | | | |\n"
+           "  | | | | | obsolete (new version available)\n"
+           "  | | | | held\n"
+           "  | | | autoloaded\n"
+           "  | | installed\n"
+           "  | unknown script (can not be downloaded/updated)\n"
+           "  popular script\n"
+           "\n"
            "Keys on script buffer:\n"
            "  alt+i  install script\n"
            "  alt+r  remove script\n"
@@ -346,15 +368,15 @@ script_command_init ()
            "  alt+v  view script\n"
            "\n"
            "Input allowed on script buffer:\n"
-           "  i/r/l/L/u/h  action on script (same as keys above)\n"
-           "  q            close buffer\n"
-           "  $            refresh buffer\n"
-           "  s:x,y        sort buffer using keys x and y (see /help "
+           "  i/r/l/L/u/A/h/v  action on script (same as keys above)\n"
+           "  q                close buffer\n"
+           "  $                refresh buffer\n"
+           "  s:x,y            sort buffer using keys x and y (see /help "
            "script.look.sort)\n"
-           "  s:           reset sort (use default sort)\n"
-           "  word(s)      filter scripts: search word(s) in scripts "
+           "  s:               reset sort (use default sort)\n"
+           "  word(s)          filter scripts: search word(s) in scripts "
            "(description, tags, ...)\n"
-           "  *            remove filter\n"
+           "  *                remove filter\n"
            "\n"
            "Mouse actions on script buffer:\n"
            "  wheel         scroll list\n"
@@ -368,7 +390,7 @@ script_command_init ()
            "  /script hold urlserver.py\n"
            "  /script reload urlserver\n"
            "  /script upgrade"),
-        "list -o|-i"
+        "list -i|-il|-o|-ol"
         " || search %(script_tags)|%(script_languages)|%(script_extensions)"
         " || show %(script_scripts)"
         " || load %(script_files)|%*"
