@@ -48,7 +48,7 @@ WEECHAT_PLUGIN_DESCRIPTION(N_("Spell checker for input"));
 WEECHAT_PLUGIN_AUTHOR("Sébastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
-WEECHAT_PLUGIN_PRIORITY(11000);
+WEECHAT_PLUGIN_PRIORITY(12000);
 
 struct t_weechat_plugin *weechat_spell_plugin = NULL;
 
@@ -195,9 +195,9 @@ spell_warning_aspell_config ()
     char *aspell_filename, *spell_filename;
 
     aspell_filename = weechat_string_eval_path_home (
-        "%h/aspell.conf", NULL, NULL, NULL);
+        "${weechat_config_dir}/aspell.conf", NULL, NULL, NULL);
     spell_filename = weechat_string_eval_path_home (
-        "%h/" SPELL_CONFIG_NAME ".conf", NULL, NULL, NULL);
+        "${weechat_config_dir}/" SPELL_CONFIG_NAME ".conf", NULL, NULL, NULL);
 
     /* if aspell.conf is there and not spell.conf, display a warning */
     if (aspell_filename && spell_filename
