@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2018 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef WEECHAT_PLUGIN_TRIGGER_CALLBACK_H
@@ -127,6 +127,14 @@ extern int trigger_callback_config_cb  (const void *pointer, void *data,
 extern struct t_hashtable *trigger_callback_focus_cb (const void *pointer,
                                                       void *data,
                                                       struct t_hashtable *info);
+extern char *trigger_callback_info_cb (const void *pointer,
+                                       void *data,
+                                       const char *info_name,
+                                       const char *arguments);
+extern struct t_hashtable *trigger_callback_info_hashtable_cb (const void *pointer,
+                                                               void *data,
+                                                               const char *info_name,
+                                                               struct t_hashtable *hashtable);
 extern void trigger_callback_init ();
 extern void trigger_callback_end ();
 
