@@ -1,7 +1,7 @@
 /*
  * trigger-config.c - trigger configuration options (file trigger.conf)
  *
- * Copyright (C) 2014-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -54,6 +54,8 @@ char *trigger_config_default_list[][1 + TRIGGER_NUM_OPTIONS] =
      * beep on highlight/private message, on following conditions:
      *   - message is displayed (not filtered)
      *     AND:
+     *   - message does not have tag "notify_none"
+     *     AND:
      *       - message is a highlight
      *         OR:
      *       - message is a message in a private buffer
@@ -61,7 +63,8 @@ char *trigger_config_default_list[][1 + TRIGGER_NUM_OPTIONS] =
     { "beep", "on",
       "print",
       "",
-      "${tg_displayed} && (${tg_highlight} || ${tg_msg_pv})",
+      "${tg_displayed} && ${tg_tags} !!- ,notify_none, "
+      "&& (${tg_highlight} || ${tg_msg_pv})",
       "",
       "/print -beep",
       "ok",
@@ -71,7 +74,6 @@ char *trigger_config_default_list[][1 + TRIGGER_NUM_OPTIONS] =
      *   - /msg nickserv id|identify|ghost|release|regain|recover
      *   - /oper
      *   - /quote pass
-     *   - /set *password*
      *   - /secure passphrase|decrypt|set
      */
     { "cmd_pass", "on",
@@ -84,7 +86,6 @@ char *trigger_config_default_list[][1 + TRIGGER_NUM_OPTIONS] =
       "recover +[^ ]+) +)|"
       "/oper +[^ ]+ +|"
       "/quote +pass +|"
-      "/set +[^ ]*password[^ ]* +|"
       "/secure +(passphrase|decrypt|set +[^ ]+) +)"
       "(.*)"
       "==${re:1}${hide:*,${re:+}}",
