@@ -38,7 +38,7 @@ WEECHAT_PLUGIN_DESCRIPTION(N_("Text replacement and command execution on events
 WEECHAT_PLUGIN_AUTHOR("Sébastien Helleu <flashcode@flashtux.org>");
 WEECHAT_PLUGIN_VERSION(WEECHAT_VERSION);
 WEECHAT_PLUGIN_LICENSE(WEECHAT_LICENSE);
-WEECHAT_PLUGIN_PRIORITY(12000);
+WEECHAT_PLUGIN_PRIORITY(13000);
 
 struct t_weechat_plugin *weechat_trigger_plugin = NULL;
 
@@ -208,6 +208,9 @@ trigger_search_with_option (struct t_config_option *option)
     char *pos_option;
     struct t_trigger *ptr_trigger;
 
+    if (!option)
+        return NULL;
+
     ptr_name = weechat_hdata_string (weechat_hdata_get ("config_option"),
                                      option, "name");
     if (!ptr_name)
@@ -675,8 +678,6 @@ trigger_regex_split (const char *str_regex,
         length_delimiter = strlen (delimiter);
 
         ptr_regex = pos;
-        if (!ptr_regex[0])
-            goto format_error;
 
         /* search the start of replacement string */
         pos_replace = strstr (ptr_regex, delimiter);
@@ -826,8 +827,9 @@ trigger_split_command (const char *command,
 }
 
 /*
- * Checks if a trigger name is valid: it must not start with "-" and not have
- * any spaces.
+ * Checks if a trigger name is valid:
+ *   - it must not start with "-"
+ *   - it must not have any space/period.
  *
  * Returns:
  *   1: name is valid
@@ -1092,7 +1094,7 @@ trigger_rename (struct t_trigger *trigger, const char *name)
     int length, i;
     char *option_name;
 
-    if (!name || !name[0] || !trigger_name_valid (name)
+    if (!trigger || !name || !name[0] || !trigger_name_valid (name)
         || trigger_search (name))
     {
         return 0;
@@ -1144,7 +1146,7 @@ trigger_rename (struct t_trigger *trigger, const char *name)
 struct t_trigger *
 trigger_copy (struct t_trigger *trigger, const char *name)
 {
-    if (!name || !name[0] || !trigger_name_valid (name)
+    if (!trigger || !name || !name[0] || !trigger_name_valid (name)
         || trigger_search (name))
     {
         return NULL;
