@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2019 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -40,7 +40,8 @@ extern struct t_config_option *xfer_config_network_fast_send;
 extern struct t_config_option *xfer_config_network_own_ip;
 extern struct t_config_option *xfer_config_network_port_range;
 extern struct t_config_option *xfer_config_network_send_ack;
-extern struct t_config_option *xfer_config_network_speed_limit;
+extern struct t_config_option *xfer_config_network_speed_limit_recv;
+extern struct t_config_option *xfer_config_network_speed_limit_send;
 extern struct t_config_option *xfer_config_network_timeout;
 
 extern struct t_config_option *xfer_config_file_auto_accept_chats;
@@ -51,6 +52,7 @@ extern struct t_config_option *xfer_config_file_auto_resume;
 extern struct t_config_option *xfer_config_file_auto_check_crc32;
 extern struct t_config_option *xfer_config_file_convert_spaces;
 extern struct t_config_option *xfer_config_file_download_path;
+extern struct t_config_option *xfer_config_file_download_temporary_suffix;
 extern struct t_config_option *xfer_config_file_upload_path;
 extern struct t_config_option *xfer_config_file_use_nick_in_filename;
 
