@@ -1,7 +1,7 @@
 /*
  * xfer-dcc.c - file transfer via DCC protocol
  *
- * Copyright (C) 2003-2014 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2003-2021 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WeeChat.  If not, see <http://www.gnu.org/licenses/>.
+ * along with WeeChat.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <stdlib.h>
@@ -26,8 +26,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
-#include <sys/select.h>
+#include <poll.h>
 #include <netinet/in.h>
+#include <netinet/tcp.h>
 #include <fcntl.h>
 #include <time.h>
 #include <netdb.h>
@@ -48,11 +49,11 @@
 void
 xfer_dcc_send_file_child (struct t_xfer *xfer)
 {
-    int num_read, num_sent, blocksize;
+    int num_read, num_sent;
     static char buffer[XFER_BLOCKSIZE_MAX];
     uint32_t ack;
     time_t last_sent, new_time, last_second, sent_ok;
-    unsigned long long sent_last_second;
+    unsigned long long blocksize, speed_limit, sent_last_second;
 
     /* empty file? just return immediately */
     if (xfer->pos >= xfer->size)
@@ -62,15 +63,15 @@ xfer_dcc_send_file_child (struct t_xfer *xfer)
         return;
     }
 
-    blocksize = xfer->blocksize;
-    if (weechat_config_integer (xfer_config_network_speed_limit) > 0)
-    {
-        if (blocksize > weechat_config_integer (xfer_config_network_speed_limit) * 1024)
-            blocksize = weechat_config_integer (xfer_config_network_speed_limit) * 1024;
-    }
+    speed_limit = (unsigned long long)weechat_config_integer (
+        xfer_config_network_speed_limit_send);
+
+    blocksize = (unsigned long long)(xfer->blocksize);
+    if ((speed_limit > 0) && (blocksize > speed_limit * 1024))
+        blocksize = speed_limit * 1024;
 
     last_sent = time (NULL);
-    last_second = time (NULL);
+    last_second = last_sent;
     sent_ok = 0;
     sent_last_second = 0;
     while (1)
@@ -112,8 +113,7 @@ xfer_dcc_send_file_child (struct t_xfer *xfer)
         if ((xfer->pos < xfer->size) &&
              (xfer->fast_send || (xfer->pos <= xfer->ack)))
         {
-            if ((weechat_config_integer (xfer_config_network_speed_limit) > 0)
-                && (sent_last_second >= (unsigned long long)weechat_config_integer (xfer_config_network_speed_limit) * 1024))
+            if ((speed_limit > 0) && (sent_last_second >= speed_limit * 1024))
             {
                 /* we're sending too fast (according to speed limit set by user) */
                 usleep (100);
@@ -255,7 +255,7 @@ xfer_dcc_resume_hash (struct t_xfer *xfer)
 
     while (fd <= 0)
     {
-        fd = open (xfer->local_filename, O_RDONLY);
+        fd = open (xfer->temp_local_filename, O_RDONLY);
         if (fd < 0)
         {
             if (errno == EINTR)
@@ -308,15 +308,22 @@ xfer_dcc_resume_hash (struct t_xfer *xfer)
 void
 xfer_dcc_recv_file_child (struct t_xfer *xfer)
 {
-    int flags, num_read, ack_enabled, ready;
+    int flags, num_read, ready;
     static char buffer[XFER_BLOCKSIZE_MAX];
-    time_t last_sent, new_time;
-    unsigned long long pos_last_ack;
-    fd_set read_fds, write_fds, except_fds;
+    time_t last_sent, last_second, new_time;
+    unsigned long long blocksize, pos_last_ack, speed_limit, recv_last_second;
+    struct pollfd poll_fd;
     ssize_t written, total_written;
     unsigned char *bin_hash;
     char hash[9];
 
+    speed_limit = (unsigned long long)weechat_config_integer (
+        xfer_config_network_speed_limit_recv);
+
+    blocksize = sizeof (buffer);
+    if ((speed_limit > 0) && (blocksize > speed_limit * 1024))
+        blocksize = speed_limit * 1024;
+
     /* if resuming, hash the portion of the file we have */
     if ((xfer->start_resume > 0) && xfer->hash_handle)
     {
@@ -345,6 +352,10 @@ xfer_dcc_recv_file_child (struct t_xfer *xfer)
         return;
     }
 
+    /* set TCP_NODELAY to be more aggressive with acks */
+    flags = 1;
+    setsockopt (xfer->sock, IPPROTO_TCP, TCP_NODELAY, &flags, sizeof (flags));
+
     /* connection is OK, change DCC status (inform parent process) */
     xfer_network_write_pipe (xfer, XFER_STATUS_ACTIVE,
                              XFER_NO_ERROR);
@@ -356,139 +367,163 @@ xfer_dcc_recv_file_child (struct t_xfer *xfer)
     fcntl (xfer->sock, F_SETFL, flags | O_NONBLOCK);
 
     last_sent = time (NULL);
-    ack_enabled = 1;
+    last_second = last_sent;
+    recv_last_second = 0;
     pos_last_ack = 0;
 
     while (1)
     {
         /* wait until there is something to read on socket (or error) */
-        FD_ZERO (&read_fds);
-        FD_ZERO (&write_fds);
-        FD_ZERO (&except_fds);
-        FD_SET (xfer->sock, &read_fds);
-        ready = select (xfer->sock + 1, &read_fds, &write_fds, &except_fds, NULL);
-        if (ready == 0)
+        poll_fd.fd = xfer->sock;
+        poll_fd.events = POLLIN;
+        poll_fd.revents = 0;
+        ready = poll (&poll_fd, 1, -1);
+        if (ready <= 0)
         {
-            xfer_network_write_pipe (xfer, XFER_STATUS_FAILED,
-                                     XFER_ERROR_RECV_BLOCK);
-            return;
+            if ((errno == EINTR) || (errno == EAGAIN))
+                continue;
+            else
+            {
+                xfer_network_write_pipe (xfer, XFER_STATUS_FAILED,
+                                         XFER_ERROR_RECV_BLOCK);
+                return;
+            }
         }
 
         /* read maximum data on socket (until nothing is available) */
         while (1)
         {
-            num_read = recv (xfer->sock, buffer, sizeof (buffer), 0);
-            if (num_read == -1)
+            if ((speed_limit > 0) && (recv_last_second >= speed_limit * 1024))
             {
-                if ((errno != EAGAIN) && (errno != EWOULDBLOCK) && (errno != EINTR))
-                {
-                    xfer_network_write_pipe (xfer, XFER_STATUS_FAILED,
-                                             XFER_ERROR_RECV_BLOCK);
-                    return;
-                }
                 /*
-                 * no more data available on socket: exit loop, send ACK, and
-                 * wait for new data on socket
+                 * we're receiving too fast
+                 * (according to speed limit set by user)
                  */
-                break;
+                usleep (100);
             }
             else
             {
-                if ((num_read == 0) && (xfer->pos < xfer->size))
+                num_read = recv (xfer->sock, buffer, blocksize, 0);
+                if (num_read == -1)
                 {
-                    xfer_network_write_pipe (xfer, XFER_STATUS_FAILED,
-                                             XFER_ERROR_RECV_BLOCK);
-                    return;
-                }
-
-                /* bytes received, write to disk */
-                total_written = 0;
-                while (total_written < num_read)
-                {
-                    written = write (xfer->file,
-                                     buffer + total_written,
-                                     num_read - total_written);
-                    if (written < 0)
+                    if ((errno != EAGAIN) && (errno != EWOULDBLOCK) && (errno != EINTR))
                     {
-                        if (errno == EINTR)
-                            continue;
                         xfer_network_write_pipe (xfer, XFER_STATUS_FAILED,
-                                                 XFER_ERROR_WRITE_LOCAL);
+                                                 XFER_ERROR_RECV_BLOCK);
                         return;
                     }
-                    else
+                    /*
+                     * no more data available on socket: exit loop, send ACK, and
+                     * wait for new data on socket
+                     */
+                    break;
+                }
+                else
+                {
+                    if ((num_read == 0) && (xfer->pos < xfer->size))
                     {
+                        xfer_network_write_pipe (xfer, XFER_STATUS_FAILED,
+                                                 XFER_ERROR_RECV_BLOCK);
+                        return;
+                    }
+
+                    /* bytes received, write to disk */
+                    total_written = 0;
+                    while (total_written < num_read)
+                    {
+                        written = write (xfer->file,
+                                         buffer + total_written,
+                                         num_read - total_written);
+                        if (written < 0)
+                        {
+                            if (errno == EINTR)
+                                continue;
+                            xfer_network_write_pipe (xfer, XFER_STATUS_FAILED,
+                                                     XFER_ERROR_WRITE_LOCAL);
+                            return;
+                        }
+                        else
+                        {
+                            if (xfer->hash_handle)
+                            {
+                                gcry_md_write (*xfer->hash_handle,
+                                               buffer + total_written,
+                                               written);
+                            }
+                            total_written += written;
+                        }
+                    }
+
+                    xfer->pos += (unsigned long long) num_read;
+                    recv_last_second += (unsigned long long) num_read;
+
+                    /* file received OK? */
+                    if (xfer->pos >= xfer->size)
+                    {
+                        /* check hash and report result to pipe */
                         if (xfer->hash_handle)
                         {
-                            gcry_md_write (*xfer->hash_handle,
-                                           buffer + total_written,
-                                           written);
+                            gcry_md_final (*xfer->hash_handle);
+                            bin_hash = gcry_md_read (*xfer->hash_handle, 0);
+                            if (bin_hash)
+                            {
+                                snprintf (hash, sizeof (hash), "%.2X%.2X%.2X%.2X",
+                                          bin_hash[0], bin_hash[1], bin_hash[2],
+                                          bin_hash[3]);
+                                if (weechat_strcasecmp (hash,
+                                                        xfer->hash_target) == 0)
+                                {
+                                    xfer_network_write_pipe (xfer,
+                                                             XFER_STATUS_HASHED,
+                                                             XFER_NO_ERROR);
+                                }
+                                else
+                                {
+                                    xfer_network_write_pipe (xfer,
+                                                             XFER_STATUS_HASHED,
+                                                             XFER_ERROR_HASH_MISMATCH);
+                                }
+                            }
                         }
-                        total_written += written;
+
+                        fsync (xfer->file);
+
+                        /*
+                         * extra delay before sending ACK, otherwise the send of ACK
+                         * may fail
+                         */
+                        usleep (100000);
+
+                        /* send ACK to sender without checking return code (file OK) */
+                        xfer_dcc_recv_file_send_ack (xfer);
+
+                        /* set status done and return */
+                        xfer_network_write_pipe (xfer, XFER_STATUS_DONE,
+                                                 XFER_NO_ERROR);
+                        return;
                     }
-                }
 
-                xfer->pos += (unsigned long long) num_read;
-
-                /* file received OK? */
-                if (xfer->pos >= xfer->size)
-                {
-                    /* check hash and report result to pipe */
-                    if (xfer->hash_handle)
+                    /* update status of DCC (parent process) */
+                    new_time = time (NULL);
+                    if (last_sent != new_time)
                     {
-                        gcry_md_final (*xfer->hash_handle);
-                        bin_hash = gcry_md_read (*xfer->hash_handle, 0);
-                        if (bin_hash)
-                        {
-                            snprintf (hash, sizeof (hash), "%.2X%.2X%.2X%.2X",
-                                      bin_hash[0], bin_hash[1], bin_hash[2],
-                                      bin_hash[3]);
-                            if (weechat_strcasecmp (hash,
-                                                    xfer->hash_target) == 0)
-                            {
-                                xfer_network_write_pipe (xfer,
-                                                         XFER_STATUS_HASHED,
-                                                         XFER_NO_ERROR);
-                            }
-                            else
-                            {
-                                xfer_network_write_pipe (xfer,
-                                                         XFER_STATUS_HASHED,
-                                                         XFER_ERROR_HASH_MISMATCH);
-                            }
-                        }
+                        last_sent = new_time;
+                        xfer_network_write_pipe (xfer, XFER_STATUS_ACTIVE,
+                                                 XFER_NO_ERROR);
                     }
-
-                    fsync (xfer->file);
-
-                    /*
-                     * extra delay before sending ACK, otherwise the send of ACK
-                     * may fail
-                     */
-                    usleep (100000);
-
-                    /* send ACK to sender without checking return code (file OK) */
-                    xfer_dcc_recv_file_send_ack (xfer);
-
-                    /* set status done and return */
-                    xfer_network_write_pipe (xfer, XFER_STATUS_DONE,
-                                             XFER_NO_ERROR);
-                    return;
                 }
+            }
 
-                /* update status of DCC (parent process) */
-                new_time = time (NULL);
-                if (last_sent != new_time)
-                {
-                    last_sent = new_time;
-                    xfer_network_write_pipe (xfer, XFER_STATUS_ACTIVE,
-                                             XFER_NO_ERROR);
-                }
+            new_time = time (NULL);
+            if (new_time > last_second)
+            {
+                last_second = new_time;
+                recv_last_second = 0;
             }
         }
 
         /* send ACK to sender (if needed) */
-        if (ack_enabled && (xfer->pos > pos_last_ack))
+        if (xfer->send_ack && (xfer->pos > pos_last_ack))
         {
             switch (xfer_dcc_recv_file_send_ack (xfer))
             {
@@ -499,7 +534,7 @@ xfer_dcc_recv_file_child (struct t_xfer *xfer)
                     return;
                 case 1:
                     /* send error, not fatal (buffer full?): disable ACKs */
-                    ack_enabled = 0;
+                    xfer->send_ack = 0;
                     break;
                 case 2:
                     /* send OK: save position in file as last ACK sent */
