@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2017-2020 Sébastien Helleu <flashcode@flashtux.org>
+# Copyright (C) 2017-2021 Sébastien Helleu <flashcode@flashtux.org>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -71,6 +71,10 @@ def test_strings():
     check(weechat.string_format_size(1) == '1 byte')
     check(weechat.string_format_size(2097152) == '2.10 MB')
     check(weechat.string_format_size(420000000) == '420.00 MB')
+    check(weechat.string_color_code_size('') == 0)
+    check(weechat.string_color_code_size('test') == 0)
+    str_color = weechat.color('yellow,red')
+    check(weechat.string_color_code_size(str_color) == 7)
     check(weechat.string_remove_color('test', '?') == 'test')
     check(weechat.string_is_command_char('/test') == 1)
     check(weechat.string_is_command_char('test') == 0)
