@@ -86,6 +86,9 @@ TEST(CoreEval, EvalCondition)
                                 NULL, NULL);
     CHECK(extra_vars);
     hashtable_set (extra_vars, "test", "value");
+    hashtable_set (extra_vars, "var_abc", "abc");
+    hashtable_set (extra_vars, "var_cond_true", "a==a");
+    hashtable_set (extra_vars, "var_cond_false", "a==b");
 
     options = hashtable_new (32,
                              WEECHAT_HASHTABLE_STRING,
@@ -175,6 +178,8 @@ TEST(CoreEval, EvalCondition)
     WEE_CHECK_EVAL("0", "${if:${buffer.number}==2?yes:no} == yes");
     WEE_CHECK_EVAL("0", "yes == ${if:${buffer.number}==2?yes:no}");
     WEE_CHECK_EVAL("0", "${if:\\$==A?yes:}");
+    WEE_CHECK_EVAL("0", "${var_does_not_exist}");
+    WEE_CHECK_EVAL("0", "${var_abc} == def");
 
     /* conditions evaluated as true */
     WEE_CHECK_EVAL("1", "1");
@@ -262,6 +267,14 @@ TEST(CoreEval, EvalCondition)
     WEE_CHECK_EVAL("1", "${if:${buffer.number}==1?yes:no} == yes");
     WEE_CHECK_EVAL("1", "yes == ${if:${buffer.number}==1?yes:no}");
     WEE_CHECK_EVAL("1", "${if:\\$==\\$?yes:}");
+    WEE_CHECK_EVAL("1", "${var_abc}");
+    WEE_CHECK_EVAL("1", "${var_abc} == abc");
+    WEE_CHECK_EVAL("1", "${var_cond_true}");
+    WEE_CHECK_EVAL("0", "${var_cond_true} == zzz");
+    WEE_CHECK_EVAL("1", "${var_cond_true} != zzz");
+    WEE_CHECK_EVAL("1", "${var_cond_false}");
+    WEE_CHECK_EVAL("0", "${var_cond_false} == zzz");
+    WEE_CHECK_EVAL("1", "${var_cond_false} != zzz");
 
     /* evaluation of extra_vars */
     hashtable_set (options, "extra", "eval");
@@ -440,10 +453,14 @@ TEST(CoreEval, EvalExpression)
 {
     struct t_hashtable *pointers, *extra_vars, *options;
     struct t_config_option *ptr_option;
-    char *value, str_value[256];
+    char *value, str_value[256], str_expr[256];
     const char *ptr_debug_output;
 
-    pointers = NULL;
+    pointers = hashtable_new (32,
+                              WEECHAT_HASHTABLE_STRING,
+                              WEECHAT_HASHTABLE_POINTER,
+                              NULL, NULL);
+    CHECK(pointers);
 
     extra_vars = hashtable_new (32,
                                 WEECHAT_HASHTABLE_STRING,
@@ -451,8 +468,15 @@ TEST(CoreEval, EvalExpression)
                                 NULL, NULL);
     CHECK(extra_vars);
     hashtable_set (extra_vars, "test", "value");
+    hashtable_set (extra_vars, "var_abc", "abc");
+    hashtable_set (extra_vars, "var_cond_true", "a==a");
+    hashtable_set (extra_vars, "var_cond_false", "a==b");
 
-    options = NULL;
+    options = hashtable_new (32,
+                             WEECHAT_HASHTABLE_STRING,
+                             WEECHAT_HASHTABLE_STRING,
+                             NULL, NULL);
+    CHECK(options);
 
     POINTERS_EQUAL(NULL, eval_expression (NULL, NULL, NULL, NULL));
 
@@ -478,6 +502,12 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("0", "${eval_cond:}");
     WEE_CHECK_EVAL("0", "${eval_cond:${buffer.number} == 2}");
     WEE_CHECK_EVAL("1", "${eval_cond:${buffer.number} == 1}");
+    WEE_CHECK_EVAL("1", "${eval_cond:${var_abc}}");
+    WEE_CHECK_EVAL("1", "${eval_cond:${var_abc} == abc}");
+    WEE_CHECK_EVAL("0", "${eval_cond:${var_abc} == def}");
+    WEE_CHECK_EVAL("1", "${eval_cond:${var_cond_true}}");
+    WEE_CHECK_EVAL("1", "${eval_cond:${var_cond_true}}");
+    WEE_CHECK_EVAL("0", "${eval_cond:${var_cond_false}}");
 
     /* test value from extra_vars */
     WEE_CHECK_EVAL("value", "${test}");
@@ -618,6 +648,85 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("14", "${lengthscr:こんにちは世界}");
     WEE_CHECK_EVAL("14", "${lengthscr:${color:green}こんにちは世界}");
 
+    /* test split of string */
+    WEE_CHECK_EVAL("", "${split:}");
+    WEE_CHECK_EVAL("", "${split:1}");
+    WEE_CHECK_EVAL("", "${split:1,}");
+    WEE_CHECK_EVAL("", "${split:1,,}");
+    WEE_CHECK_EVAL("", "${split:count}");
+    WEE_CHECK_EVAL("", "${split:count,}");
+    WEE_CHECK_EVAL("", "${split:count,,}");
+    WEE_CHECK_EVAL("0", "${split:count,,,}");
+
+    WEE_CHECK_EVAL("abc", "${split:random,,,abc}");
+
+    WEE_CHECK_EVAL("", "${split:0,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("abc", "${split:1,,,abc}");
+    WEE_CHECK_EVAL("abc", "${split:1,,,abc,def}");
+    WEE_CHECK_EVAL("abc", "${split:1,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("def", "${split:2,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("", "${split:4,,,abc,def,ghi}");
+
+    WEE_CHECK_EVAL("ghi", "${split:-1,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("def", "${split:-2,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("abc", "${split:-3,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("", "${split:-4,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("3", "${split:count,,,abc,def,ghi}");
+
+    WEE_CHECK_EVAL("abc", "${split:1, ,,abc def ghi}");
+    WEE_CHECK_EVAL("def", "${split:2, ,,abc def ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3, ,,abc def ghi}");
+    WEE_CHECK_EVAL("", "${split:4, ,,abc def ghi}");
+    WEE_CHECK_EVAL("3", "${split:count, ,,abc def ghi}");
+
+    WEE_CHECK_EVAL("abc", "${split:1,_-,,abc-def_ghi}");
+    WEE_CHECK_EVAL("def", "${split:2,_-,,abc-def_ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,_-,,abc-def_ghi}");
+    WEE_CHECK_EVAL("", "${split:4,_-,,abc-def_ghi}");
+    WEE_CHECK_EVAL("3", "${split:count,_-,,abc-def_ghi}");
+
+    WEE_CHECK_EVAL("abc,def,ghi", "${split:1,,keep_eol,abc,def,ghi}");
+    WEE_CHECK_EVAL("def,ghi", "${split:2,,keep_eol,abc,def,ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,keep_eol,abc,def,ghi}");
+
+    WEE_CHECK_EVAL("abc,def,ghi", "${split:1,,keep_eol+strip_left,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("def,ghi", "${split:2,,keep_eol+strip_left,,,abc,def,ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,keep_eol+strip_left,,,abc,def,ghi}");
+
+    WEE_CHECK_EVAL("abc,def,ghi", "${split:1,,keep_eol+strip_left+strip_right,,,abc,def,ghi,,}");
+    WEE_CHECK_EVAL("def,ghi", "${split:2,,keep_eol+strip_left+strip_right,,,abc,def,ghi,,}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,keep_eol+strip_left+strip_right,,,abc,def,ghi,,}");
+
+    WEE_CHECK_EVAL("abc", "${split:1,,collapse_seps,abc,,,def,,ghi}");
+    WEE_CHECK_EVAL("def", "${split:2,,collapse_seps,abc,,,def,,ghi}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,collapse_seps,abc,,,def,,ghi}");
+
+    WEE_CHECK_EVAL("abc", "${split:1,,strip_items=_,_abc_,_def_,_ghi_}");
+    WEE_CHECK_EVAL("def", "${split:2,,strip_items=_,_abc_,_def_,_ghi_}");
+    WEE_CHECK_EVAL("ghi", "${split:3,,strip_items=_,_abc_,_def_,_ghi_}");
+
+    WEE_CHECK_EVAL("def", "${split:2,,max_items=0,abc,def,ghi}");
+    WEE_CHECK_EVAL("def", "${split:2,,max_items=2,abc,def,ghi}");
+    WEE_CHECK_EVAL("", "${split:2,,max_items=1,abc,def,ghi}");
+
+    /* test split of shell arguments */
+    WEE_CHECK_EVAL("", "${split_shell:}");
+    WEE_CHECK_EVAL("", "${split_shell:1}");
+    WEE_CHECK_EVAL("", "${split_shell:1,}");
+    WEE_CHECK_EVAL("", "${split_shell:count}");
+    WEE_CHECK_EVAL("0", "${split_shell:count,}");
+
+    WEE_CHECK_EVAL("first arg", "${split_shell:random,\"first arg\"}");
+
+    WEE_CHECK_EVAL("", "${split_shell:0,\"first arg\" arg2}");
+    WEE_CHECK_EVAL("first arg", "${split_shell:1,\"first arg\" arg2}");
+    WEE_CHECK_EVAL("arg2", "${split_shell:2,\"first arg\" arg2}");
+
+    WEE_CHECK_EVAL("arg2", "${split_shell:-1,\"first arg\" arg2}");
+    WEE_CHECK_EVAL("first arg", "${split_shell:-2,\"first arg\" arg2}");
+    WEE_CHECK_EVAL("", "${split_shell:-3,\"first arg\" arg2}");
+
     /* test color */
     WEE_CHECK_EVAL(gui_color_get_custom ("green"), "${color:green}");
     WEE_CHECK_EVAL(gui_color_get_custom ("*214"), "${color:*214}");
@@ -647,7 +756,7 @@ TEST(CoreEval, EvalExpression)
                    "${modifier:color_decode_ansi,0,test_\x1B[92mno_color}");
     snprintf (str_value, sizeof (str_value),
               "test_%slightgreen",
-              gui_color_get_custom ("lightgreen"));
+              gui_color_get_custom ("|lightgreen"));
     WEE_CHECK_EVAL(str_value,
                    "${modifier:color_decode_ansi,1,test_\x1B[92mlightgreen}");
     snprintf (str_value, sizeof (str_value),
@@ -694,13 +803,24 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("yes-no", "${if:5>2?${if:1>7?yes-yes:yes-no}:${if:9>4?no-yes:no-no}}");
     WEE_CHECK_EVAL("no-yes", "${if:1>7?${if:6>3?yes-yes:yes-no}:${if:9>4?no-yes:no-no}}");
     WEE_CHECK_EVAL("no-no", "${if:1>7?${if:1>7?yes-yes:yes-no}:${if:1>7?no-yes:no-no}}");
+    WEE_CHECK_EVAL("0", "${if:}");
     WEE_CHECK_EVAL("0", "${if:0}");
     WEE_CHECK_EVAL("1", "${if:1}");
+    WEE_CHECK_EVAL("1", "${if:abc}");
     WEE_CHECK_EVAL("0", "${if:abc!=abc}");
     WEE_CHECK_EVAL("1", "${if:abc==abc}");
     WEE_CHECK_EVAL("1", "${if:${if:abc==abc}}");
     WEE_CHECK_EVAL("0", "${if:${rev:${if:42==42?hello:bye}}==eyb}");
     WEE_CHECK_EVAL("1", "${if:${rev:${if:42==42?hello:bye}}==olleh}");
+    WEE_CHECK_EVAL("1", "${if:${var_abc}}");
+    WEE_CHECK_EVAL("1", "${if:${var_abc} == abc}");
+    WEE_CHECK_EVAL("0", "${if:${var_abc} == def}");
+    WEE_CHECK_EVAL("1", "${if:${var_cond_true}}");
+    WEE_CHECK_EVAL("0", "${if:${var_cond_true} == zzz}");
+    WEE_CHECK_EVAL("1", "${if:${var_cond_true} != zzz}");
+    WEE_CHECK_EVAL("1", "${if:${var_cond_false}}");
+    WEE_CHECK_EVAL("0", "${if:${var_cond_false} == zzz}");
+    WEE_CHECK_EVAL("1", "${if:${var_cond_false} != zzz}");
 
     /* test calc */
     WEE_CHECK_EVAL("0", "${calc:}");
@@ -710,6 +830,29 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("18", "${calc:(5+1)*3}");
     WEE_CHECK_EVAL("123129", "${calc:${repeat:2,123}+2*3}");
 
+    /* test random */
+    WEE_CHECK_EVAL("0", "${random:}");
+    WEE_CHECK_EVAL("0", "${random:1}");
+    WEE_CHECK_EVAL("0", "${random:1,}");
+    WEE_CHECK_EVAL("0", "${random:5,1}");
+    WEE_CHECK_EVAL("2", "${random:2,2}");
+    snprintf (str_expr, sizeof (str_expr),
+              "${random:-1,%ld}",
+              (long)RAND_MAX);
+    WEE_CHECK_EVAL("0", str_expr);
+    snprintf (str_expr, sizeof (str_expr),
+              "${random:%ld,%ld}",
+              (long)RAND_MAX,
+              (long)RAND_MAX);
+    snprintf (str_value, sizeof (str_value),
+              "%ld",
+              (long)RAND_MAX);
+    WEE_CHECK_EVAL(str_value, str_expr);
+
+    /* test translation */
+    WEE_CHECK_EVAL("", "${translate:}");
+    WEE_CHECK_EVAL("abcdef", "${translate:abcdef}");
+
     /* test option */
     hashtable_set (secure_hashtable_data, "sec_option", "sec_value");
     WEE_CHECK_EVAL("sec_value", "${sec.data.sec_option}");
@@ -728,6 +871,9 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("weechat", "${name}");
 
     /* test hdata */
+    hashtable_set (pointers, "my_null_pointer", (const void *)0x0);
+    hashtable_set (pointers, "my_buffer_pointer", gui_buffers);
+    hashtable_set (pointers, "my_other_pointer", (const void *)0x1234abcd);
     WEE_CHECK_EVAL("x", "x${buffer.number");
     WEE_CHECK_EVAL("x${buffer.number}1",
                    "x\\${buffer.number}${buffer.number}");
@@ -735,7 +881,20 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("1", "${window.buffer.number}");
     WEE_CHECK_EVAL("core.weechat", "${buffer.full_name}");
     WEE_CHECK_EVAL("core.weechat", "${window.buffer.full_name}");
+    WEE_CHECK_EVAL("", "${buffer[].full_name}");
     WEE_CHECK_EVAL("", "${buffer[0x0].full_name}");
+    WEE_CHECK_EVAL("", "${buffer[0x1].full_name}");
+    WEE_CHECK_EVAL("", "${buffer[unknown_list].full_name}");
+    WEE_CHECK_EVAL("", "${unknown_pointer}");
+    WEE_CHECK_EVAL("", "${my_null_pointer}");
+    snprintf (str_value, sizeof (str_value),
+              "0x%lx", (long unsigned int)gui_buffers);
+    WEE_CHECK_EVAL(str_value, "${my_buffer_pointer}");
+    WEE_CHECK_EVAL("0x1234abcd", "${my_other_pointer}");
+    WEE_CHECK_EVAL("", "${buffer[unknown_pointer].full_name}");
+    WEE_CHECK_EVAL("", "${buffer[my_null_pointer].full_name}");
+    WEE_CHECK_EVAL("core.weechat", "${buffer[my_buffer_pointer].full_name}");
+    WEE_CHECK_EVAL("", "${buffer[my_other_pointer].full_name}");
     WEE_CHECK_EVAL("core.weechat", "${buffer[gui_buffers].full_name}");
     snprintf (str_value, sizeof (str_value),
               "${buffer[0x%lx].full_name}", (long unsigned int)gui_buffers);
@@ -754,13 +913,9 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL(str_value, "${window.buffer.local_variables}");
     WEE_CHECK_EVAL("core", "${window.buffer.local_variables.plugin}");
     WEE_CHECK_EVAL("weechat", "${window.buffer.local_variables.name}");
+    hashtable_remove_all (pointers);
 
     /* test with another prefix/suffix */
-    options = hashtable_new (32,
-                             WEECHAT_HASHTABLE_STRING,
-                             WEECHAT_HASHTABLE_STRING,
-                             NULL, NULL);
-    CHECK(options);
     hashtable_set (options, "prefix", "<<<");
     WEE_CHECK_EVAL("${info:version}", "${info:version}");
     WEE_CHECK_EVAL("<info:version}", "<info:version}");
@@ -773,14 +928,9 @@ TEST(CoreEval, EvalExpression)
     WEE_CHECK_EVAL("<<info:version>>", "<<info:version>>");
     WEE_CHECK_EVAL(version_get_version (), "<<<info:version>>>");
     WEE_CHECK_EVAL("1", "<<<buffer.number>>>");
-    hashtable_free (options);
+    hashtable_remove_all (options);
 
     /* test with debug level 1 */
-    options = hashtable_new (32,
-                             WEECHAT_HASHTABLE_STRING,
-                             WEECHAT_HASHTABLE_STRING,
-                             NULL, NULL);
-    CHECK(options);
     hashtable_set (options, "debug", "1");
     WEE_CHECK_EVAL("fedcba", "${rev:abcdef}");
     ptr_debug_output = (const char *)hashtable_get (options, "debug_output");
@@ -791,14 +941,9 @@ TEST(CoreEval, EvalExpression)
                  "  2:== \"fedcba\"\n"
                  "1:== \"fedcba\"",
                  ptr_debug_output);
-    hashtable_free (options);
+    hashtable_remove_all (options);
 
     /* test with debug level 2 */
-    options = hashtable_new (32,
-                             WEECHAT_HASHTABLE_STRING,
-                             WEECHAT_HASHTABLE_STRING,
-                             NULL, NULL);
-    CHECK(options);
     hashtable_set (options, "debug", "2");
     WEE_CHECK_EVAL("fedcba", "${rev:abcdef}");
     ptr_debug_output = (const char *)hashtable_get (options, "debug_output");
@@ -809,9 +954,11 @@ TEST(CoreEval, EvalExpression)
                  "  2:== \"fedcba\"\n"
                  "1:== \"fedcba\"",
                  ptr_debug_output);
-    hashtable_free (options);
+    hashtable_remove_all (options);
 
+    hashtable_free (pointers);
     hashtable_free (extra_vars);
+    hashtable_free (options);
 }
 
 /*
@@ -921,6 +1068,18 @@ TEST(CoreEval, EvalReplaceRegex)
     hashtable_set (options, "regex_replace", "${re:z}");
     WEE_CHECK_EVAL("", "abc");
 
+    /* use replace index: add number before each item */
+    hashtable_remove (pointers, "regex");
+    hashtable_set (options, "regex", "[^,]+");
+    hashtable_set (options, "regex_replace", "${re:repl_index}.${re:0}");
+    WEE_CHECK_EVAL("1.item1,2.item2,3.item3", "item1,item2,item3");
+
+    /* use replace index: replace each letter by its position */
+    hashtable_remove (pointers, "regex");
+    hashtable_set (options, "regex", ".");
+    hashtable_set (options, "regex_replace", "${re:repl_index}");
+    WEE_CHECK_EVAL("1234", "test");
+
     hashtable_free (pointers);
     hashtable_free (extra_vars);
     hashtable_free (options);
