@@ -571,6 +571,18 @@ TEST(CoreString, Match)
     LONGS_EQUAL(1, string_match ("aabaa", "aa*", 1));
     LONGS_EQUAL(1, string_match ("aabaabaabaa", "aa*", 0));
     LONGS_EQUAL(1, string_match ("aabaabaabaa", "aa*", 1));
+    LONGS_EQUAL(1, string_match ("script.color.description", "*script.color*", 0));
+    LONGS_EQUAL(1, string_match ("script.color.description", "*script.color*", 1));
+    LONGS_EQUAL(1, string_match ("script.color.description", "*script.COLOR*", 0));
+    LONGS_EQUAL(0, string_match ("script.color.description", "*script.COLOR*", 1));
+    LONGS_EQUAL(1, string_match ("script.color.description", "*script*color*", 0));
+    LONGS_EQUAL(1, string_match ("script.color.description", "*script*color*", 1));
+    LONGS_EQUAL(1, string_match ("script.color.description", "*script*COLOR*", 0));
+    LONGS_EQUAL(0, string_match ("script.color.description", "*script*COLOR*", 1));
+    LONGS_EQUAL(1, string_match ("script.script.script", "scr*scr*scr*", 0));
+    LONGS_EQUAL(1, string_match ("script.script.script", "SCR*SCR*SCR*", 0));
+    LONGS_EQUAL(0, string_match ("script.script.script", "SCR*SCR*SCR*", 1));
+    LONGS_EQUAL(0, string_match ("script.script.script", "scr*scr*scr*scr*", 0));
 }
 
 /*
@@ -664,13 +676,17 @@ TEST(CoreString, ExpandHome)
 TEST(CoreString, EvalPathHome)
 {
     char *home, *result;
-    int length_home, length_weechat_home;
-    struct t_hashtable *extra_vars;
+    int length_home, length_weechat_config_dir, length_weechat_data_dir;
+    int length_weechat_cache_dir, length_weechat_runtime_dir;
+    struct t_hashtable *extra_vars, *options;
 
     home = getenv ("HOME");
     length_home = strlen (home);
 
-    length_weechat_home = strlen (weechat_home);
+    length_weechat_config_dir = strlen (weechat_config_dir);
+    length_weechat_data_dir = strlen (weechat_data_dir);
+    length_weechat_cache_dir = strlen (weechat_cache_dir);
+    length_weechat_runtime_dir = strlen (weechat_runtime_dir);
 
     POINTERS_EQUAL(NULL, string_eval_path_home (NULL, NULL, NULL, NULL));
 
@@ -684,19 +700,91 @@ TEST(CoreString, EvalPathHome)
     STRCMP_EQUAL(result + length_home, "/test");
     free (result);
 
+    /* "%h" is weechat_data_dir by default */
     result = string_eval_path_home ("%h/test", NULL, NULL, NULL);
-    CHECK(strncmp (result, weechat_home, length_weechat_home) == 0);
-    LONGS_EQUAL(length_weechat_home + 5, strlen (result));
-    STRCMP_EQUAL(result + length_weechat_home, "/test");
+    CHECK(strncmp (result, weechat_data_dir, length_weechat_data_dir) == 0);
+    LONGS_EQUAL(length_weechat_data_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_data_dir, "/test");
+    free (result);
+
+    options = hashtable_new (32,
+                             WEECHAT_HASHTABLE_STRING,
+                             WEECHAT_HASHTABLE_STRING,
+                             NULL, NULL);
+
+    /* "%h" with forced config dir */
+    hashtable_set (options, "directory", "config");
+    result = string_eval_path_home ("%h/test", NULL, NULL, options);
+    CHECK(strncmp (result, weechat_config_dir, length_weechat_config_dir) == 0);
+    LONGS_EQUAL(length_weechat_config_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_config_dir, "/test");
+    free (result);
+
+    /* "%h" with forced data dir */
+    hashtable_set (options, "directory", "data");
+    result = string_eval_path_home ("%h/test", NULL, NULL, options);
+    CHECK(strncmp (result, weechat_data_dir, length_weechat_data_dir) == 0);
+    LONGS_EQUAL(length_weechat_data_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_data_dir, "/test");
+    free (result);
+
+    /* "%h" with forced cache dir */
+    hashtable_set (options, "directory", "cache");
+    result = string_eval_path_home ("%h/test", NULL, NULL, options);
+    CHECK(strncmp (result, weechat_cache_dir, length_weechat_cache_dir) == 0);
+    LONGS_EQUAL(length_weechat_cache_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_cache_dir, "/test");
+    free (result);
+
+    /* "%h" with forced runtime dir */
+    hashtable_set (options, "directory", "runtime");
+    result = string_eval_path_home ("%h/test", NULL, NULL, options);
+    CHECK(strncmp (result, weechat_runtime_dir, length_weechat_runtime_dir) == 0);
+    LONGS_EQUAL(length_weechat_runtime_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_runtime_dir, "/test");
+    free (result);
+
+    hashtable_free (options);
+
+    /* config dir */
+    result = string_eval_path_home ("${weechat_config_dir}/path",
+                                    NULL, NULL, NULL);
+    CHECK(strncmp (result, weechat_config_dir, length_weechat_config_dir) == 0);
+    LONGS_EQUAL(length_weechat_config_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_config_dir, "/path");
+    free (result);
+
+    /* data dir */
+    result = string_eval_path_home ("${weechat_data_dir}/path",
+                                    NULL, NULL, NULL);
+    CHECK(strncmp (result, weechat_data_dir, length_weechat_data_dir) == 0);
+    LONGS_EQUAL(length_weechat_data_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_data_dir, "/path");
+    free (result);
+
+    /* cache dir */
+    result = string_eval_path_home ("${weechat_cache_dir}/path",
+                                    NULL, NULL, NULL);
+    CHECK(strncmp (result, weechat_cache_dir, length_weechat_cache_dir) == 0);
+    LONGS_EQUAL(length_weechat_cache_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_cache_dir, "/path");
+    free (result);
+
+    /* runtime dir */
+    result = string_eval_path_home ("${weechat_runtime_dir}/path",
+                                    NULL, NULL, NULL);
+    CHECK(strncmp (result, weechat_runtime_dir, length_weechat_runtime_dir) == 0);
+    LONGS_EQUAL(length_weechat_runtime_dir + 5, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_runtime_dir, "/path");
     free (result);
 
     setenv ("WEECHAT_TEST_PATH", "path1", 1);
 
     result = string_eval_path_home ("%h/${env:WEECHAT_TEST_PATH}/path2",
                                     NULL, NULL, NULL);
-    CHECK(strncmp (result, weechat_home, length_weechat_home) == 0);
-    LONGS_EQUAL(length_weechat_home + 12, strlen (result));
-    STRCMP_EQUAL(result + length_weechat_home, "/path1/path2");
+    CHECK(strncmp (result, weechat_data_dir, length_weechat_data_dir) == 0);
+    LONGS_EQUAL(length_weechat_data_dir + 12, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_data_dir, "/path1/path2");
     free (result);
 
     extra_vars = hashtable_new (32,
@@ -708,9 +796,9 @@ TEST(CoreString, EvalPathHome)
 
     result = string_eval_path_home ("%h/${env:WEECHAT_TEST_PATH}/${path2}",
                                     NULL, extra_vars, NULL);
-    CHECK(strncmp (result, weechat_home, length_weechat_home) == 0);
-    LONGS_EQUAL(length_weechat_home + 12, strlen (result));
-    STRCMP_EQUAL(result + length_weechat_home, "/path1/value");
+    CHECK(strncmp (result, weechat_data_dir, length_weechat_data_dir) == 0);
+    LONGS_EQUAL(length_weechat_data_dir + 12, strlen (result));
+    STRCMP_EQUAL(result + length_weechat_data_dir, "/path1/value");
     free (result);
 
     hashtable_free (extra_vars);
@@ -1835,6 +1923,9 @@ TEST(CoreString, Base64)
                                                   str));
         STRCMP_EQUAL(str_base64[i][1], str);
     }
+    /* test with a \0 in string */
+    LONGS_EQUAL(20, string_base64_encode ("This is\0a test.", 15, str));
+    STRCMP_EQUAL("VGhpcyBpcwBhIHRlc3Qu", str);
 
     /* string_base64_decode */
     LONGS_EQUAL(-1, string_base64_decode (NULL, NULL));
@@ -1847,6 +1938,10 @@ TEST(CoreString, Base64)
         LONGS_EQUAL(length, string_base64_decode (str_base64[i][1], str));
         STRCMP_EQUAL(str_base64[i][0], str);
     }
+    /* test with a \0 in string */
+    LONGS_EQUAL(15, string_base64_decode ("VGhpcyBpcwBhIHRlc3Qu", str));
+    MEMCMP_EQUAL("This is\0a test.", str, 15);
+
     /* invalid base64 string, missing two "=" at the end */
     LONGS_EQUAL(4, string_base64_decode ("dGVzdA", str));
     STRCMP_EQUAL("test", str);
@@ -1998,13 +2093,12 @@ TEST(CoreString, InputForBuffer)
     char *str;
 
     POINTERS_EQUAL(NULL, string_input_for_buffer (NULL));
+    POINTERS_EQUAL(NULL, string_input_for_buffer ("/"));
     POINTERS_EQUAL(NULL, string_input_for_buffer ("/abc"));
+
     str = strdup ("");
     STRCMP_EQUAL(str, string_input_for_buffer (str));
     free (str);
-    str = strdup ("/");
-    STRCMP_EQUAL(str, string_input_for_buffer (str));
-    free (str);
     str = strdup ("/ ");
     STRCMP_EQUAL(str, string_input_for_buffer (str));
     free (str);
